@@ -1,14 +1,11 @@
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-
-#include "jpeg.h"
-
 #include <liboil/liboil.h>
 #include <liboil/liboildebug.h>
+#include <cogcompat.h>
+
+#include "jpeg.h"
+
 
-#define CLAMP(x,a,b) ((x)<(a) ? (a) : ((x)>(b) ? (b) : (x)))
 #define oil_argb(a,r,g,b) \
   ((oil_clamp_255(a)<<24) | \
    (oil_clamp_255(r)<<16) | \
@@ -47,7 +44,7 @@ static void scanlinescale2_u8 (unsigned char *dest, unsigned char *src,
 
 
 int jpeg_decode_argb (uint8_t *data, int length, uint32_t **image,
-    int *width, int *height)
+    unsigned int *width, unsigned int *height)
 {
   JpegDecoder *dec;
   int ret;
@@ -57,7 +54,10 @@ int jpeg_decode_argb (uint8_t *data, int length, uint32_t **image,
   jpeg_decoder_addbits (dec, data, length);
   ret = jpeg_decoder_decode(dec);
 
-  if (!ret) return FALSE;
+  if (!ret) {
+    jpeg_decoder_free (dec);
+    return FALSE;
+  }
 
   jpeg_decoder_get_image_size (dec, width, height);
   *image = (uint32_t *)jpeg_decoder_get_argb_image (dec);
