@@ -1,7 +1,7 @@
 /* Swfdec
  * Copyright (C) 2003-2006 David Schleef <ds@schleef.org>
  *		 2005-2006 Eric Anholt <eric@anholt.net>
- *		 2006-2007 Benjamin Otte <otte@gnome.org>
+ *		 2006-2008 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,6 +29,7 @@
 #include <swfdec/swfdec_as_function.h>
 #include <swfdec/swfdec_as_native_function.h>
 #include <swfdec/swfdec_as_object.h>
+#include <swfdec/swfdec_as_relay.h>
 #include <swfdec/swfdec_as_types.h>
 #include <swfdec/swfdec_script.h>
 
@@ -46,5 +47,6 @@
 #include <swfdec/swfdec_stream.h>
 #include <swfdec/swfdec_system.h>
 #include <swfdec/swfdec_url.h>
+#include <swfdec/swfdec_version.h>
 
 #endif
