@@ -22,11 +22,13 @@
 #endif
 
 #include "swfdec_actor.h"
+#include "swfdec_as_internal.h"
 #include "swfdec_as_strings.h"
 #include "swfdec_debug.h"
 #include "swfdec_button_movie.h"
 #include "swfdec_player_internal.h"
 #include "swfdec_resource.h"
+#include "swfdec_sandbox.h"
 #include "swfdec_sprite_movie.h"
 
 
@@ -66,7 +68,7 @@ swfdec_actor_mouse_events (SwfdecActor *actor)
   if (actor->events && swfdec_event_list_has_mouse_events (actor->events))
     return TRUE;
   /* otherwise, require at least one of the custom script handlers */
-  object = SWFDEC_AS_OBJECT (actor);
+  object = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (actor));
   if (swfdec_as_object_has_variable (object, SWFDEC_AS_STR_onRollOver) ||
       swfdec_as_object_has_variable (object, SWFDEC_AS_STR_onRollOut) ||
       swfdec_as_object_has_variable (object, SWFDEC_AS_STR_onDragOver) ||
@@ -81,7 +83,7 @@ swfdec_actor_mouse_events (SwfdecActor *actor)
 static void
 swfdec_actor_mouse_in (SwfdecActor *actor)
 {
-  if (swfdec_player_is_mouse_pressed (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (actor)->context)))
+  if (swfdec_player_is_mouse_pressed (SWFDEC_PLAYER (swfdec_gc_object_get_context (actor))))
     swfdec_actor_queue_script (actor, SWFDEC_EVENT_DRAG_OVER);
   else
     swfdec_actor_queue_script (actor, SWFDEC_EVENT_ROLL_OVER);
@@ -90,7 +92,7 @@ swfdec_actor_mouse_in (SwfdecActor *actor)
 static void
 swfdec_actor_mouse_out (SwfdecActor *actor)
 {
-  if (swfdec_player_is_mouse_pressed (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (actor)->context)))
+  if (swfdec_player_is_mouse_pressed (SWFDEC_PLAYER (swfdec_gc_object_get_context (actor))))
     swfdec_actor_queue_script (actor, SWFDEC_EVENT_DRAG_OUT);
   else
     swfdec_actor_queue_script (actor, SWFDEC_EVENT_ROLL_OUT);
@@ -112,7 +114,7 @@ swfdec_actor_mouse_release (SwfdecActor *actor, guint button)
   if (button != 0)
     return;
 
-  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (actor)->context);
+  player = SWFDEC_PLAYER (swfdec_gc_object_get_context (actor));
   if (player->priv->mouse_below == actor)
     swfdec_actor_queue_script (actor, SWFDEC_EVENT_RELEASE);
   else
@@ -137,11 +139,32 @@ swfdec_actor_key_release (SwfdecActor *actor, guint keycode, guint character)
   swfdec_actor_queue_script (actor, SWFDEC_EVENT_KEY_UP);
 }
 
+static GObject *
+swfdec_actor_constructor (GType type, guint n_construct_properties,
+    GObjectConstructParam *construct_properties)
+{
+  SwfdecPlayerPrivate *priv;
+  GObject *object;
+
+  object = G_OBJECT_CLASS (swfdec_actor_parent_class)->constructor (type, 
+      n_construct_properties, construct_properties);
+
+  priv = SWFDEC_PLAYER (swfdec_gc_object_get_context (object))->priv;
+  /* NB: adding to the movies list happens before swfdec_movie_initialize().
+   * swfdec_movie_initialize() does a gotoAndPlay(0) for Sprites which can
+   * cause new movies to be created (and added to this list).
+   */
+  priv->actors = g_list_prepend (priv->actors, object);
+
+  return object;
+}
+
 static void
 swfdec_actor_class_init (SwfdecActorClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->constructor = swfdec_actor_constructor;
   object_class->dispose = swfdec_actor_dispose;
 
   klass->iterate_end = swfdec_actor_iterate_end;
@@ -161,12 +184,13 @@ swfdec_actor_init (SwfdecActor *actor)
   swfdec_sound_matrix_init_identity (&actor->sound_matrix);
 }
 
-static void
+static gboolean
 swfdec_sprite_movie_set_constructor (SwfdecSpriteMovie *movie)
 {
   SwfdecMovie *mov = SWFDEC_MOVIE (movie);
-  SwfdecAsContext *context = SWFDEC_AS_OBJECT (movie)->context;
+  SwfdecAsContext *context = swfdec_gc_object_get_context (movie);
   SwfdecAsObject *constructor = NULL;
+  SwfdecAsObject *object = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (movie));
 
   g_assert (mov->resource != NULL);
 
@@ -181,19 +205,26 @@ swfdec_sprite_movie_set_constructor (SwfdecSpriteMovie *movie)
 	  name);
     }
   }
-  if (constructor == NULL)
-    constructor = mov->resource->sandbox->MovieClip;
-
-  swfdec_as_object_set_constructor (SWFDEC_AS_OBJECT (movie), constructor);
+  if (constructor == NULL) {
+    swfdec_sandbox_use (SWFDEC_MOVIE (movie)->resource->sandbox);
+    swfdec_as_object_set_constructor_by_name (object,
+	SWFDEC_AS_STR_MovieClip, NULL);
+    swfdec_sandbox_unuse (SWFDEC_MOVIE (movie)->resource->sandbox);
+    return FALSE;
+  } else {
+    swfdec_as_object_set_constructor (object, constructor);
+    return TRUE;
+  }
 }
 
 void
 swfdec_actor_execute (SwfdecActor *actor, SwfdecEventType condition,
     guint8 key)
 {
-  SwfdecAsObject *thisp;
+  SwfdecMovie *thisp;
   const char *name;
   guint version;
+  gboolean need_constructor = FALSE;
 
   g_return_if_fail (SWFDEC_IS_ACTOR (actor));
 
@@ -203,51 +234,53 @@ swfdec_actor_execute (SwfdecActor *actor, SwfdecEventType condition,
     /* these conditions don't exist for buttons */
     if (condition == SWFDEC_EVENT_CONSTRUCT || condition < SWFDEC_EVENT_PRESS)
       return;
-    thisp = SWFDEC_AS_OBJECT (SWFDEC_MOVIE (actor)->parent);
+    thisp = SWFDEC_MOVIE (actor)->parent;
     if (version <= 5) {
       while (!SWFDEC_IS_SPRITE_MOVIE (thisp))
-	thisp = SWFDEC_AS_OBJECT (SWFDEC_MOVIE (thisp)->parent);
+	thisp = SWFDEC_MOVIE (thisp)->parent;
     }
     g_assert (thisp);
   } else {
-    thisp = SWFDEC_AS_OBJECT (actor);
+    thisp = SWFDEC_MOVIE (actor);
   }
 
   /* special cases */
   if (condition == SWFDEC_EVENT_CONSTRUCT) {
     if (version <= 5)
       return;
-    swfdec_sprite_movie_set_constructor (SWFDEC_SPRITE_MOVIE (actor));
+    need_constructor = swfdec_sprite_movie_set_constructor (SWFDEC_SPRITE_MOVIE (actor));
   } else if (condition == SWFDEC_EVENT_ENTER) {
     if (SWFDEC_MOVIE (actor)->state >= SWFDEC_MOVIE_STATE_REMOVED)
       return;
   } else if (condition == SWFDEC_EVENT_SCROLL || condition == SWFDEC_EVENT_CHANGED) {
     SwfdecAsValue argv[2];
+    SwfdecMovie *movie = SWFDEC_MOVIE (actor);
 
     if (condition == SWFDEC_EVENT_SCROLL)
       SWFDEC_AS_VALUE_SET_STRING (&argv[0], SWFDEC_AS_STR_onScroller);
     else
       SWFDEC_AS_VALUE_SET_STRING (&argv[0], SWFDEC_AS_STR_onChanged);
-    SWFDEC_AS_VALUE_SET_OBJECT (&argv[1], SWFDEC_AS_OBJECT (actor));
-    swfdec_sandbox_use (SWFDEC_MOVIE (actor)->resource->sandbox);
-    swfdec_as_object_call (SWFDEC_AS_OBJECT (actor),
+    SWFDEC_AS_VALUE_SET_MOVIE (&argv[1], movie);
+    swfdec_sandbox_use (movie->resource->sandbox);
+    swfdec_as_relay_call (SWFDEC_AS_RELAY (actor),
 	SWFDEC_AS_STR_broadcastMessage, 2, argv, NULL);
-    swfdec_sandbox_unuse (SWFDEC_MOVIE (actor)->resource->sandbox);
+    swfdec_sandbox_unuse (movie->resource->sandbox);
     return;
   }
 
   swfdec_sandbox_use (SWFDEC_MOVIE (actor)->resource->sandbox);
   if (actor->events) {
-    swfdec_event_list_execute (actor->events, thisp, condition, key);
+    swfdec_event_list_execute (actor->events, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (thisp)),
+	condition, key);
   }
   /* FIXME: how do we compute the version correctly here? */
   if (version > 5) {
     name = swfdec_event_type_get_name (condition);
     if (name != NULL) {
-      swfdec_as_object_call (SWFDEC_AS_OBJECT (actor), name, 0, NULL, NULL);
+      swfdec_as_relay_call (SWFDEC_AS_RELAY (actor), name, 0, NULL, NULL);
     }
-    if (condition == SWFDEC_EVENT_CONSTRUCT)
-      swfdec_as_object_call (thisp, SWFDEC_AS_STR_constructor, 0, NULL, NULL);
+    if (condition == SWFDEC_EVENT_CONSTRUCT && need_constructor)
+      swfdec_as_relay_call (SWFDEC_AS_RELAY (thisp), SWFDEC_AS_STR_constructor, 0, NULL, NULL);
   }
   swfdec_sandbox_unuse (SWFDEC_MOVIE (actor)->resource->sandbox);
 }
@@ -309,7 +342,7 @@ swfdec_actor_queue_script_with_key (SwfdecActor *actor,
       g_return_if_reached ();
   }
 
-  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (actor)->context);
+  player = SWFDEC_PLAYER (swfdec_gc_object_get_context (actor));
   swfdec_player_add_action (player, actor, condition, key, importance);
 }
 
