@@ -24,6 +24,7 @@
 #include "swfdec_amf.h"
 #include "swfdec_as_array.h"
 #include "swfdec_as_date.h"
+#include "swfdec_as_strings.h"
 #include "swfdec_debug.h"
 
 typedef gboolean (* SwfdecAmfParseFunc) (SwfdecAsContext *cx, SwfdecBits *bits, SwfdecAsValue *val);
@@ -39,7 +40,7 @@ swfdec_amf_parse_boolean (SwfdecAsContext *context, SwfdecBits *bits, SwfdecAsVa
 static gboolean
 swfdec_amf_parse_number (SwfdecAsContext *context, SwfdecBits *bits, SwfdecAsValue *val)
 {
-  SWFDEC_AS_VALUE_SET_NUMBER (val, swfdec_bits_get_bdouble (bits));
+  *val = swfdec_as_value_from_number (context, swfdec_bits_get_bdouble (bits));
   return TRUE;
 }
 
@@ -69,7 +70,7 @@ swfdec_amf_parse_properties (SwfdecAsContext *context, SwfdecBits *bits, SwfdecA
 
     if (!swfdec_amf_parse_string (context, bits, &val))
       return FALSE;
-    name = SWFDEC_AS_VALUE_GET_STRING (&val);
+    name = SWFDEC_AS_VALUE_GET_STRING (val);
     type = swfdec_bits_get_u8 (bits);
     if (type == SWFDEC_AMF_END_OBJECT)
       break;
@@ -96,9 +97,7 @@ swfdec_amf_parse_object (SwfdecAsContext *context, SwfdecBits *bits, SwfdecAsVal
 {
   SwfdecAsObject *object;
   
-  object = swfdec_as_object_new (context);
-  if (object == NULL)
-    return FALSE;
+  object = swfdec_as_object_new (context, SWFDEC_AS_STR_Object, NULL);
   if (!swfdec_amf_parse_properties (context, bits, object))
     return FALSE;
   SWFDEC_AS_VALUE_SET_OBJECT (val, object);
@@ -113,8 +112,6 @@ swfdec_amf_parse_mixed_array (SwfdecAsContext *context, SwfdecBits *bits, Swfdec
   
   len = swfdec_bits_get_bu32 (bits);
   array = swfdec_as_array_new (context);
-  if (array == NULL)
-    return FALSE;
   if (!swfdec_amf_parse_properties (context, bits, array))
     return FALSE;
   SWFDEC_AS_VALUE_SET_OBJECT (val, array);
@@ -131,8 +128,6 @@ swfdec_amf_parse_array (SwfdecAsContext *context, SwfdecBits *bits, SwfdecAsValu
   
   len = swfdec_bits_get_bu32 (bits);
   array = swfdec_as_array_new (context);
-  if (array == NULL)
-    return FALSE;
   for (i = 0; i < len; i++) {
     SwfdecAsValue tmp;
     type = swfdec_bits_get_u8 (bits);
@@ -143,7 +138,7 @@ swfdec_amf_parse_array (SwfdecAsContext *context, SwfdecBits *bits, SwfdecAsValu
     }
     if (!func (context, bits, &tmp))
       goto fail;
-    swfdec_as_array_push (SWFDEC_AS_ARRAY (array), &tmp);
+    swfdec_as_array_push (array, &tmp);
   }
 
   SWFDEC_AS_VALUE_SET_OBJECT (val, array);
@@ -157,14 +152,17 @@ fail:
 static gboolean
 swfdec_amf_parse_date (SwfdecAsContext *context, SwfdecBits *bits, SwfdecAsValue *val)
 {
-  double milliseconds = swfdec_bits_get_bdouble (bits);
-  int utc_offset = swfdec_bits_get_bu16 (bits);
+  SwfdecAsDate *date;
+  SwfdecAsObject *object;
 
-  if (utc_offset > 12 * 60)
-    utc_offset -= 12 * 60;
+  object = swfdec_as_object_new (context, SWFDEC_AS_STR_Date, NULL);
+  date = SWFDEC_AS_DATE (object->relay);
+  date->milliseconds = swfdec_bits_get_bdouble (bits);
+  date->utc_offset = swfdec_bits_get_bu16 (bits);
+  if (date->utc_offset > 12 * 60)
+    date->utc_offset -= 12 * 60;
+  SWFDEC_AS_VALUE_SET_OBJECT (val, object);
 
-  SWFDEC_AS_VALUE_SET_OBJECT (val,
-      swfdec_as_date_new (context, milliseconds, utc_offset));
   return TRUE;
 }
 
@@ -198,7 +196,6 @@ swfdec_amf_parse_one (SwfdecAsContext *context, SwfdecBits *bits,
   guint type;
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), 0);
-  g_return_val_if_fail (context->global != NULL, 0);
   g_return_val_if_fail (bits != NULL, FALSE);
   g_return_val_if_fail (rval != NULL, FALSE);
   g_return_val_if_fail (expected_type < SWFDEC_AMF_N_TYPES, FALSE);
@@ -224,7 +221,6 @@ swfdec_amf_parse (SwfdecAsContext *context, SwfdecBits *bits, guint n_items, ...
   guint i;
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), 0);
-  g_return_val_if_fail (context->global != NULL, 0);
   g_return_val_if_fail (bits != NULL, 0);
 
   va_start (args, n_items);
