@@ -37,23 +37,15 @@
 #include "swfdec_utils.h"
 #include "swfdec_debug.h"
 
-G_DEFINE_TYPE (SwfdecAsArray, swfdec_as_array, SWFDEC_TYPE_AS_OBJECT)
-
 /**
- * SECTION:SwfdecAsArray
- * @title: SwfdecAsArray
- * @short_description: the array object
+ * SECTION:Arrays
+ * @title: arrays
+ * @short_description: utility functions for treating objects as arrays
  *
  * The array object provides some convenience functions for creating and
  * modifying arrays.
  */
 
-/**
- * SwfdecAsArray
- *
- * This is the type of the array object.
- */
-
 /*
  * Internal helper functions
  */
@@ -84,17 +76,27 @@ swfdec_as_array_length_as_integer (SwfdecAsObject *object)
   g_return_val_if_fail (object != NULL, 0);
 
   swfdec_as_object_get_variable (object, SWFDEC_AS_STR_length, &val);
-  length = swfdec_as_value_to_integer (swfdec_gc_object_get_context (object), &val);
+  length = swfdec_as_value_to_integer (object->context, val);
 
   return length;
 }
 
-static gint32
-swfdec_as_array_length (SwfdecAsObject *object)
+/**
+ * swfdec_as_array_get_length:
+ * @array: the array
+ *
+ * Gets the current length of the @array.
+ *
+ * Returns: Current length of the @array, always >= 0
+ **/
+gint32
+swfdec_as_array_get_length (SwfdecAsObject *array)
 {
   gint32 length;
 
-  length = swfdec_as_array_length_as_integer (object);
+  g_return_val_if_fail (array != NULL, 0);
+
+  length = swfdec_as_array_length_as_integer (array);
 
   if (length < 0)
     return 0;
@@ -102,56 +104,42 @@ swfdec_as_array_length (SwfdecAsObject *object)
   return length;
 }
 
-/**
- * swfdec_as_array_get_length:
- * @array: a #SwfdecAsArray
- *
- * Gets the current length of the @array.
- *
- * Returns: Current length of the @array, always >= 0
- **/
-gint32
-swfdec_as_array_get_length (SwfdecAsArray *array)
-{
-  return swfdec_as_array_length (SWFDEC_AS_OBJECT (array));
-}
-
 static void
 swfdec_as_array_set_length_object (SwfdecAsObject *object, gint32 length)
 {
   SwfdecAsValue val;
+  gboolean was_array;
 
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
+  g_return_if_fail (object != NULL);
 
-  if (SWFDEC_IS_AS_ARRAY (object))
-    SWFDEC_AS_ARRAY (object)->check_length = FALSE;
+  was_array = object->array;
+  object->array = FALSE;
 
-  SWFDEC_AS_VALUE_SET_INT (&val, length);
+  val = swfdec_as_value_from_integer (object->context, length);
   swfdec_as_object_set_variable_and_flags (object, SWFDEC_AS_STR_length, &val,
       SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
 
-  if (SWFDEC_IS_AS_ARRAY (object))
-    SWFDEC_AS_ARRAY (object)->check_length = TRUE;
+  object->array = was_array;
 }
 
 /**
  * swfdec_as_array_set_length:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @length: the new length
  *
  * Sets the length of the @array. Values outside the new length will be
  * removed.
  **/
 void
-swfdec_as_array_set_length (SwfdecAsArray *array, gint32 length)
+swfdec_as_array_set_length (SwfdecAsObject *array, gint32 length)
 {
   SwfdecAsValue val;
 
-  g_return_if_fail (SWFDEC_IS_AS_ARRAY (array));
+  g_return_if_fail (array != NULL);
   g_return_if_fail (length >= 0);
 
-  SWFDEC_AS_VALUE_SET_INT (&val, length);
-  swfdec_as_object_set_variable_and_flags (SWFDEC_AS_OBJECT (array),
+  val = swfdec_as_value_from_integer (array->context, length);
+  swfdec_as_object_set_variable_and_flags (array,
       SWFDEC_AS_STR_length, &val,
       SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
 }
@@ -181,7 +169,7 @@ swfdec_as_array_foreach_remove_range (SwfdecAsObject *object,
   return FALSE;
 }
 
-static void
+void
 swfdec_as_array_remove_range (SwfdecAsObject *object, gint32 start_index,
     gint32 num)
 {
@@ -193,8 +181,8 @@ swfdec_as_array_remove_range (SwfdecAsObject *object, gint32 start_index,
 
   // to avoid foreach loop, use special case when removing just one variable
   if (num == 1) {
-    swfdec_as_object_delete_variable (object, swfdec_as_integer_to_string (
-	  swfdec_gc_object_get_context (object), start_index));
+    swfdec_as_object_delete_variable (object, 
+	swfdec_as_integer_to_string (object->context, start_index));
   } else {
     ForeachRemoveRangeData fdata = { start_index, num };
     swfdec_as_object_foreach_remove (object,
@@ -220,7 +208,7 @@ swfdec_as_array_foreach_move_range (SwfdecAsObject *object,
     return variable;
 
   if (idx >= fdata->start_index && idx < fdata->start_index + fdata->num) {
-    return swfdec_as_integer_to_string (swfdec_gc_object_get_context (object),
+    return swfdec_as_integer_to_string (object->context,
 	fdata->to_index + idx - fdata->start_index);
   } else if (idx >= fdata->to_index && idx < fdata->to_index + fdata->num) {
     return NULL;
@@ -235,7 +223,7 @@ swfdec_as_array_move_range (SwfdecAsObject *object, gint32 from_index,
 {
   ForeachMoveRangeData fdata = { from_index, num, to_index };
 
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
+  g_return_if_fail (object != NULL);
   g_return_if_fail (from_index >= 0);
   g_return_if_fail (num >= 0);
   g_return_if_fail (to_index >= 0);
@@ -247,7 +235,7 @@ swfdec_as_array_move_range (SwfdecAsObject *object, gint32 from_index,
       &fdata);
 
   // only changes length if it becomes bigger, not if it becomes smaller
-  if (to_index + num > swfdec_as_array_length (object))
+  if (to_index + num > swfdec_as_array_get_length (object))
     swfdec_as_array_set_length_object (object, to_index + num);
 }
 
@@ -260,12 +248,12 @@ swfdec_as_array_set_range_with_flags (SwfdecAsObject *object,
   const char *var;
 
   // allow negative indexes
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
+  g_return_if_fail (object != NULL);
   g_return_if_fail (num >= 0);
   g_return_if_fail (num == 0 || value != NULL);
 
   for (i = 0; i < num; i++) {
-    var = swfdec_as_integer_to_string (swfdec_gc_object_get_context (object), start_index + i);
+    var = swfdec_as_integer_to_string (object->context, start_index + i);
     swfdec_as_object_set_variable_and_flags (object, var, &value[i], flags);
   }
 }
@@ -288,7 +276,7 @@ swfdec_as_array_append_internal (SwfdecAsObject *object, guint n,
 
 /**
  * swfdec_as_array_push:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @value: the value to add
  *
  * Adds the given @value to the @array. This is a macro that just calls
@@ -297,7 +285,7 @@ swfdec_as_array_append_internal (SwfdecAsObject *object, guint n,
 
 /**
  * swfdec_as_array_push_with_flags:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @value: the value to add
  * @flags: the #SwfdecAsVariableFlag flags to use
  *
@@ -307,7 +295,7 @@ swfdec_as_array_append_internal (SwfdecAsObject *object, guint n,
 
 /**
  * swfdec_as_array_append:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @n: number of values to add
  * @values: the values to add
  *
@@ -317,7 +305,7 @@ swfdec_as_array_append_internal (SwfdecAsObject *object, guint n,
 
 /**
  * swfdec_as_array_append_with_flags:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @n: number of values to add
  * @values: the values to add
  * @flags: the #SwfdecAsVariableFlag flags to use
@@ -325,20 +313,20 @@ swfdec_as_array_append_internal (SwfdecAsObject *object, guint n,
  * Appends the given @values to the @array using the given @flags.
  **/
 void
-swfdec_as_array_append_with_flags (SwfdecAsArray *array, guint n,
+swfdec_as_array_append_with_flags (SwfdecAsObject *array, guint n,
     const SwfdecAsValue *value, SwfdecAsVariableFlag flags)
 {
-  g_return_if_fail (SWFDEC_IS_AS_ARRAY (array));
+  g_return_if_fail (array != NULL);
   g_return_if_fail (n == 0 || value != NULL);
 
   // don't allow negative length
-  swfdec_as_array_set_range_with_flags (SWFDEC_AS_OBJECT (array),
-      swfdec_as_array_length (SWFDEC_AS_OBJECT (array)), n, value, flags);
+  swfdec_as_array_set_range_with_flags (array,
+      swfdec_as_array_get_length (array), n, value, flags);
 }
 
 /**
  * swfdec_as_array_insert:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @idx: index to insert the value to
  * @value: a #SwfdecAsValue
  *
@@ -348,7 +336,7 @@ swfdec_as_array_append_with_flags (SwfdecAsArray *array, guint n,
  **/
 /**
  * swfdec_as_array_insert_with_flags:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @idx: index to insert the value to
  * @value: a #SwfdecAsValue
  * @flags: the #SwfdecAsVariableFlag flags to use
@@ -357,54 +345,49 @@ swfdec_as_array_append_with_flags (SwfdecAsArray *array, guint n,
  * it by moving elements to bigger indexes if necessary.
  **/
 void
-swfdec_as_array_insert_with_flags (SwfdecAsArray *array, gint32 idx,
+swfdec_as_array_insert_with_flags (SwfdecAsObject *array, gint32 idx,
     const SwfdecAsValue *value, SwfdecAsVariableFlag flags)
 {
   gint32 length;
-  SwfdecAsObject *object;
 
-  g_return_if_fail (SWFDEC_IS_AS_ARRAY (array));
+  g_return_if_fail (array != NULL);
   g_return_if_fail (idx >= 0);
-  g_return_if_fail (SWFDEC_IS_AS_VALUE (value));
 
-  object = SWFDEC_AS_OBJECT (array);
-  length = swfdec_as_array_length (object);
+  length = swfdec_as_array_get_length (array);
 
   if (idx < length)
-    swfdec_as_array_move_range (object, idx, length - idx, idx + 1);
-  swfdec_as_array_set_range_with_flags (object, idx, 1, value, flags);
+    swfdec_as_array_move_range (array, idx, length - idx, idx + 1);
+  swfdec_as_array_set_range_with_flags (array, idx, 1, value, flags);
 }
 
 /**
  * swfdec_as_array_remove:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @idx: index of the value to remove
  *
  * Removes value at given index from the @array, elements with higher indexes
  * will be moved towards the start of the @array.
  **/
 void
-swfdec_as_array_remove (SwfdecAsArray *array, gint32 idx)
+swfdec_as_array_remove (SwfdecAsObject *array, gint32 idx)
 {
   gint32 length;
-  SwfdecAsObject *object;
 
-  g_return_if_fail (SWFDEC_IS_AS_ARRAY (array));
+  g_return_if_fail (array != NULL);
   g_return_if_fail (idx >= 0);
 
-  object = SWFDEC_AS_OBJECT (array);
-  length = swfdec_as_array_length (object);
+  length = swfdec_as_array_get_length (array);
 
   if (idx >= length)
     return;
 
-  swfdec_as_array_move_range (object, idx + 1, length - (idx + 1), idx);
+  swfdec_as_array_move_range (array, idx + 1, length - (idx + 1), idx);
   swfdec_as_array_set_length (array, length - 1);
 }
 
 /**
  * swfdec_as_array_get_value:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @idx: index of the value to get
  * @value: a pointer to #SwfdecAsValue that will be set
  *
@@ -412,22 +395,22 @@ swfdec_as_array_remove (SwfdecAsArray *array, gint32 idx)
  * value is set.
  **/
 void
-swfdec_as_array_get_value (SwfdecAsArray *array, gint32 idx,
+swfdec_as_array_get_value (SwfdecAsObject *array, gint32 idx,
     SwfdecAsValue *value)
 {
   const char *var;
 
-  g_assert (SWFDEC_IS_AS_ARRAY (array));
+  g_assert (array != NULL);
   g_assert (idx >= 0);
   g_assert (value != NULL);
 
-  var = swfdec_as_integer_to_string (swfdec_gc_object_get_context (array), idx);
-  swfdec_as_object_get_variable (SWFDEC_AS_OBJECT (array), var, value);
+  var = swfdec_as_integer_to_string (array->context, idx);
+  swfdec_as_object_get_variable (array, var, value);
 }
 
 /**
  * swfdec_as_array_set_value:
- * @array: a #SwfdecAsArray
+ * @array: the array
  * @idx: index of the value to set
  * @value: a pointer to #SwfdecAsValue
  *
@@ -435,17 +418,16 @@ swfdec_as_array_get_value (SwfdecAsArray *array, gint32 idx,
  * necessary.
  **/
 void
-swfdec_as_array_set_value (SwfdecAsArray *array, gint32 idx,
+swfdec_as_array_set_value (SwfdecAsObject *array, gint32 idx,
     SwfdecAsValue *value)
 {
   const char *var;
 
-  g_assert (SWFDEC_IS_AS_ARRAY (array));
+  g_assert (array != NULL);
   g_assert (idx >= 0);
-  g_assert (SWFDEC_IS_AS_VALUE (value));
 
-  var = swfdec_as_integer_to_string (swfdec_gc_object_get_context (array), idx);
-  swfdec_as_object_set_variable (SWFDEC_AS_OBJECT (array), var, value);
+  var = swfdec_as_integer_to_string (array->context, idx);
+  swfdec_as_object_set_variable (array, var, value);
 }
 
 typedef struct {
@@ -465,7 +447,7 @@ swfdec_as_array_foreach_append_array_range (SwfdecAsObject *object,
 
   idx = swfdec_as_array_to_index (variable);
   if (idx >= fdata->start_index && idx < fdata->start_index + fdata->num) {
-    var = swfdec_as_integer_to_string (swfdec_gc_object_get_context (fdata->object_to),
+    var = swfdec_as_integer_to_string (fdata->object_to->context,
 	fdata->offset + (idx - fdata->start_index));
     swfdec_as_object_set_variable (fdata->object_to, var, value);
   }
@@ -474,20 +456,20 @@ swfdec_as_array_foreach_append_array_range (SwfdecAsObject *object,
 }
 
 static void
-swfdec_as_array_append_array_range (SwfdecAsArray *array_to,
+swfdec_as_array_append_array_range (SwfdecAsObject *array_to,
     SwfdecAsObject *object_from, gint32 start_index, gint32 num)
 {
   ForeachAppendArrayRangeData fdata;
 
-  g_return_if_fail (SWFDEC_IS_AS_ARRAY (array_to));
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object_from));
+  g_return_if_fail (array_to != NULL);
+  g_return_if_fail (object_from != NULL);
   g_return_if_fail (start_index >= 0);
 
   if (num == 0)
     return;
 
-  fdata.object_to = SWFDEC_AS_OBJECT (array_to);
-  fdata.offset = swfdec_as_array_length (SWFDEC_AS_OBJECT (array_to));
+  fdata.object_to = array_to;
+  fdata.offset = swfdec_as_array_get_length (array_to);
   fdata.start_index = start_index;
   fdata.num = num;
 
@@ -498,64 +480,11 @@ swfdec_as_array_append_array_range (SwfdecAsArray *array_to,
 }
 
 static void
-swfdec_as_array_append_array (SwfdecAsArray *array_to,
+swfdec_as_array_append_array (SwfdecAsObject *array_to,
     SwfdecAsObject *object_from)
 {
   swfdec_as_array_append_array_range (array_to, object_from, 0,
-      swfdec_as_array_length (object_from));
-}
-
-/*
- * Class functions
- */
-
-static void
-swfdec_as_array_set (SwfdecAsObject *object, const char *variable,
-    const SwfdecAsValue *val, guint flags)
-{
-  gboolean indexvar;
-  char *end;
-  gint32 l;
-
-  // we have to allow negative values here
-  l = strtoul (variable, &end, 10);
-  indexvar = (*end == 0);
-
-  // if we changed to smaller length, destroy all values that are outside it
-  //
-  if (SWFDEC_AS_ARRAY (object)->check_length &&
-      !swfdec_strcmp (swfdec_gc_object_get_context (object)->version, variable,
-	SWFDEC_AS_STR_length))
-  {
-    gint32 length_old = swfdec_as_array_length (object);
-    gint32 length_new = swfdec_as_value_to_integer (swfdec_gc_object_get_context (object), val);
-    length_new = MAX (0, length_new);
-    if (length_old > length_new) {
-      swfdec_as_array_remove_range (object, length_new,
-	  length_old - length_new);
-    }
-  }
-
-  SWFDEC_AS_OBJECT_CLASS (swfdec_as_array_parent_class)->set (object, variable,
-      val, flags);
-
-  // if we added new value outside the current length, set a bigger length
-  if (indexvar && ++l > swfdec_as_array_length_as_integer (object))
-    swfdec_as_array_set_length_object (object, l);
-}
-
-static void
-swfdec_as_array_class_init (SwfdecAsArrayClass *klass)
-{
-  SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
-
-  asobject_class->set = swfdec_as_array_set;
-}
-
-static void
-swfdec_as_array_init (SwfdecAsArray *array)
-{
-  array->check_length = TRUE;
+      swfdec_as_array_get_length (object_from));
 }
 
 /*
@@ -566,7 +495,7 @@ swfdec_as_array_init (SwfdecAsArray *array)
  * swfdec_as_array_new:
  * @context: a #SwfdecAsContext
  *
- * Creates a new #SwfdecAsArray. 
+ * Creates a new array.
  *
  * Returns: the new array
  **/
@@ -577,7 +506,8 @@ swfdec_as_array_new (SwfdecAsContext *context)
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
 
-  ret = g_object_new (SWFDEC_TYPE_AS_ARRAY, "context", context, NULL);
+  ret = swfdec_as_object_new (context, NULL);
+  ret->array = TRUE;
   swfdec_as_object_set_constructor_by_name (ret, SWFDEC_AS_STR_Array, NULL);
 
   swfdec_as_array_set_length_object (ret, 0);
@@ -596,25 +526,25 @@ swfdec_as_array_join (SwfdecAsContext *cx, SwfdecAsObject *object, guint argc,
   const char *var, *str, *sep;
   SwfdecAsValue val;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
   if (argc > 0) {
-    sep = swfdec_as_value_to_string (cx, &argv[0]);
+    sep = swfdec_as_value_to_string (cx, argv[0]);
   } else {
     sep = SWFDEC_AS_STR_COMMA;
   }
 
   // note: we don't cache length
-  if (swfdec_as_array_length (object) > 0) {
+  if (swfdec_as_array_get_length (object) > 0) {
     GString *string;
     swfdec_as_object_get_variable (object, SWFDEC_AS_STR_0, &val);
-    str = swfdec_as_value_to_string (cx, &val);
+    str = swfdec_as_value_to_string (cx, val);
     string = g_string_new (str);
-    for (i = 1; i < swfdec_as_array_length (object); i++) {
+    for (i = 1; i < swfdec_as_array_get_length (object); i++) {
       var = swfdec_as_integer_to_string (cx, i);
       swfdec_as_object_get_variable (object, var, &val);
-      var = swfdec_as_value_to_string (cx, &val);
+      var = swfdec_as_value_to_string (cx, val);
       g_string_append (string, sep);
       g_string_append (string, var);
     }
@@ -631,7 +561,7 @@ void
 swfdec_as_array_toString (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
   swfdec_as_array_join (cx, object, 0, NULL, ret);
@@ -642,7 +572,7 @@ void
 swfdec_as_array_do_push (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
   // if 0 args, just return the length
@@ -653,7 +583,7 @@ swfdec_as_array_do_push (SwfdecAsContext *cx, SwfdecAsObject *object,
     swfdec_as_array_set_length_object (object, length + argc);
   }
 
-  SWFDEC_AS_VALUE_SET_INT (ret, swfdec_as_array_length_as_integer (object));
+  *ret = swfdec_as_value_from_integer (cx, swfdec_as_array_length_as_integer (object));
 }
 
 SWFDEC_AS_NATIVE (252, 2, swfdec_as_array_do_pop)
@@ -664,7 +594,7 @@ swfdec_as_array_do_pop (SwfdecAsContext *cx, SwfdecAsObject *object,
   gint32 length;
   const char *var;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
   // we allow negative indexes here, but not 0
@@ -672,14 +602,14 @@ swfdec_as_array_do_pop (SwfdecAsContext *cx, SwfdecAsObject *object,
   if (length == 0)
     return;
 
-  var = swfdec_as_integer_to_string (swfdec_gc_object_get_context (object), length - 1);
+  var = swfdec_as_integer_to_string (object->context, length - 1);
   swfdec_as_object_get_variable (object, var, ret);
 
   swfdec_as_object_delete_variable (object, var);
 
   // if Array, the length is reduced by one
   // else the length is not reduced at all, but the variable is still deleted
-  if (SWFDEC_IS_AS_ARRAY (object))
+  if (object->array)
     swfdec_as_array_set_length_object (object, length - 1);
 }
 
@@ -690,20 +620,20 @@ swfdec_as_array_do_unshift (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   gint32 length;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
   if (argc) {
     // don't allow negative length
-    length = swfdec_as_array_length (object);
+    length = swfdec_as_array_get_length (object);
     swfdec_as_array_move_range (object, 0, length, argc);
     swfdec_as_array_set_range (object, 0, argc, argv);
     // if not Array, leave the length unchanged
-    if (!SWFDEC_IS_AS_ARRAY (object))
+    if (!object->array)
       swfdec_as_array_set_length_object (object, length);
   }
 
-  SWFDEC_AS_VALUE_SET_INT (ret, swfdec_as_array_length (object));
+  *ret = swfdec_as_value_from_integer (cx, swfdec_as_array_get_length (object));
 }
 
 SWFDEC_AS_NATIVE (252, 4, swfdec_as_array_do_shift)
@@ -714,11 +644,11 @@ swfdec_as_array_do_shift (SwfdecAsContext *cx, SwfdecAsObject *object,
   gint32 length;
   const char *var;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
   // don't allow negative length
-  length = swfdec_as_array_length (object);
+  length = swfdec_as_array_get_length (object);
   if (length <= 0)
     return;
 
@@ -727,18 +657,18 @@ swfdec_as_array_do_shift (SwfdecAsContext *cx, SwfdecAsObject *object,
   swfdec_as_array_move_range (object, 1, length - 1, 0);
 
   // if not Array, leave the length unchanged, and don't remove the element
-  if (SWFDEC_IS_AS_ARRAY (object)) {
+  if (object->array) {
     swfdec_as_array_set_length_object (object, length - 1);
   } else {
     // we have to put the last element back, because we used move, not copy
     SwfdecAsValue val;
     if (length > 1) {
-      var = swfdec_as_integer_to_string (swfdec_gc_object_get_context (object), length - 2);
+      var = swfdec_as_integer_to_string (object->context, length - 2);
       swfdec_as_object_get_variable (object, var, &val);
     } else {
       val = *ret;
     }
-    var = swfdec_as_integer_to_string (swfdec_gc_object_get_context (object), length - 1);
+    var = swfdec_as_integer_to_string (object->context, length - 1);
     swfdec_as_object_set_variable (object, var, &val);
   }
 }
@@ -754,7 +684,7 @@ swfdec_as_array_foreach_reverse (SwfdecAsObject *object, const char *variable,
   if (idx == -1 || idx >= *length)
     return variable;
 
-  return swfdec_as_integer_to_string (swfdec_gc_object_get_context (object), *length - 1 - idx);
+  return swfdec_as_integer_to_string (object->context, *length - 1 - idx);
 }
 
 SWFDEC_AS_NATIVE (252, 11, swfdec_as_array_reverse)
@@ -764,10 +694,10 @@ swfdec_as_array_reverse (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   gint32 length;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
-  length = swfdec_as_array_length (object);
+  length = swfdec_as_array_get_length (object);
   swfdec_as_object_foreach_rename (object, swfdec_as_array_foreach_reverse,
       &length);
 
@@ -780,36 +710,31 @@ swfdec_as_array_concat (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   guint j;
-  SwfdecAsObject *object_new;
-  SwfdecAsArray *array_new;
+  SwfdecAsObject *array_new;
   const char *var;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
-  object_new = swfdec_as_array_new (cx);
-  if (object_new == NULL)
-    return;
-  array_new = SWFDEC_AS_ARRAY (object_new);
+  array_new = swfdec_as_array_new (cx);
 
   swfdec_as_array_append_array (array_new, object);
 
   for (j = 0; j < argc; j++) {
-    if (SWFDEC_AS_VALUE_IS_OBJECT (&argv[j]) &&
-	SWFDEC_IS_AS_ARRAY (SWFDEC_AS_VALUE_GET_OBJECT (&argv[j])))
+    if (SWFDEC_AS_VALUE_IS_OBJECT (argv[j]) &&
+	SWFDEC_AS_VALUE_GET_OBJECT (argv[j])->array)
     {
       swfdec_as_array_append_array (array_new,
-	  SWFDEC_AS_VALUE_GET_OBJECT (&argv[j]));
+	  SWFDEC_AS_VALUE_GET_OBJECT (argv[j]));
     }
     else
     {
-      var = swfdec_as_integer_to_string (swfdec_gc_object_get_context (object),
-	  swfdec_as_array_length (object_new));
-      swfdec_as_object_set_variable (object_new, var, &argv[j]);
+      var = swfdec_as_integer_to_string (cx, swfdec_as_array_get_length (array_new));
+      swfdec_as_object_set_variable (array_new, var, &argv[j]);
     }
   }
 
-  SWFDEC_AS_VALUE_SET_OBJECT (ret, object_new);
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, array_new);
 }
 
 SWFDEC_AS_NATIVE (252, 6, swfdec_as_array_slice)
@@ -818,15 +743,15 @@ swfdec_as_array_slice (SwfdecAsContext *cx, SwfdecAsObject *object, guint argc,
     SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   gint32 length, start_index, num;
-  SwfdecAsArray *array_new;
+  SwfdecAsObject *array_new;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
-  length = swfdec_as_array_length (object);
+  length = swfdec_as_array_get_length (object);
 
   if (argc > 0) {
-    start_index = swfdec_as_value_to_integer (cx, &argv[0]);
+    start_index = swfdec_as_value_to_integer (cx, argv[0]);
     if (start_index < 0)
       start_index = length + start_index;
     start_index = CLAMP (start_index, 0, length);
@@ -835,7 +760,7 @@ swfdec_as_array_slice (SwfdecAsContext *cx, SwfdecAsObject *object, guint argc,
   }
 
   if (argc > 1) {
-    gint32 endIndex = swfdec_as_value_to_integer (cx, &argv[1]);
+    gint32 endIndex = swfdec_as_value_to_integer (cx, argv[1]);
     if (endIndex < 0)
       endIndex = length + endIndex;
     endIndex = CLAMP (endIndex, start_index, length);
@@ -844,13 +769,11 @@ swfdec_as_array_slice (SwfdecAsContext *cx, SwfdecAsObject *object, guint argc,
     num = length - start_index;
   }
 
-  array_new = SWFDEC_AS_ARRAY (swfdec_as_array_new (cx));
-  if (array_new == NULL)
-    return;
+  array_new = swfdec_as_array_new (cx);
 
   swfdec_as_array_append_array_range (array_new, object, start_index, num);
 
-  SWFDEC_AS_VALUE_SET_OBJECT (ret, SWFDEC_AS_OBJECT (array_new));
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, array_new);
 }
 
 SWFDEC_AS_NATIVE (252, 8, swfdec_as_array_splice)
@@ -859,20 +782,20 @@ swfdec_as_array_splice (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   gint32 length, start_index, num_remove, num_add, at_end;
-  SwfdecAsArray *array_new;
+  SwfdecAsObject *array_new;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object) || argc == 0)
+  if (object == NULL || object->movie || argc == 0)
     return;
 
-  length = swfdec_as_array_length (object);
+  length = swfdec_as_array_get_length (object);
 
-  start_index = swfdec_as_value_to_integer (cx, &argv[0]);
+  start_index = swfdec_as_value_to_integer (cx, argv[0]);
   if (start_index < 0)
     start_index = length + start_index;
   start_index = CLAMP (start_index, 0, length);
 
   if (argc > 1) {
-    int tmp = swfdec_as_value_to_integer (cx, &argv[1]);
+    int tmp = swfdec_as_value_to_integer (cx, argv[1]);
     if (tmp < 0)
       return;
     num_remove = MIN (tmp, length - start_index);
@@ -884,10 +807,10 @@ swfdec_as_array_splice (SwfdecAsContext *cx, SwfdecAsObject *object,
   at_end = length - num_remove - start_index;
 
   /* create return value */
-  array_new = SWFDEC_AS_ARRAY (swfdec_as_array_new (cx));
+  array_new = swfdec_as_array_new (cx);
   swfdec_as_array_append_array_range (array_new, object, start_index,
       num_remove);
-  SWFDEC_AS_VALUE_SET_OBJECT (ret, SWFDEC_AS_OBJECT (array_new));
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, array_new);
 
   /* move old data to the right spot */
   swfdec_as_array_move_range (object, start_index + num_remove,
@@ -929,8 +852,6 @@ swfdec_as_array_sort_compare_values (SwfdecAsContext *cx,
   int retval;
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (cx), 0);
-  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (a), 0);
-  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (b), 0);
   g_return_val_if_fail (custom_function == NULL ||
       SWFDEC_IS_AS_FUNCTION (custom_function), 0);
 
@@ -938,23 +859,22 @@ swfdec_as_array_sort_compare_values (SwfdecAsContext *cx,
   {
     SwfdecAsValue argv[2] = { *a, *b };
     SwfdecAsValue ret;
-    SwfdecAsContext *context = swfdec_gc_object_get_context (custom_function);
     swfdec_as_function_call (custom_function, NULL, 2, argv, &ret);
-    retval = swfdec_as_value_to_integer (context, &ret);
+    retval = swfdec_as_value_to_integer (cx, ret);
   }
   else if (options & SORT_OPTION_NUMERIC &&
-      (SWFDEC_AS_VALUE_IS_NUMBER (a) ||
-       SWFDEC_AS_VALUE_IS_NUMBER (b)) &&
-      !SWFDEC_AS_VALUE_IS_UNDEFINED (a) &&
-      !SWFDEC_AS_VALUE_IS_UNDEFINED (b))
+      (SWFDEC_AS_VALUE_IS_NUMBER (*a) ||
+       SWFDEC_AS_VALUE_IS_NUMBER (*b)) &&
+      !SWFDEC_AS_VALUE_IS_UNDEFINED (*a) &&
+      !SWFDEC_AS_VALUE_IS_UNDEFINED (*b))
   {
-    if (!SWFDEC_AS_VALUE_IS_NUMBER (a)) {
+    if (!SWFDEC_AS_VALUE_IS_NUMBER (*a)) {
       retval = 1;
-    } else if (!SWFDEC_AS_VALUE_IS_NUMBER (b)) {
+    } else if (!SWFDEC_AS_VALUE_IS_NUMBER (*b)) {
       retval = -1;
     } else {
-      double an = swfdec_as_value_to_number (cx, a);
-      double bn = swfdec_as_value_to_number (cx, b);
+      double an = swfdec_as_value_to_number (cx, *a);
+      double bn = swfdec_as_value_to_number (cx, *b);
       retval = (an < bn ? -1 : (an > bn ? 1 : 0));
     }
   }
@@ -962,8 +882,8 @@ swfdec_as_array_sort_compare_values (SwfdecAsContext *cx,
   {
     // can't pass swfdec_as_value_to_string calls directly to compare
     // functions, since the order of these is important
-    const char *a_str = swfdec_as_value_to_string (cx, a);
-    const char *b_str = swfdec_as_value_to_string (cx, b);
+    const char *a_str = swfdec_as_value_to_string (cx, *a);
+    const char *b_str = swfdec_as_value_to_string (cx, *b);
 
     if (options & SORT_OPTION_CASEINSENSITIVE) {
       retval = g_strcasecmp (a_str, b_str);
@@ -995,8 +915,6 @@ swfdec_as_array_sort_compare (gconstpointer a_ptr, gconstpointer b_ptr,
   SortCompareData *data = user_data;
   int retval;
 
-  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (a), 0);
-  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (b), 0);
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (data->context), 0);
   g_return_val_if_fail (data->options != NULL, 0);
   g_return_val_if_fail (data->custom_function == NULL ||
@@ -1013,14 +931,14 @@ swfdec_as_array_sort_compare (gconstpointer a_ptr, gconstpointer b_ptr,
 
     i = 0;
     do {
-      object = swfdec_as_value_to_object (data->context, a);
+      object = swfdec_as_value_to_object (data->context, *a);
       if (object) {
 	swfdec_as_object_get_variable (object, data->fields[i], &a_comp);
       } else {
 	SWFDEC_AS_VALUE_SET_UNDEFINED (&a_comp);
       }
 
-      object = swfdec_as_value_to_object (data->context, b);
+      object = swfdec_as_value_to_object (data->context, *b);
       if (object) {
 	swfdec_as_object_get_variable (object, data->fields[i], &b_comp);
       } else {
@@ -1106,7 +1024,7 @@ swfdec_as_array_foreach_sort_collect (SwfdecAsObject *object,
   if (index_ == -1 || index_ >= collect_data->length)
     return TRUE;
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (value))
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (*value))
     return TRUE;
 
   collect_data->array[index_].value = *value;
@@ -1130,13 +1048,13 @@ swfdec_as_array_do_sort (SwfdecAsContext *cx, SwfdecAsObject *object,
   gboolean descending;
 
   g_return_if_fail (SWFDEC_IS_AS_CONTEXT (cx));
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (object));
+  g_return_if_fail (object != NULL);
   g_return_if_fail (options != NULL);
   g_return_if_fail (custom_function == NULL ||
       SWFDEC_IS_AS_FUNCTION (custom_function));
   g_return_if_fail (fields == NULL || fields[0] != NULL);
 
-  length = swfdec_as_array_length (object);
+  length = swfdec_as_array_get_length (object);
   if (length == 0) {
     // special case for empty array, because g_try_new0 would return NULL
     SWFDEC_AS_VALUE_SET_OBJECT (ret, object);
@@ -1201,19 +1119,17 @@ swfdec_as_array_do_sort (SwfdecAsContext *cx, SwfdecAsObject *object,
 	  break;
       }
       if (i < length - 1) {
-	SWFDEC_AS_VALUE_SET_INT (ret, 0);
+	*ret = swfdec_as_value_from_integer (cx, 0);
 	goto done;
       }
     } else if (compare_data.equal_found) {
-      SWFDEC_AS_VALUE_SET_INT (ret, 0);
+      *ret = swfdec_as_value_from_integer (cx, 0);
       goto done;
     }
   }
 
   if (options[0] & SORT_OPTION_RETURNINDEXEDARRAY) {
     target = swfdec_as_array_new (cx);
-    if (!target)
-      goto done;
   } else {
     target = object;
   }
@@ -1228,7 +1144,7 @@ swfdec_as_array_do_sort (SwfdecAsContext *cx, SwfdecAsObject *object,
 
     var = swfdec_as_integer_to_string (cx, (descending ? length - i - 1 : i));
     if (options[0] & SORT_OPTION_RETURNINDEXEDARRAY) {
-      SWFDEC_AS_VALUE_SET_INT (&val, entry->index_);
+      val = swfdec_as_value_from_integer (cx, entry->index_);
       swfdec_as_object_set_variable (target, var, &val);
     } else {
       swfdec_as_object_set_variable (target, var, &entry->value);
@@ -1251,16 +1167,16 @@ swfdec_as_array_sort (SwfdecAsContext *cx, SwfdecAsObject *object, guint argc,
   SortOption options;
   SwfdecAsFunction *custom_function;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
   pos = 0;
 
-  if (argc > pos && !SWFDEC_AS_VALUE_IS_NUMBER (&argv[pos])) {
+  if (argc > pos && !SWFDEC_AS_VALUE_IS_NUMBER (argv[pos])) {
     SwfdecAsFunction *fun;
-    if (!SWFDEC_AS_VALUE_IS_OBJECT (&argv[pos]) ||
+    if (!SWFDEC_AS_VALUE_IS_OBJECT (argv[pos]) ||
 	!SWFDEC_IS_AS_FUNCTION (
-	  fun = (SwfdecAsFunction *) SWFDEC_AS_VALUE_GET_OBJECT (&argv[pos])))
+	  fun = (SwfdecAsFunction *) SWFDEC_AS_VALUE_GET_OBJECT (argv[pos])->relay))
 	return;
     custom_function = fun;
     pos++;
@@ -1269,7 +1185,7 @@ swfdec_as_array_sort (SwfdecAsContext *cx, SwfdecAsObject *object, guint argc,
   }
 
   if (argc > pos) {
-    options = swfdec_as_value_to_integer (cx, &argv[pos]) & MASK_SORT_OPTION;
+    options = swfdec_as_value_to_integer (cx, argv[pos]) & MASK_SORT_OPTION;
   } else {
     options = 0;
   }
@@ -1288,21 +1204,21 @@ swfdec_as_array_sortOn (SwfdecAsContext *cx, SwfdecAsObject *object,
   SwfdecAsObject *array;
   SwfdecAsValue val;
 
-  if (object == NULL || SWFDEC_IS_MOVIE (object))
+  if (object == NULL || object->movie)
     return;
 
   if (argc < 1)
     return;
 
-  if (SWFDEC_AS_VALUE_IS_OBJECT (&argv[0])) {
+  if (SWFDEC_AS_VALUE_IS_OBJECT (argv[0])) {
 
-    array = SWFDEC_AS_VALUE_GET_OBJECT (&argv[0]);
-    if (!SWFDEC_IS_AS_ARRAY (array)) {
+    array = SWFDEC_AS_VALUE_GET_OBJECT (argv[0]);
+    if (!array->array) {
       SWFDEC_AS_VALUE_SET_OBJECT (ret, object);
       return;
     }
 
-    num_fields = swfdec_as_array_get_length (SWFDEC_AS_ARRAY (array));
+    num_fields = swfdec_as_array_get_length (array);
     if (num_fields <= 0) {
       SWFDEC_AS_VALUE_SET_OBJECT (ret, object);
       return;
@@ -1310,41 +1226,38 @@ swfdec_as_array_sortOn (SwfdecAsContext *cx, SwfdecAsObject *object,
 
     fields = g_new (const char *, num_fields + 1);
     for (i = 0; i < num_fields; i++) {
-      swfdec_as_array_get_value (SWFDEC_AS_ARRAY (array), i, &val);
-      if (SWFDEC_AS_VALUE_IS_OBJECT (&val) &&
-	  SWFDEC_IS_AS_STRING (SWFDEC_AS_VALUE_GET_OBJECT (&val))) {
-	fields[i] =
-	  SWFDEC_AS_STRING (SWFDEC_AS_VALUE_GET_OBJECT (&val))->string;
-      } else {
-	fields[i] = swfdec_as_value_to_string (cx, &val);
-      }
+      swfdec_as_array_get_value (array, i, &val);
+      fields[i] = swfdec_as_value_to_string (cx, val);
     }
 
     fields[i] = NULL;
   } else {
+    if (SWFDEC_AS_VALUE_IS_MOVIE (argv[0])) {
+      SWFDEC_FIXME ("how do we treat movies here?");
+    }
     num_fields = 1;
     fields = g_new (const char *, num_fields + 1);
-    fields[0] = swfdec_as_value_to_string (cx, &argv[0]);
+    fields[0] = swfdec_as_value_to_string (cx, argv[0]);
     fields[1] = NULL;
   }
 
   options = g_new0 (SortOption, num_fields);
 
   if (argc > 1) {
-    if (SWFDEC_AS_VALUE_IS_OBJECT (&argv[1])) {
-      array = SWFDEC_AS_VALUE_GET_OBJECT (&argv[1]);
+    if (SWFDEC_AS_VALUE_IS_OBJECT (argv[1])) {
+      array = SWFDEC_AS_VALUE_GET_OBJECT (argv[1]);
 
-      if (SWFDEC_IS_AS_ARRAY (array) &&
-	swfdec_as_array_get_length (SWFDEC_AS_ARRAY (array)) == num_fields) {
+      if (array->array &&
+	swfdec_as_array_get_length (array) == num_fields) {
 	for (i = 0; i < num_fields; i++) {
-	  swfdec_as_array_get_value (SWFDEC_AS_ARRAY (array), i, &val);
+	  swfdec_as_array_get_value (array, i, &val);
 	  options[i] =
-	    swfdec_as_value_to_integer (cx, &val) & MASK_SORT_OPTION;
+	    swfdec_as_value_to_integer (cx, val) & MASK_SORT_OPTION;
 	}
       }
     } else {
       options[0] =
-	swfdec_as_value_to_integer (cx, &argv[1]) & MASK_SORT_OPTION;
+	swfdec_as_value_to_integer (cx, argv[1]) & MASK_SORT_OPTION;
       for (i = 1; i < num_fields; i++) {
 	options[i] = options[0];
       }
@@ -1359,26 +1272,25 @@ swfdec_as_array_sortOn (SwfdecAsContext *cx, SwfdecAsObject *object,
 
 // Constructor
 
-SWFDEC_AS_CONSTRUCTOR (252, 0, swfdec_as_array_construct, swfdec_as_array_get_type)
+SWFDEC_AS_NATIVE (252, 0, swfdec_as_array_construct)
 void
 swfdec_as_array_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  SwfdecAsArray *array;
-
-  if (!cx->frame->construct) {
-    object = g_object_new (SWFDEC_TYPE_AS_ARRAY, "context", cx, NULL);
+  if (!swfdec_as_context_is_constructing (cx)) {
+    object = swfdec_as_object_new (cx, NULL);
     swfdec_as_object_set_constructor_by_name (object, SWFDEC_AS_STR_Array, NULL);
   }
+  swfdec_as_object_set_relay (object, NULL);
+  object->array = TRUE;
 
-  array = SWFDEC_AS_ARRAY (object);
-  if (argc == 1 && SWFDEC_AS_VALUE_IS_NUMBER (&argv[0])) {
-    int l = swfdec_as_value_to_integer (cx, &argv[0]);
-    swfdec_as_array_set_length (array, l < 0 ? 0 : l);
+  if (argc == 1 && SWFDEC_AS_VALUE_IS_NUMBER (argv[0])) {
+    int l = swfdec_as_value_to_integer (cx, argv[0]);
+    swfdec_as_array_set_length (object, l < 0 ? 0 : l);
   } else if (argc > 0) {
-    swfdec_as_array_append (array, argc, argv);
+    swfdec_as_array_append (object, argc, argv);
   } else {
-    swfdec_as_array_set_length (array, 0);
+    swfdec_as_array_set_length (object, 0);
   }
 
   SWFDEC_AS_VALUE_SET_OBJECT (ret, object);
