@@ -28,9 +28,11 @@
 #include "swfdec_as_array.h"
 #include "swfdec_as_frame_internal.h"
 #include "swfdec_as_function.h"
+#include "swfdec_as_gcable.h"
 #include "swfdec_as_initialize.h"
 #include "swfdec_as_internal.h"
 #include "swfdec_as_interpret.h"
+#include "swfdec_as_movie_value.h"
 #include "swfdec_as_native_function.h"
 #include "swfdec_as_object.h"
 #include "swfdec_as_stack.h"
@@ -248,59 +250,75 @@ swfdec_as_context_unuse_mem (SwfdecAsContext *context, gsize bytes)
 
 /*** GC ***/
 
-static gboolean
-swfdec_as_context_remove_strings (gpointer key, gpointer value, gpointer data)
+static void
+swfdec_as_context_remove_gc_objects (SwfdecAsContext *context)
 {
-  SwfdecAsContext *context = data;
-  char *string;
+  SwfdecGcObject *gc, *prev, *next;
 
-  string = (char *) value;
-  /* it doesn't matter that rooted strings aren't destroyed, they're constant */
-  if (string[0] & SWFDEC_AS_GC_ROOT) {
-    SWFDEC_LOG ("rooted: %s", (char *) key);
-    return FALSE;
-  } else if (string[0] & SWFDEC_AS_GC_MARK) {
-    SWFDEC_LOG ("marked: %s", (char *) key);
-    string[0] &= ~SWFDEC_AS_GC_MARK;
-    return FALSE;
-  } else {
-    gsize len;
-    SWFDEC_LOG ("deleted: %s", (char *) key);
-    len = (strlen ((char *) key) + 2);
-    swfdec_as_context_unuse_mem (context, len);
-    g_slice_free1 (len, value);
-    return TRUE;
+  prev = NULL;
+  gc = context->gc_objects;
+  while (gc) {
+    next = gc->next;
+    /* we only check for mark here, not root, since this works on destroy, too */
+    if (gc->flags & SWFDEC_AS_GC_MARK) {
+      gc->flags &= ~SWFDEC_AS_GC_MARK;
+      SWFDEC_LOG ("%s: %s %p", (gc->flags & SWFDEC_AS_GC_ROOT) ? "rooted" : "marked",
+	  G_OBJECT_TYPE_NAME (gc), gc);
+      prev = gc;
+    } else {
+      SWFDEC_LOG ("deleted: %s %p", G_OBJECT_TYPE_NAME (gc), gc);
+      g_object_unref (gc);
+      if (prev) {
+	prev->next = next;
+      } else {
+	context->gc_objects = next;
+      }
+    }
+    gc = next;
   }
 }
 
-static gboolean
-swfdec_as_context_remove_objects (gpointer key, gpointer value, gpointer debugger)
+static void
+swfdec_as_context_collect_string (SwfdecAsContext *context, gpointer gc)
 {
-  SwfdecGcObject *gc;
+  SwfdecAsStringValue *string;
 
-  gc = key;
-  /* we only check for mark here, not root, since this works on destroy, too */
-  if (gc->flags & SWFDEC_AS_GC_MARK) {
-    gc->flags &= ~SWFDEC_AS_GC_MARK;
-    SWFDEC_LOG ("%s: %s %p", (gc->flags & SWFDEC_AS_GC_ROOT) ? "rooted" : "marked",
-	G_OBJECT_TYPE_NAME (gc), gc);
-    return FALSE;
-  } else {
-    SWFDEC_LOG ("deleted: %s %p", G_OBJECT_TYPE_NAME (gc), gc);
-    g_object_unref (gc);
-    return TRUE;
+  string = gc;
+  if (!g_hash_table_remove (context->interned_strings, string->string)) {
+    g_assert_not_reached ();
   }
+  swfdec_as_gcable_free (context, gc, sizeof (SwfdecAsStringValue) + string->length + 1);
+}
+
+static void
+swfdec_as_context_collect_double (SwfdecAsContext *context, gpointer gc)
+{
+  swfdec_as_gcable_free (context, gc, sizeof (SwfdecAsDoubleValue));
+}
+
+static void
+swfdec_as_context_collect_movie (SwfdecAsContext *context, gpointer gc)
+{
+  swfdec_as_movie_value_free ((SwfdecAsMovieValue *) gc);
 }
 
 static void
 swfdec_as_context_collect (SwfdecAsContext *context)
 {
-  SWFDEC_INFO (">> collecting garbage");
   /* NB: This functions is called without GC from swfdec_as_context_dispose */
-  g_hash_table_foreach_remove (context->strings, 
-    swfdec_as_context_remove_strings, context);
-  g_hash_table_foreach_remove (context->objects, 
-    swfdec_as_context_remove_objects, context->debugger);
+  SWFDEC_INFO (">> collecting garbage");
+  
+  swfdec_as_context_remove_gc_objects (context);
+
+  context->objects = swfdec_as_gcable_collect (context, context->objects,
+      (SwfdecAsGcableDestroyNotify) swfdec_as_object_free);
+  context->strings = swfdec_as_gcable_collect (context, context->strings,
+      swfdec_as_context_collect_string);
+  context->numbers = swfdec_as_gcable_collect (context, context->numbers,
+      swfdec_as_context_collect_double);
+  context->movies = swfdec_as_gcable_collect (context, context->movies,
+      swfdec_as_context_collect_movie);
+
   SWFDEC_INFO (">> done collecting garbage");
 }
 
@@ -314,13 +332,13 @@ swfdec_as_context_collect (SwfdecAsContext *context)
 void
 swfdec_as_string_mark (const char *string)
 {
-  char *str;
+  SwfdecAsStringValue *value;
 
   g_return_if_fail (string != NULL);
 
-  str = (char *) string - 1;
-  if (*str == 0)
-    *str = SWFDEC_AS_GC_MARK;
+  value = (SwfdecAsStringValue *) (gpointer) ((guint8 *) string - G_STRUCT_OFFSET (SwfdecAsStringValue, string));
+  if (!SWFDEC_AS_GCABLE_FLAG_IS_SET (value, SWFDEC_AS_GC_ROOT))
+    SWFDEC_AS_GCABLE_SET_FLAG (value, SWFDEC_AS_GC_MARK);
 }
 
 /**
@@ -334,24 +352,37 @@ swfdec_as_string_mark (const char *string)
 void
 swfdec_as_value_mark (SwfdecAsValue *value)
 {
-  g_return_if_fail (SWFDEC_IS_AS_VALUE (value));
+  SwfdecAsGcable *gcable;
+  SwfdecAsValueType type = SWFDEC_AS_VALUE_GET_TYPE (*value);
+  
+  if (!SWFDEC_AS_TYPE_IS_GCABLE (type))
+    return;
 
-  if (SWFDEC_AS_VALUE_IS_OBJECT (value)) {
-    swfdec_gc_object_mark (value->value.object);
-  } else if (SWFDEC_AS_VALUE_IS_STRING (value)) {
-    swfdec_as_string_mark (SWFDEC_AS_VALUE_GET_STRING (value));
+  gcable = SWFDEC_AS_VALUE_GET_VALUE (*value);
+  if (SWFDEC_AS_GCABLE_FLAG_IS_SET (gcable, SWFDEC_AS_GC_ROOT | SWFDEC_AS_GC_MARK))
+    return;
+
+  switch (type) {
+    case SWFDEC_AS_TYPE_STRING:
+    case SWFDEC_AS_TYPE_NUMBER:
+      SWFDEC_AS_GCABLE_SET_FLAG (gcable, SWFDEC_AS_GC_MARK);
+      break;
+    case SWFDEC_AS_TYPE_OBJECT:
+      swfdec_as_object_mark ((SwfdecAsObject *) gcable);
+      break;
+    case SWFDEC_AS_TYPE_MOVIE:
+      swfdec_as_movie_value_mark ((SwfdecAsMovieValue *) gcable);
+      break;
+    case SWFDEC_AS_TYPE_UNDEFINED:
+    case SWFDEC_AS_TYPE_NULL:
+    case SWFDEC_AS_TYPE_BOOLEAN:
+    case SWFDEC_AS_TYPE_INT:
+    default:
+      g_assert_not_reached ();
+      break;
   }
 }
 
-static void
-swfdec_as_context_mark_roots (gpointer key, gpointer value, gpointer data)
-{
-  SwfdecGcObject *object = key;
-
-  if ((object->flags & (SWFDEC_AS_GC_MARK | SWFDEC_AS_GC_ROOT)) == SWFDEC_AS_GC_ROOT)
-    swfdec_gc_object_mark (object);
-}
-
 /* FIXME: replace this with refcounted strings? */
 static void 
 swfdec_as_context_mark_constant_pools (gpointer key, gpointer value, gpointer unused)
@@ -370,16 +401,10 @@ static void
 swfdec_as_context_do_mark (SwfdecAsContext *context)
 {
   /* This if is needed for SwfdecPlayer */
-  if (context->global) {
-    swfdec_gc_object_mark (context->global);
-    swfdec_gc_object_mark (context->Function);
-    swfdec_gc_object_mark (context->Function_prototype);
-    swfdec_gc_object_mark (context->Object);
-    swfdec_gc_object_mark (context->Object_prototype);
-  }
+  if (context->global)
+    swfdec_as_object_mark (context->global);
   if (context->exception)
     swfdec_as_value_mark (&context->exception_value);
-  g_hash_table_foreach (context->objects, swfdec_as_context_mark_roots, NULL);
   g_hash_table_foreach (context->constant_pools, swfdec_as_context_mark_constant_pools, NULL);
 }
 
@@ -513,11 +538,12 @@ swfdec_as_context_dispose (GObject *object)
   if (context->memory != 0) {
     g_critical ("%zu bytes of memory left over\n", context->memory);
   }
-  g_assert (g_hash_table_size (context->objects) == 0);
+  g_assert (context->strings == NULL);
+  g_assert (context->numbers == NULL);
   g_assert (g_hash_table_size (context->constant_pools) == 0);
+  g_assert (context->gc_objects == 0);
   g_hash_table_destroy (context->constant_pools);
-  g_hash_table_destroy (context->objects);
-  g_hash_table_destroy (context->strings);
+  g_hash_table_destroy (context->interned_strings);
   g_rand_free (context->rand);
   if (context->debugger) {
     g_object_unref (context->debugger);
@@ -569,18 +595,16 @@ swfdec_as_context_class_init (SwfdecAsContextClass *klass)
 static void
 swfdec_as_context_init (SwfdecAsContext *context)
 {
-  const char *s;
+  const SwfdecAsConstantStringValue *s;
 
   context->version = G_MAXUINT;
 
-  context->strings = g_hash_table_new (g_str_hash, g_str_equal);
-  context->objects = g_hash_table_new (g_direct_hash, g_direct_equal);
+  context->interned_strings = g_hash_table_new (g_str_hash, g_str_equal);
   context->constant_pools = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-  for (s = swfdec_as_strings; *s == '\2'; s += strlen (s) + 1) {
-    g_hash_table_insert (context->strings, (char *) s + 1, (char *) s);
+  for (s = swfdec_as_strings; s->next; s++) {
+    g_hash_table_insert (context->interned_strings, (gpointer) s->string, (gpointer) s);
   }
-  g_assert (*s == 0);
   context->rand = g_rand_new ();
   g_get_current_time (&context->start_time);
 }
@@ -590,20 +614,16 @@ swfdec_as_context_init (SwfdecAsContext *context)
 static const char *
 swfdec_as_context_create_string (SwfdecAsContext *context, const char *string, gsize len)
 {
-  char *new;
-  
-  if (!swfdec_as_context_try_use_mem (context, sizeof (char) * (2 + len))) {
-    swfdec_as_context_abort (context, "Out of memory");
-    return SWFDEC_AS_STR_EMPTY;
-  }
+  SwfdecAsStringValue *new;
 
-  new = g_slice_alloc (2 + len);
-  memcpy (&new[1], string, len);
-  new[len + 1] = 0;
-  new[0] = 0; /* GC flags */
-  g_hash_table_insert (context->strings, new + 1, new);
+  new = swfdec_as_gcable_alloc (context, sizeof (SwfdecAsStringValue) + len + 1);
+  new->length = len;
+  memcpy (new->string, string, new->length + 1);
+  g_hash_table_insert (context->interned_strings, new->string, new);
+  SWFDEC_AS_GCABLE_SET_NEXT (new, context->strings);
+  context->strings = new;
 
-  return new + 1;
+  return new->string;
 }
 
 /**
@@ -620,14 +640,15 @@ swfdec_as_context_create_string (SwfdecAsContext *context, const char *string, g
 const char *
 swfdec_as_context_get_string (SwfdecAsContext *context, const char *string)
 {
-  const char *ret;
+  const SwfdecAsStringValue *ret;
   gsize len;
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
   g_return_val_if_fail (string != NULL, NULL);
 
-  if (g_hash_table_lookup_extended (context->strings, string, (gpointer) &ret, NULL))
-    return ret;
+  ret = g_hash_table_lookup (context->interned_strings, string);
+  if (ret)
+    return ret->string;
 
   len = strlen (string);
   return swfdec_as_context_create_string (context, string, len);
@@ -682,14 +703,7 @@ swfdec_as_context_is_constructing (SwfdecAsContext *context)
  * @context: a #SwfdecAsContext
  *
  * This is a debugging function. It gets the topmost stack frame that is 
- * currently executing. If no function is executing, %NULL is returned. You can
- * easily get a backtrace with code like this:
- * |[for (frame = swfdec_as_context_get_frame (context); frame != NULL; 
- *     frame = swfdec_as_frame_get_next (frame)) {
- *   char *s = swfdec_as_object_get_debug (SWFDEC_AS_OBJECT (frame));
- *   g_print ("%s\n", s);
- *   g_free (s);
- * }]|
+ * currently executing. If no function is executing, %NULL is returned.
  *
  * Returns: the currently executing frame or %NULL if none
  **/
@@ -713,7 +727,6 @@ void
 swfdec_as_context_throw (SwfdecAsContext *context, const SwfdecAsValue *value)
 {
   g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
-  g_return_if_fail (SWFDEC_IS_AS_VALUE (value));
   g_return_if_fail (!context->exception);
 
   context->exception = TRUE;
@@ -771,6 +784,84 @@ swfdec_as_context_get_time (SwfdecAsContext *context, GTimeVal *tv)
     g_get_current_time (tv);
 }
 
+/**
+ * swfdec_as_context_return:
+ * @context: the context to return the topmost frame in
+ * @return_value: return value of the function or %NULL for none. An undefined
+ *                value will be used in that case.
+ *
+ * Ends execution of the currently executing frame and continues execution with 
+ * its parent frame.
+ **/
+void
+swfdec_as_context_return (SwfdecAsContext *context, SwfdecAsValue *return_value)
+{
+  SwfdecAsValue retval;
+  SwfdecAsFrame *frame, *next;
+
+  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
+  g_return_if_fail (context->frame != NULL);
+
+  frame = context->frame;
+
+  /* save return value in case it was on the stack somewhere */
+  if (frame->construct) {
+    retval = frame->thisp;
+  } else if (return_value) {
+    retval = *return_value;
+  } else {
+    SWFDEC_AS_VALUE_SET_UNDEFINED (&retval);
+  }
+  /* pop frame and leftover stack */
+  next = frame->next;
+  context->frame = next;
+  g_assert (context->call_depth > 0);
+  context->call_depth--;
+  while (context->base > frame->stack_begin || 
+      context->end < frame->stack_begin)
+    swfdec_as_stack_pop_segment (context);
+  context->cur = frame->stack_begin;
+  /* setup stack for previous frame */
+  if (next) {
+    if (next->stack_begin >= &context->stack->elements[0] &&
+	next->stack_begin <= context->cur) {
+      context->base = next->stack_begin;
+    } else {
+      context->base = &context->stack->elements[0];
+    }
+  } else {
+    g_assert (context->stack->next == NULL);
+    context->base = &context->stack->elements[0];
+  }
+  /* pop argv if on stack */
+  if (frame->argv == NULL && frame->argc > 0) {
+    guint i = frame->argc;
+    while (TRUE) {
+      guint n = context->cur - context->base;
+      n = MIN (n, i);
+      swfdec_as_stack_pop_n (context, n);
+      i -= n;
+      if (i == 0)
+	break;
+      swfdec_as_stack_pop_segment (context);
+    }
+  }
+  if (context->debugger) {
+    SwfdecAsDebuggerClass *klass = SWFDEC_AS_DEBUGGER_GET_CLASS (context->debugger);
+
+    if (klass->leave_frame)
+      klass->leave_frame (context->debugger, context, frame, &retval);
+  }
+  /* set return value */
+  if (frame->return_value) {
+    *frame->return_value = retval;
+  } else {
+    swfdec_as_stack_ensure_free (context, 1);
+    *swfdec_as_stack_push (context) = retval;
+  }
+  swfdec_as_frame_free (context, frame);
+}
+
 /**
  * swfdec_as_context_run:
  * @context: a #SwfdecAsContext
@@ -826,7 +917,6 @@ swfdec_as_context_run (SwfdecAsContext *context)
     step = NULL;
   }
 
-  g_assert (frame->target);
   script = frame->script;
   context->version = script->version;
   startpc = script->buffer->data;
@@ -837,14 +927,14 @@ swfdec_as_context_run (SwfdecAsContext *context)
 
   while (context->state < SWFDEC_AS_CONTEXT_ABORTED) {
     if (context->exception) {
-      swfdec_as_frame_handle_exception (frame);
+      swfdec_as_frame_handle_exception (context, frame);
       if (frame != context->frame)
 	goto out;
       pc = frame->pc;
       continue;
     }
     if (pc == exitpc) {
-      swfdec_as_frame_return (frame, NULL);
+      swfdec_as_context_return (context, NULL);
       goto out;
     }
     if (pc < startpc || pc >= endpc) {
@@ -951,7 +1041,7 @@ swfdec_as_context_run (SwfdecAsContext *context)
 
 error:
   if (context->frame == frame)
-    swfdec_as_frame_return (frame, NULL);
+    swfdec_as_context_return (context, NULL);
 out:
   context->version = original_version;
   return;
@@ -992,11 +1082,11 @@ swfdec_as_context_ASSetPropFlags (SwfdecAsContext *cx, SwfdecAsObject *object,
   if (argc < 3)
     return;
 
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&argv[0]))
+  if (!SWFDEC_AS_VALUE_IS_COMPOSITE (argv[0]))
     return;
-  obj = SWFDEC_AS_VALUE_GET_OBJECT (&argv[0]);
-  flags[0] = swfdec_as_value_to_integer (cx, &argv[2]);
-  flags[1] = (argc > 3) ? swfdec_as_value_to_integer (cx, &argv[3]) : 0;
+  obj = SWFDEC_AS_VALUE_GET_COMPOSITE (argv[0]);
+  flags[0] = swfdec_as_value_to_integer (cx, argv[2]);
+  flags[1] = (argc > 3) ? swfdec_as_value_to_integer (cx, argv[3]) : 0;
 
   if (flags[0] == 0 && flags[1] == 0) {
     // we should add autosizing length attribute here
@@ -1004,11 +1094,11 @@ swfdec_as_context_ASSetPropFlags (SwfdecAsContext *cx, SwfdecAsObject *object,
     return;
   }
 
-  if (SWFDEC_AS_VALUE_IS_NULL (&argv[1])) {
+  if (SWFDEC_AS_VALUE_IS_NULL (argv[1])) {
     swfdec_as_object_foreach (obj, swfdec_as_context_ASSetPropFlags_foreach, flags);
   } else {
     char **split =
-      g_strsplit (swfdec_as_value_to_string (cx, &argv[1]), ",", -1);
+      g_strsplit (swfdec_as_value_to_string (cx, argv[1]), ",", -1);
     guint i;
     for (i = 0; split[i]; i++) {
       swfdec_as_context_ASSetPropFlags_set_one_flag (obj, 
@@ -1028,7 +1118,7 @@ swfdec_as_context_isFinite (SwfdecAsContext *cx, SwfdecAsObject *object,
   if (argc < 1)
     return;
 
-  d = swfdec_as_value_to_number (cx, &argv[0]);
+  d = swfdec_as_value_to_number (cx, argv[0]);
   SWFDEC_AS_VALUE_SET_BOOLEAN (retval, isfinite (d) ? TRUE : FALSE);
 }
 
@@ -1042,7 +1132,7 @@ swfdec_as_context_isNaN (SwfdecAsContext *cx, SwfdecAsObject *object,
   if (argc < 1)
     return;
 
-  d = swfdec_as_value_to_number (cx, &argv[0]);
+  d = swfdec_as_value_to_number (cx, argv[0]);
   SWFDEC_AS_VALUE_SET_BOOLEAN (retval, isnan (d) ? TRUE : FALSE);
 }
 
@@ -1059,14 +1149,14 @@ swfdec_as_context_parseInt (SwfdecAsContext *cx, SwfdecAsObject *object,
   SWFDEC_AS_CHECK (0, NULL, "s|i", &s, &radix);
 
   if (argc >= 2 && (radix < 2 || radix > 36)) {
-    SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
+    *retval = swfdec_as_value_from_number (cx, NAN);
     return;
   }
 
   // special case, don't allow sign in front of the 0x
   if ((s[0] == '-' || s[0] == '+') && s[1] == '0' &&
       (s[2] == 'x' || s[2] == 'X')) {
-    SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
+    *retval = swfdec_as_value_from_number (cx, NAN);
     return;
   }
 
@@ -1092,7 +1182,7 @@ swfdec_as_context_parseInt (SwfdecAsContext *cx, SwfdecAsObject *object,
     if (skip != s && (skip[0] == '-' || skip[0] == '+'))
       skip++;
     if (skip != s && skip[0] == '0' && (skip[1] == 'x' || skip[1] == 'X')) {
-      SWFDEC_AS_VALUE_SET_NUMBER (retval, 0);
+      *retval = swfdec_as_value_from_number (cx, 0);
       return;
     }
   }
@@ -1100,14 +1190,14 @@ swfdec_as_context_parseInt (SwfdecAsContext *cx, SwfdecAsObject *object,
   i = g_ascii_strtoll (s, &tail, radix);
 
   if (tail == s) {
-    SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
+    *retval = swfdec_as_value_from_number (cx, NAN);
     return;
   }
 
   if (i > G_MAXINT32 || i < G_MININT32) {
-    SWFDEC_AS_VALUE_SET_NUMBER (retval, i);
+    *retval = swfdec_as_value_from_number (cx, i);
   } else {
-    SWFDEC_AS_VALUE_SET_INT (retval, i);
+    *retval = swfdec_as_value_from_integer (cx, i);
   }
 }
 
@@ -1124,7 +1214,7 @@ swfdec_as_context_parseFloat (SwfdecAsContext *cx, SwfdecAsObject *object,
 
   // we need to remove everything after x or I, since strtod parses hexadecimal
   // numbers and Infinity
-  s = g_strdup (swfdec_as_value_to_string (cx, &argv[0]));
+  s = g_strdup (swfdec_as_value_to_string (cx, argv[0]));
   if ((p = strpbrk (s, "xXiI")) != NULL) {
     *p = '\0';
   }
@@ -1132,9 +1222,9 @@ swfdec_as_context_parseFloat (SwfdecAsContext *cx, SwfdecAsObject *object,
   d = g_ascii_strtod (s, &tail);
 
   if (tail == s) {
-    SWFDEC_AS_VALUE_SET_NUMBER (retval, NAN);
+    *retval = swfdec_as_value_from_number (cx, NAN);
   } else {
-    SWFDEC_AS_VALUE_SET_NUMBER (retval, d);
+    *retval = swfdec_as_value_from_number (cx, d);
   }
 
   g_free (s);
@@ -1145,9 +1235,9 @@ swfdec_as_context_init_global (SwfdecAsContext *context)
 {
   SwfdecAsValue val;
 
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, NAN);
+  val = swfdec_as_value_from_number (context, NAN);
   swfdec_as_object_set_variable (context->global, SWFDEC_AS_STR_NaN, &val);
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, HUGE_VAL);
+  val = swfdec_as_value_from_number (context, HUGE_VAL);
   swfdec_as_object_set_variable (context->global, SWFDEC_AS_STR_Infinity, &val);
 }
 
@@ -1197,7 +1287,6 @@ swfdec_as_context_startup (SwfdecAsContext *context)
   /* FIXME: remove them for normal contexts? */
   swfdec_player_preinit_global (context);
   /* get the necessary objects up to define objects and functions sanely */
-  swfdec_as_function_init_context (context);
   swfdec_as_object_init_context (context);
   /* define the global object and other important ones */
   swfdec_as_context_init_global (context);
