@@ -25,6 +25,7 @@
 #include "swfdec_as_frame.h"
 #include "swfdec_as_array.h"
 #include "swfdec_as_context.h"
+#include "swfdec_as_internal.h"
 #include "swfdec_as_stack.h"
 #include "swfdec_as_strings.h"
 #include "swfdec_as_super.h"
@@ -66,6 +67,7 @@
 /**
  * swfdec_as_stack_iterator_init_arguments:
  * @iter: iterator to be initialized
+ * @context: context @frame belongs to
  * @frame: the frame to initialize from
  *
  * Initializes a stack iterator to walk the arguments passed to the given @frame. See
@@ -74,11 +76,11 @@
  * Returns: The value of the first argument
  **/
 SwfdecAsValue *
-swfdec_as_stack_iterator_init_arguments (SwfdecAsStackIterator *iter, SwfdecAsFrame *frame)
+swfdec_as_stack_iterator_init_arguments (SwfdecAsStackIterator *iter, 
+    SwfdecAsContext *context, SwfdecAsFrame *frame)
 {
-  SwfdecAsContext *context;
-
   g_return_val_if_fail (iter != NULL, NULL);
+  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
   g_return_val_if_fail (frame != NULL, NULL);
 
   if (frame->argc == 0) {
@@ -87,7 +89,6 @@ swfdec_as_stack_iterator_init_arguments (SwfdecAsStackIterator *iter, SwfdecAsFr
     iter->current = NULL;
     return NULL;
   }
-  context = swfdec_gc_object_get_context (frame->target);
   if (frame->argv) {
     iter->stack = NULL;
     iter->current = (SwfdecAsValue *) frame->argv;
@@ -111,6 +112,7 @@ swfdec_as_stack_iterator_init_arguments (SwfdecAsStackIterator *iter, SwfdecAsFr
 /**
  * swfdec_as_stack_iterator_init:
  * @iter: a #SwfdecStackIterator
+ * @context: context @frame belongs to
  * @frame: the frame to initialize from
  *
  * Initializes @iter to walk the stack of @frame. The first value on the stack
@@ -126,15 +128,13 @@ swfdec_as_stack_iterator_init_arguments (SwfdecAsStackIterator *iter, SwfdecAsFr
  * Returns: the topmost value on the stack of @frame or %NULL if none
  **/
 SwfdecAsValue *
-swfdec_as_stack_iterator_init (SwfdecAsStackIterator *iter, SwfdecAsFrame *frame)
+swfdec_as_stack_iterator_init (SwfdecAsStackIterator *iter, SwfdecAsContext *context, SwfdecAsFrame *frame)
 {
-  SwfdecAsContext *context;
   SwfdecAsStack *stack;
 
   g_return_val_if_fail (iter != NULL, NULL);
   g_return_val_if_fail (frame != NULL, NULL);
 
-  context = swfdec_gc_object_get_context (frame->target);
   iter->i = 0;
   stack = context->stack;
   if (context->frame == frame) {
@@ -265,12 +265,12 @@ swfdec_as_frame_pop_block (SwfdecAsFrame *frame, SwfdecAsContext *cx)
 
 /*** FRAME ***/
 
-static void
-swfdec_as_frame_free (SwfdecAsFrame *frame)
+void
+swfdec_as_frame_free (SwfdecAsContext *context, SwfdecAsFrame *frame)
 {
   /* pop blocks while state is intact */
   while (frame->blocks->len > 0)
-    swfdec_as_frame_pop_block (frame, swfdec_gc_object_get_context (frame->target));
+    swfdec_as_frame_pop_block (frame, context);
 
   /* clean up */
   g_slice_free1 (sizeof (SwfdecAsValue) * frame->n_registers, frame->registers);
@@ -319,89 +319,14 @@ swfdec_as_frame_init_native (SwfdecAsFrame *frame, SwfdecAsContext *context)
   frame->stack_begin = context->cur;
   context->base = frame->stack_begin;
   frame->next = context->frame;
+  if (frame->next) {
+    frame->original_target = frame->next->original_target;
+    frame->target = frame->original_target;
+  }
   context->frame = frame;
   context->call_depth++;
 }
 
-/**
- * swfdec_as_frame_return:
- * @frame: a #SwfdecAsFrame that is currently executing.
- * @return_value: return value of the function or %NULL for none. An undefined
- *                value will be used in that case.
- *
- * Ends execution of the frame and instructs the frame's context to continue 
- * execution with its parent frame. This function may only be called on the
- * currently executing frame.
- **/
-void
-swfdec_as_frame_return (SwfdecAsFrame *frame, SwfdecAsValue *return_value)
-{
-  SwfdecAsContext *context;
-  SwfdecAsValue retval;
-  SwfdecAsFrame *next;
-
-  g_return_if_fail (frame != NULL);
-  context = swfdec_gc_object_get_context (frame->target ? (gpointer) frame->target : (gpointer) frame->function);
-  g_return_if_fail (frame == context->frame);
-
-  /* save return value in case it was on the stack somewhere */
-  if (frame->construct) {
-    SWFDEC_AS_VALUE_SET_OBJECT (&retval, frame->thisp);
-  } else if (return_value) {
-    retval = *return_value;
-  } else {
-    SWFDEC_AS_VALUE_SET_UNDEFINED (&retval);
-  }
-  /* pop frame and leftover stack */
-  next = frame->next;
-  context->frame = next;
-  g_assert (context->call_depth > 0);
-  context->call_depth--;
-  while (context->base > frame->stack_begin || 
-      context->end < frame->stack_begin)
-    swfdec_as_stack_pop_segment (context);
-  context->cur = frame->stack_begin;
-  /* setup stack for previous frame */
-  if (next) {
-    if (next->stack_begin >= &context->stack->elements[0] &&
-	next->stack_begin <= context->cur) {
-      context->base = next->stack_begin;
-    } else {
-      context->base = &context->stack->elements[0];
-    }
-  } else {
-    g_assert (context->stack->next == NULL);
-    context->base = &context->stack->elements[0];
-  }
-  /* pop argv if on stack */
-  if (frame->argv == NULL && frame->argc > 0) {
-    guint i = frame->argc;
-    while (TRUE) {
-      guint n = context->cur - context->base;
-      n = MIN (n, i);
-      swfdec_as_stack_pop_n (context, n);
-      i -= n;
-      if (i == 0)
-	break;
-      swfdec_as_stack_pop_segment (context);
-    }
-  }
-  if (context->debugger) {
-    SwfdecAsDebuggerClass *klass = SWFDEC_AS_DEBUGGER_GET_CLASS (context->debugger);
-
-    if (klass->leave_frame)
-      klass->leave_frame (context->debugger, context, frame, &retval);
-  }
-  /* set return value */
-  if (frame->return_value) {
-    *frame->return_value = retval;
-  } else {
-    swfdec_as_stack_ensure_free (context, 1);
-    *swfdec_as_stack_push (context) = retval;
-  }
-  swfdec_as_frame_free (frame);
-}
-
 /**
  * swfdec_as_frame_set_this:
  * @frame: a #SwfdecAsFrame
@@ -416,15 +341,11 @@ void
 swfdec_as_frame_set_this (SwfdecAsFrame *frame, SwfdecAsObject *thisp)
 {
   g_return_if_fail (frame != NULL);
-  g_return_if_fail (frame->thisp == NULL);
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (thisp));
+  g_return_if_fail (SWFDEC_AS_VALUE_IS_UNDEFINED (frame->thisp));
+  g_return_if_fail (thisp != NULL);
 
   g_assert (!SWFDEC_IS_AS_SUPER (thisp));
-  frame->thisp = thisp;
-  if (frame->target == NULL) {
-    frame->target = thisp;
-    frame->original_target = thisp;
-  }
+  SWFDEC_AS_VALUE_SET_COMPOSITE (&frame->thisp, thisp);
 }
 
 /**
@@ -444,36 +365,30 @@ swfdec_as_frame_set_this (SwfdecAsFrame *frame, SwfdecAsObject *thisp)
  * Returns: Object in scope chain that contained the variable.
  **/
 SwfdecAsObject *
-swfdec_as_frame_get_variable_and_flags (SwfdecAsFrame *frame, const char *variable,
-    SwfdecAsValue *value, guint *flags, SwfdecAsObject **pobject)
+swfdec_as_frame_get_variable_and_flags (SwfdecAsContext *cx, SwfdecAsFrame *frame, 
+    const char *variable, SwfdecAsValue *value, guint *flags, SwfdecAsObject **pobject)
 {
+  SwfdecMovie *target;
   GSList *walk;
-  SwfdecAsContext *cx;
 
+  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (cx), NULL);
   g_return_val_if_fail (frame != NULL, NULL);
   g_return_val_if_fail (variable != NULL, NULL);
 
-  cx = swfdec_gc_object_get_context (frame->target);
-
   for (walk = frame->scope_chain; walk; walk = walk->next) {
     if (swfdec_as_object_get_variable_and_flags (walk->data, variable, value, 
 	  flags, pobject))
       return walk->data;
   }
   /* we've walked the scope chain down. Now look in the special objects. */
-  /* 1) the target (if removed, use original target) */
-  if (SWFDEC_IS_MOVIE (frame->target) &&
-      SWFDEC_MOVIE(frame->target)->state < SWFDEC_MOVIE_STATE_DESTROYED) {
-    if (swfdec_as_object_get_variable_and_flags (frame->target, variable,
-	  value, flags, pobject))
-      return frame->target;
-  } else {
-    if (swfdec_as_object_get_variable_and_flags (frame->original_target,
-	  variable, value, flags, pobject))
-      return frame->original_target;
+  /* 1) the current target */
+  target = swfdec_as_frame_get_target (frame);
+  if (target) {
+    SwfdecAsObject *object = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (target));
+    if (swfdec_as_object_get_variable_and_flags (object, variable, value, flags, pobject))
+      return object;
   }
   /* 2) the global object */
-  /* FIXME: ignored on version 4, but should it never be created instead? */
   if (cx->version > 4 && swfdec_as_object_get_variable_and_flags (cx->global,
 	variable, value, flags, pobject))
     return cx->global;
@@ -482,8 +397,9 @@ swfdec_as_frame_get_variable_and_flags (SwfdecAsFrame *frame, const char *variab
 }
 
 void
-swfdec_as_frame_set_variable_and_flags (SwfdecAsFrame *frame, const char *variable,
-    const SwfdecAsValue *value, guint default_flags, gboolean overwrite, gboolean local)
+swfdec_as_frame_set_variable_and_flags (SwfdecAsContext *context, SwfdecAsFrame *frame, 
+    const char *variable, const SwfdecAsValue *value, guint default_flags, 
+    gboolean overwrite, gboolean local)
 {
   SwfdecAsObject *pobject, *set;
   GSList *walk;
@@ -502,11 +418,18 @@ swfdec_as_frame_set_variable_and_flags (SwfdecAsFrame *frame, const char *variab
     }
   }
   if (set == NULL) {
-    if (local && frame->activation) {
+    /* the !frame->original_target check is exclusively for init scripts */
+    if (frame->activation && (local || !frame->original_target)) {
       set = frame->activation;
     } else {
-      set = frame->target;
+      SwfdecMovie *target = swfdec_as_frame_get_target (frame);
+      if (target)
+	set = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (target));
+      else
+	set = NULL;
     }
+    if (set == NULL)
+      return;
   }
 
   if (!overwrite) {
@@ -518,10 +441,11 @@ swfdec_as_frame_set_variable_and_flags (SwfdecAsFrame *frame, const char *variab
 }
 
 SwfdecAsDeleteReturn
-swfdec_as_frame_delete_variable (SwfdecAsFrame *frame, const char *variable)
+swfdec_as_frame_delete_variable (SwfdecAsContext *cx, SwfdecAsFrame *frame, const char *variable)
 {
   GSList *walk;
   SwfdecAsDeleteReturn ret;
+  SwfdecMovie *target;
 
   g_return_val_if_fail (frame != NULL, FALSE);
   g_return_val_if_fail (variable != NULL, FALSE);
@@ -533,27 +457,31 @@ swfdec_as_frame_delete_variable (SwfdecAsFrame *frame, const char *variable)
   }
   /* we've walked the scope chain down. Now look in the special objects. */
   /* 1) the target set via SetTarget */
-  ret = swfdec_as_object_delete_variable (frame->target, variable);
-  if (ret)
-    return ret;
+
+  target = swfdec_as_frame_get_target (frame);
+  if (target) {
+    ret = swfdec_as_object_delete_variable (swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (target)), variable);
+    if (ret)
+      return ret;
+  }
   /* 2) the global object */
-  return swfdec_as_object_delete_variable (swfdec_gc_object_get_context (frame->target)->global, variable);
+  return swfdec_as_object_delete_variable (cx->global, variable);
 }
 
 /**
  * swfdec_as_frame_set_target:
  * @frame: a #SwfdecAsFrame
- * @target: the new object to use as target or %NULL to unset
+ * @target: the movie to use as target or %NULL to unset
  *
  * Sets the new target to be used in this @frame. The target is a legacy 
  * Actionscript concept that is similar to "with". If you don't have to,
  * you shouldn't use this function.
  **/
 void
-swfdec_as_frame_set_target (SwfdecAsFrame *frame, SwfdecAsObject *target)
+swfdec_as_frame_set_target (SwfdecAsFrame *frame, SwfdecMovie *target)
 {
   g_return_if_fail (frame != NULL);
-  g_return_if_fail (target == NULL || SWFDEC_IS_AS_OBJECT (target));
+  g_return_if_fail (target == NULL || SWFDEC_IS_MOVIE (target));
 
   if (target) {
     frame->target = target;
@@ -563,20 +491,19 @@ swfdec_as_frame_set_target (SwfdecAsFrame *frame, SwfdecAsObject *target)
 }
 
 void
-swfdec_as_frame_preload (SwfdecAsFrame *frame)
+swfdec_as_frame_preload (SwfdecAsContext *context, SwfdecAsFrame *frame)
 {
   SwfdecAsObject *object, *args;
   guint i, current_reg = 1;
   SwfdecScript *script;
   SwfdecAsValue val;
   const SwfdecAsValue *cur;
-  SwfdecAsContext *context;
   SwfdecAsStackIterator iter;
 
+  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
   g_return_if_fail (frame != NULL);
 
   /* setup */
-  context = swfdec_gc_object_get_context (frame->target ? (gpointer) frame->target : (gpointer) frame->function);
   script = frame->script;
   if (frame->script == NULL)
     goto out;
@@ -588,9 +515,9 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
   if ((script->flags & (SWFDEC_SCRIPT_PRELOAD_ARGS | SWFDEC_SCRIPT_SUPPRESS_ARGS)) != SWFDEC_SCRIPT_SUPPRESS_ARGS) {
     SwfdecAsFrame *next;
     args = swfdec_as_array_new (context);
-    for (cur = swfdec_as_stack_iterator_init_arguments (&iter, frame); cur != NULL;
+    for (cur = swfdec_as_stack_iterator_init_arguments (&iter, context, frame); cur != NULL;
 	cur = swfdec_as_stack_iterator_next (&iter)) {
-      swfdec_as_array_push (SWFDEC_AS_ARRAY (args), cur);
+      swfdec_as_array_push (args, cur);
     }
 
     next = frame->next;
@@ -599,7 +526,7 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
       next = next->next;
     }
     if (next != NULL) {
-      SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (next->function));
+      SWFDEC_AS_VALUE_SET_OBJECT (&val, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (next->function)));
     } else {
       SWFDEC_AS_VALUE_SET_NULL (&val);
     }
@@ -607,7 +534,7 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
 	SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
 
     if (frame->function != NULL) {
-      SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (frame->function));
+      SWFDEC_AS_VALUE_SET_OBJECT (&val, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (frame->function)));
     } else {
       SWFDEC_AS_VALUE_SET_NULL (&val);
     }
@@ -620,12 +547,7 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
 
   /* set the default variables (unless suppressed */
   if (!(script->flags & SWFDEC_SCRIPT_SUPPRESS_THIS)) {
-    if (frame->thisp) {
-      SWFDEC_AS_VALUE_SET_OBJECT (&val, frame->thisp);
-    } else {
-      SWFDEC_AS_VALUE_SET_UNDEFINED (&val);
-    }
-    swfdec_as_object_set_variable (object, SWFDEC_AS_STR_this, &val);
+    swfdec_as_object_set_variable (object, SWFDEC_AS_STR_this, &frame->thisp);
   }
   if (!(script->flags & SWFDEC_SCRIPT_SUPPRESS_ARGS)) {
     SWFDEC_AS_VALUE_SET_OBJECT (&val, args);
@@ -633,7 +555,7 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
   }
   if (!(script->flags & SWFDEC_SCRIPT_SUPPRESS_SUPER)) {
     if (frame->super) {
-      SWFDEC_AS_VALUE_SET_OBJECT (&val, frame->super);
+      SWFDEC_AS_VALUE_SET_OBJECT (&val, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (frame->super)));
     } else {
       SWFDEC_AS_VALUE_SET_UNDEFINED (&val);
     }
@@ -642,7 +564,7 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
 
   /* set and preload argument variables */
   SWFDEC_AS_VALUE_SET_UNDEFINED (&val);
-  cur = swfdec_as_stack_iterator_init_arguments (&iter, frame);
+  cur = swfdec_as_stack_iterator_init_arguments (&iter, context, frame);
   for (i = 0; i < script->n_arguments; i++) {
     if (cur == NULL)
       cur = &val;
@@ -665,30 +587,27 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
   /* preload from flags */
   if ((script->flags & (SWFDEC_SCRIPT_PRELOAD_THIS | SWFDEC_SCRIPT_SUPPRESS_THIS)) == SWFDEC_SCRIPT_PRELOAD_THIS
       && current_reg < script->n_registers) {
-    if (frame->thisp) {
-      SWFDEC_AS_VALUE_SET_OBJECT (&frame->registers[current_reg++], frame->thisp);
-    } else {
-      SWFDEC_AS_VALUE_SET_UNDEFINED (&frame->registers[current_reg++]);
-    }
+    frame->registers[current_reg++] = frame->thisp;
   }
   if (script->flags & SWFDEC_SCRIPT_PRELOAD_ARGS && current_reg < script->n_registers) {
     SWFDEC_AS_VALUE_SET_OBJECT (&frame->registers[current_reg++], args);
   }
   if (script->flags & SWFDEC_SCRIPT_PRELOAD_SUPER && current_reg < script->n_registers) {
     if (frame->super) {
-      SWFDEC_AS_VALUE_SET_OBJECT (&frame->registers[current_reg++], frame->super);
+      SWFDEC_AS_VALUE_SET_OBJECT (&frame->registers[current_reg++], 
+	  swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (frame->super)));
     } else {
       SWFDEC_AS_VALUE_SET_UNDEFINED (&frame->registers[current_reg++]);
     }
   }
   if (script->flags & SWFDEC_SCRIPT_PRELOAD_ROOT && current_reg < script->n_registers) {
-    if (!swfdec_as_frame_get_variable (frame, SWFDEC_AS_STR__root, &frame->registers[current_reg])) {
+    if (!swfdec_as_frame_get_variable (context, frame, SWFDEC_AS_STR__root, &frame->registers[current_reg])) {
       SWFDEC_WARNING ("no root to preload");
     }
     current_reg++;
   }
   if (script->flags & SWFDEC_SCRIPT_PRELOAD_PARENT && current_reg < script->n_registers) {
-    if (!swfdec_as_frame_get_variable (frame, SWFDEC_AS_STR__parent, &frame->registers[current_reg])) {
+    if (!swfdec_as_frame_get_variable (context, frame, SWFDEC_AS_STR__parent, &frame->registers[current_reg])) {
       SWFDEC_WARNING ("no root to preload");
     }
     current_reg++;
@@ -702,7 +621,7 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
 
 out:
   if (context->state == SWFDEC_AS_CONTEXT_ABORTED) {
-    swfdec_as_frame_return (frame, NULL);
+    swfdec_as_context_return (context, NULL);
     return;
   }
   if (context->debugger) {
@@ -714,12 +633,9 @@ out:
 }
 
 void
-swfdec_as_frame_handle_exception (SwfdecAsFrame *frame)
+swfdec_as_frame_handle_exception (SwfdecAsContext *cx, SwfdecAsFrame *frame)
 {
-  SwfdecAsContext *cx;
-
   g_return_if_fail (frame != NULL);
-  cx = swfdec_gc_object_get_context (frame->target);
   g_return_if_fail (cx->exception);
 
   /* pop blocks in the hope that we are inside a Try block */
@@ -728,10 +644,22 @@ swfdec_as_frame_handle_exception (SwfdecAsFrame *frame)
   }
   /* no Try blocks caught it, exit frame */
   if (cx->exception) {
-    swfdec_as_frame_return (frame, NULL);
+    swfdec_as_context_return (cx, NULL);
   }
 }
 
+SwfdecMovie *
+swfdec_as_frame_get_target (SwfdecAsFrame *frame) 
+{
+  if (SWFDEC_IS_MOVIE (frame->target) &&
+      SWFDEC_MOVIE (frame->target)->state < SWFDEC_MOVIE_STATE_DESTROYED)
+    return SWFDEC_MOVIE (frame->target);
+  if (SWFDEC_IS_MOVIE (frame->original_target) &&
+      SWFDEC_MOVIE (frame->original_target)->state < SWFDEC_MOVIE_STATE_DESTROYED)
+    return SWFDEC_MOVIE (frame->original_target);
+  return NULL;
+}
+
 /**
  * swfdec_as_frame_get_next:
  * @frame: a #SwfdecAsFrame
@@ -766,20 +694,3 @@ swfdec_as_frame_get_script (SwfdecAsFrame *frame)
   return frame->script;
 }
 
-/**
- * swfdec_as_frame_get_this:
- * @frame: a #SwfdecAsFrame
- *
- * Gets the this object of the given @frame. If the frame has no this object,
- * %NULL is returned.
- *
- * Returns: The this object of the frame or %NULL if none.
- **/
-SwfdecAsObject *
-swfdec_as_frame_get_this (SwfdecAsFrame *frame)
-{
-  g_return_val_if_fail (frame != NULL, NULL);
-
-  return frame->thisp;
-}
-
