@@ -49,8 +49,8 @@ swfdec_as_super_call (SwfdecAsFunction *function, SwfdecAsObject *thisp,
   }
 
   swfdec_as_object_get_variable (super->object, SWFDEC_AS_STR___constructor__, &val);
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&val) ||
-      !SWFDEC_IS_AS_FUNCTION (fun = (SwfdecAsFunction *) SWFDEC_AS_VALUE_GET_OBJECT (&val)))
+  if (!SWFDEC_AS_VALUE_IS_OBJECT (val) ||
+      !SWFDEC_IS_AS_FUNCTION (fun = (SwfdecAsFunction *) (SWFDEC_AS_VALUE_GET_OBJECT (val)->relay)))
     return;
 
   if (construct) {
@@ -61,76 +61,11 @@ swfdec_as_super_call (SwfdecAsFunction *function, SwfdecAsObject *thisp,
       super->object->prototype, n_args, args, return_value);
 }
 
-static gboolean
-swfdec_as_super_get (SwfdecAsObject *object, SwfdecAsObject *orig,
-    const char *variable, SwfdecAsValue *val, guint *flags)
-{
-  SwfdecAsSuper *super = SWFDEC_AS_SUPER (object);
-  SwfdecAsObjectClass *klass;
-  SwfdecAsObject *cur;
-  guint i;
-
-  if (super->object == NULL) {
-    SWFDEC_WARNING ("super.%s () called without an object.", variable);
-    return FALSE;
-  }
-  cur = super->object->prototype;
-  for (i = 0; i <= SWFDEC_AS_OBJECT_PROTOTYPE_RECURSION_LIMIT && cur != NULL; i++) {
-    klass = SWFDEC_AS_OBJECT_GET_CLASS (cur);
-    /* FIXME: is the orig pointer correct? */
-    if (klass->get (cur, super->object, variable, val, flags))
-      return TRUE;
-    /* FIXME: need get_prototype_internal here? */
-    cur = swfdec_as_object_get_prototype (cur);
-  }
-  if (i > SWFDEC_AS_OBJECT_PROTOTYPE_RECURSION_LIMIT) {
-    swfdec_as_context_abort (swfdec_gc_object_get_context (object),
-	"Prototype recursion limit exceeded");
-  }
-  SWFDEC_AS_VALUE_SET_UNDEFINED (val);
-  *flags = 0;
-  return FALSE;
-}
-
-static void
-swfdec_as_super_set (SwfdecAsObject *object, const char *variable, const SwfdecAsValue *val, guint flags)
-{
-  /* This seems to be ignored completely */
-}
-
-static void
-swfdec_as_super_set_flags (SwfdecAsObject *object, const char *variable, guint flags, guint mask)
-{
-  /* if we have no variables, we also can't set its flags... */
-}
-
-static SwfdecAsDeleteReturn
-swfdec_as_super_delete (SwfdecAsObject *object, const char *variable)
-{
-  /* if we have no variables... */
-  return SWFDEC_AS_DELETE_NOT_FOUND;
-}
-
-static SwfdecAsObject *
-swfdec_as_super_resolve (SwfdecAsObject *object)
-{
-  SwfdecAsSuper *super = SWFDEC_AS_SUPER (object);
-
-  return super->thisp;
-}
-
 static void
 swfdec_as_super_class_init (SwfdecAsSuperClass *klass)
 {
-  SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
   SwfdecAsFunctionClass *function_class = SWFDEC_AS_FUNCTION_CLASS (klass);
 
-  asobject_class->get = swfdec_as_super_get;
-  asobject_class->set = swfdec_as_super_set;
-  asobject_class->set_flags = swfdec_as_super_set_flags;
-  asobject_class->del = swfdec_as_super_delete;
-  asobject_class->resolve = swfdec_as_super_resolve;
-
   function_class->call = swfdec_as_super_call;
 }
 
@@ -143,26 +78,30 @@ void
 swfdec_as_super_new (SwfdecAsFrame *frame, SwfdecAsObject *thisp, SwfdecAsObject *ref)
 {
   SwfdecAsContext *context;
+  SwfdecAsObject *object;
   SwfdecAsSuper *super;
 
   g_return_if_fail (frame != NULL);
-  g_return_if_fail (SWFDEC_IS_AS_OBJECT (thisp));
-  g_return_if_fail (ref == NULL || SWFDEC_IS_AS_OBJECT (ref));
+  g_return_if_fail (thisp != NULL);
   
   if (frame->super != NULL)
     return;
-  context = swfdec_gc_object_get_context (thisp);
+  context = thisp->context;
   if (context->version <= 5)
     return;
 
   super = g_object_new (SWFDEC_TYPE_AS_SUPER, "context", context, NULL);
-  frame->super = SWFDEC_AS_OBJECT (super);
+  frame->super = super;
   super->thisp = swfdec_as_object_resolve (thisp);
   if (context->version <= 5) {
     super->object = NULL;
   } else {
     super->object = ref;
   }
+
+  object = swfdec_as_object_new_empty (context);
+  object->super = TRUE;
+  swfdec_as_object_set_relay (object, SWFDEC_AS_RELAY (super));
 }
 
 SwfdecAsObject *
