@@ -25,10 +25,12 @@
 #include <string.h>
 
 #include "swfdec_as_types.h"
-#include "swfdec_as_object.h"
 #include "swfdec_as_context.h"
 #include "swfdec_as_function.h"
+#include "swfdec_as_gcable.h"
+#include "swfdec_as_internal.h"
 #include "swfdec_as_number.h"
+#include "swfdec_as_object.h"
 #include "swfdec_as_stack.h"
 #include "swfdec_as_string.h"
 #include "swfdec_as_strings.h"
@@ -60,6 +62,7 @@
 /**
  * SwfdecAsValueType:
  * @SWFDEC_AS_TYPE_UNDEFINED: the special undefined value
+ * @SWFDEC_AS_TYPE_NULL: the spaecial null value
  * @SWFDEC_AS_TYPE_BOOLEAN: a boolean value - true or false
  * @SWFDEC_AS_TYPE_INT: reserved value for integers. Should the need arise for
  *                      performance enhancements - especially on embedded 
@@ -68,8 +71,9 @@
  *                      it will cause Swfdec to crash.
  * @SWFDEC_AS_TYPE_NUMBER: a double value - also used for integer numbers
  * @SWFDEC_AS_TYPE_STRING: a string. Strings are garbage-collected and unique.
- * @SWFDEC_AS_TYPE_NULL: the spaecial null value
  * @SWFDEC_AS_TYPE_OBJECT: an object - must be of type #SwfdecAsObject
+ * @SWFDEC_AS_TYPE_MOVIE: an internal type used only inside #SwfdecPlayer 
+ *                        objects. It is not exported in the API.
  *
  * These are the possible values the Swfdec Actionscript engine knows about.
  */
@@ -80,9 +84,21 @@
  *
  * This is the type used to present an opaque value in the Actionscript 
  * engine. See #SwfdecAsValueType for possible types. It's similar in 
- * spirit to #GValue. The value held is garbage-collected. Apart from the type 
- * member, use the provided macros to access this structure.
- * <note>If you memset a SwfdecAsValue to 0, it is a valid undefined value.</note>
+ * spirit to #GValue. Use the provided macros to access this structure.
+ */
+
+/**
+ * SWFDEC_AS_VALUE_GET_TYPE:
+ * @val: The value to extract the type from
+ *
+ * Extracts the type from a given value.
+ */
+
+/**
+ * SWFDEC_AS_VALUE_UNDEFINED:
+ *
+ * The special "undefined" value. Use SWFDEC_AS_VALUE_IS_UNDEFINED() to
+ * check if a value equals this value.
  */
 
 /**
@@ -93,6 +109,18 @@
  * you can instead use code such as |[ SwfdecAsValue val = { 0, }; ]|
  */
 
+/**
+ * SWFDEC_AS_VALUE_FALSE:
+ *
+ * The boolean value false.
+ */
+
+/**
+ * SWFDEC_AS_VALUE_TRUE:
+ *
+ * The boolean value true.
+ */
+
 /**
  * SWFDEC_AS_VALUE_GET_BOOLEAN:
  * @val: value to get, the value must reference a boolean
@@ -103,6 +131,17 @@
  * Returns: %TRUE or %FALSE
  */
 
+/**
+ * SWFDEC_AS_VALUE_FROM_BOOLEAN:
+ * @b: boolean to convert
+ *
+ * Converts the given value to a boolean #SwfdecAsValue. When knowing the value
+ * at compile-time, use the static values such as %SWFDEC_AS_VALUE_TRUE instead
+ * of SWFDEC_AS_VALUE_FROM_BOOLEAN(%TRUE).
+ *
+ * Returns: %SWFDEC_AS_VALUE_TRUE or %SWFDEC_AS_VALUE_FALSE
+ */
+
 /**
  * SWFDEC_AS_VALUE_SET_BOOLEAN:
  * @val: value to set
@@ -122,25 +161,6 @@
  * Returns: a double. It can be NaN or +-Infinity, but not -0.0
  */
 
-/**
- * SWFDEC_AS_VALUE_SET_NUMBER:
- * @val: value to set
- * @d: double value to set
- *
- * Sets @val to the given value. If you are sure the value is a valid
- * integer value, use SWFDEC_AS_VALUE_SET_INT() instead.
- */
-
-/**
- * SWFDEC_AS_VALUE_SET_INT:
- * @val: value to set
- * @d: integer value to set
- *
- * Sets @val to the given value. Currently this macro is equivalent to
- * SWFDEC_AS_VALUE_SET_NUMBER(), but this may change in future versions of
- * Swfdec.
- */
-
 /**
  * SWFDEC_AS_VALUE_GET_STRING:
  * @val: value to get, the value must reference a string
@@ -151,6 +171,15 @@
  * Returns: a garbage-collected string.
  */
 
+/**
+ * SWFDEC_AS_VALUE_FROM_STRING:
+ * @s: garbage-collected string to convert
+ *
+ * Converts the given string to a #SwfdecAsValue.
+ *
+ * Returns: a SwfdecAsValue representing the given string
+ */
+
 /**
  * SWFDEC_AS_VALUE_SET_STRING:
  * @val: value to set
@@ -159,6 +188,13 @@
  * Sets @val to the given string value.
  */
 
+/**
+ * SWFDEC_AS_VALUE_NULL:
+ *
+ * The special "null" value. Use SWFDEC_AS_VALUE_IS_NULL() to
+ * check if a value equals this value.
+ */
+
 /**
  * SWFDEC_AS_VALUE_SET_NULL:
  * @val: value to set
@@ -176,6 +212,15 @@
  * Returns: a #SwfdecAsObject
  */
 
+/**
+ * SWFDEC_AS_VALUE_FROM_OBJECT:
+ * @o: the #SwfdecAsObject to convert
+ *
+ * Converts the given object to a #SwfdecAsValue.
+ *
+ * Returns: a SwfdecAsValue representing the given object
+ */
+
 /**
  * SWFDEC_AS_VALUE_SET_OBJECT:
  * @val: value to set
@@ -187,6 +232,43 @@
 
 /*** actual code ***/
 
+/**
+ * swfdec_as_value_from_integer:
+ * @cx: The context to use
+ * @i: integer value to set
+ *
+ * Creates a garbage-collected value representing @i and returns it.
+ * Currently this function is a macro that calls swfdec_as_value_set_number(),
+ * but this may change in future versions of Swfdec.
+ *
+ * Returns: The new value representing @i
+ */
+
+/**
+ * swfdec_as_value_from_number:
+ * @context: The context to use
+ * @number: double value to set
+ *
+ * Creates a garbage-collected value representing @number and returns it.
+ *
+ * Returns: The new value representing @number
+ */
+SwfdecAsValue
+swfdec_as_value_from_number (SwfdecAsContext *context,
+    double d)
+{
+  SwfdecAsDoubleValue *dval;
+
+  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), SWFDEC_AS_VALUE_UNDEFINED);
+
+  dval = swfdec_as_gcable_new (context, SwfdecAsDoubleValue);
+  dval->number = d;
+  SWFDEC_AS_GCABLE_SET_NEXT (dval, context->numbers);
+  context->numbers = dval;
+
+  return SWFDEC_AS_VALUE_COMBINE (dval, SWFDEC_AS_TYPE_NUMBER);
+}
+
 /**
  * swfdec_as_str_concat:
  * @cx: a #SwfdecAsContext
@@ -363,12 +445,11 @@ swfdec_as_double_to_string (SwfdecAsContext *context, double d)
  *          never be %NULL.
  **/
 const char *
-swfdec_as_value_to_string (SwfdecAsContext *context, const SwfdecAsValue *value)
+swfdec_as_value_to_string (SwfdecAsContext *context, SwfdecAsValue value)
 {
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), SWFDEC_AS_STR_EMPTY);
-  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (value), SWFDEC_AS_STR_EMPTY);
 
-  switch (value->type) {
+  switch (SWFDEC_AS_VALUE_GET_TYPE (value)) {
     case SWFDEC_AS_TYPE_STRING:
       return SWFDEC_AS_VALUE_GET_STRING (value);
     case SWFDEC_AS_TYPE_UNDEFINED:
@@ -385,29 +466,31 @@ swfdec_as_value_to_string (SwfdecAsContext *context, const SwfdecAsValue *value)
     case SWFDEC_AS_TYPE_OBJECT:
       {
 	SwfdecAsObject *object = SWFDEC_AS_VALUE_GET_OBJECT (value);
-	if (SWFDEC_IS_MOVIE (object)) {
-	  SwfdecMovie *movie = swfdec_movie_resolve (SWFDEC_MOVIE (object));
-	  if (movie == NULL) {
-	    return SWFDEC_AS_STR_EMPTY;
-	  } else {
-	    char *str = swfdec_movie_get_path (SWFDEC_MOVIE (object), TRUE);
-	    return swfdec_as_context_give_string (context, str);
-	  }
-	} else if (SWFDEC_IS_AS_STRING (object)) {
-	  return SWFDEC_AS_STRING (object)->string;
+	if (SWFDEC_IS_AS_STRING (object->relay)) {
+	  return SWFDEC_AS_STRING (object->relay)->string;
 	} else {
 	  SwfdecAsValue ret;
 	  swfdec_as_object_call (object, SWFDEC_AS_STR_toString, 0, NULL, &ret);
-	  if (SWFDEC_AS_VALUE_IS_STRING (&ret))
-	    return SWFDEC_AS_VALUE_GET_STRING (&ret);
-	  else if (SWFDEC_IS_AS_SUPER (SWFDEC_AS_VALUE_GET_OBJECT (value)))
+	  if (SWFDEC_AS_VALUE_IS_STRING (ret))
+	    return SWFDEC_AS_VALUE_GET_STRING (ret);
+	  else if (SWFDEC_IS_AS_SUPER (object->relay))
 	    return SWFDEC_AS_STR__type_Object_;
-	  else if (SWFDEC_IS_AS_FUNCTION (SWFDEC_AS_VALUE_GET_OBJECT (value)))
+	  else if (SWFDEC_IS_AS_FUNCTION (object->relay))
 	    return SWFDEC_AS_STR__type_Function_;
 	  else
 	    return SWFDEC_AS_STR__type_Object_;
 	}
       }
+    case SWFDEC_AS_TYPE_MOVIE:
+      {
+	SwfdecMovie *movie = SWFDEC_AS_VALUE_GET_MOVIE (value);
+	char *str;
+
+	if (movie == NULL)
+	  return SWFDEC_AS_STR_EMPTY;
+	str = swfdec_movie_get_path (movie, TRUE);
+	return swfdec_as_context_give_string (context, str);
+      }
     case SWFDEC_AS_TYPE_INT:
     default:
       g_assert_not_reached ();
@@ -415,40 +498,6 @@ swfdec_as_value_to_string (SwfdecAsContext *context, const SwfdecAsValue *value)
   }
 }
 
-/**
- * swfdec_as_value_to_debug:
- * @value: a #SwfdecAsValue
- *
- * Converts the given @value to a string in a safe way. It will not call into
- * the scripting engine. Its intended use is for output in debuggers.
- *
- * Returns: a newly allocated string. Free with g_free().
- **/
-char *
-swfdec_as_value_to_debug (const SwfdecAsValue *value)
-{
-  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (value), NULL);
-
-  switch (value->type) {
-    case SWFDEC_AS_TYPE_STRING:
-      return g_shell_quote (SWFDEC_AS_VALUE_GET_STRING (value));
-    case SWFDEC_AS_TYPE_UNDEFINED:
-      return g_strdup ("undefined");
-    case SWFDEC_AS_TYPE_BOOLEAN:
-      return g_strdup (SWFDEC_AS_VALUE_GET_BOOLEAN (value) ? "true" : "false");
-    case SWFDEC_AS_TYPE_NULL:
-      return g_strdup ("null");
-    case SWFDEC_AS_TYPE_NUMBER:
-      return g_strdup_printf ("%g", SWFDEC_AS_VALUE_GET_NUMBER (value));
-    case SWFDEC_AS_TYPE_OBJECT:
-      return swfdec_as_object_get_debug (SWFDEC_AS_VALUE_GET_OBJECT (value));
-    case SWFDEC_AS_TYPE_INT:
-    default:
-      g_assert_not_reached ();
-      return NULL;
-  }
-}
-
 /**
  * swfdec_as_value_to_number:
  * @context: a #SwfdecAsContext
@@ -462,24 +511,20 @@ swfdec_as_value_to_debug (const SwfdecAsValue *value)
  * Returns: a double value. It can be NaN or +-Infinity. It will not be -0.0.
  **/
 double
-swfdec_as_value_to_number (SwfdecAsContext *context, const SwfdecAsValue *value)
+swfdec_as_value_to_number (SwfdecAsContext *context, SwfdecAsValue value)
 {
-  SwfdecAsValue tmp;
-
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), 0.0);
-  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (value), 0.0);
 
-  tmp = *value;
-  swfdec_as_value_to_primitive (&tmp);
+  value = swfdec_as_value_to_primitive (value);
 
-  switch (tmp.type) {
+  switch (SWFDEC_AS_VALUE_GET_TYPE (value)) {
     case SWFDEC_AS_TYPE_UNDEFINED:
     case SWFDEC_AS_TYPE_NULL:
       return (context->version >= 7) ? NAN : 0.0;
     case SWFDEC_AS_TYPE_BOOLEAN:
-      return SWFDEC_AS_VALUE_GET_BOOLEAN (&tmp) ? 1 : 0;
+      return SWFDEC_AS_VALUE_GET_BOOLEAN (value) ? 1 : 0;
     case SWFDEC_AS_TYPE_NUMBER:
-      return SWFDEC_AS_VALUE_GET_NUMBER (&tmp);
+      return SWFDEC_AS_VALUE_GET_NUMBER (value);
     case SWFDEC_AS_TYPE_STRING:
       {
 	const char *s;
@@ -487,7 +532,7 @@ swfdec_as_value_to_number (SwfdecAsContext *context, const SwfdecAsValue *value)
 	double d;
 	
 	// FIXME: We should most likely copy Tamarin's code here (MathUtils.cpp)
-	s = SWFDEC_AS_VALUE_GET_STRING (&tmp);
+	s = SWFDEC_AS_VALUE_GET_STRING (value);
 	if (s == SWFDEC_AS_STR_EMPTY)
 	  return (context->version >= 5) ? NAN : 0.0;
 	if (context->version > 5 && s[0] == '0' &&
@@ -508,6 +553,7 @@ swfdec_as_value_to_number (SwfdecAsContext *context, const SwfdecAsValue *value)
 	  return NAN;
       }
     case SWFDEC_AS_TYPE_OBJECT:
+    case SWFDEC_AS_TYPE_MOVIE:
       return (context->version >= 5) ? NAN : 0.0;
     case SWFDEC_AS_TYPE_INT:
     default:
@@ -550,7 +596,7 @@ swfdec_as_double_to_integer (double d)
  * Returns: An Integer that can be represented in 32 bits.
  **/
 int
-swfdec_as_value_to_integer (SwfdecAsContext *context, const SwfdecAsValue *value)
+swfdec_as_value_to_integer (SwfdecAsContext *context, SwfdecAsValue value)
 {
   double d;
   
@@ -572,16 +618,15 @@ swfdec_as_value_to_integer (SwfdecAsContext *context, const SwfdecAsValue *value
  * Returns: object representing @value or %NULL.
  **/
 SwfdecAsObject *
-swfdec_as_value_to_object (SwfdecAsContext *context, const SwfdecAsValue *value)
+swfdec_as_value_to_object (SwfdecAsContext *context, SwfdecAsValue value)
 {
   SwfdecAsFunction *fun;
   SwfdecAsValue val;
   const char *s;
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
-  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (value), NULL);
 
-  switch (value->type) {
+  switch (SWFDEC_AS_VALUE_GET_TYPE (value)) {
     case SWFDEC_AS_TYPE_UNDEFINED:
     case SWFDEC_AS_TYPE_NULL:
       return NULL;
@@ -595,7 +640,8 @@ swfdec_as_value_to_object (SwfdecAsContext *context, const SwfdecAsValue *value)
       s = SWFDEC_AS_STR_Boolean;
       break;
     case SWFDEC_AS_TYPE_OBJECT:
-      return SWFDEC_AS_VALUE_GET_OBJECT (value);
+    case SWFDEC_AS_TYPE_MOVIE:
+      return SWFDEC_AS_VALUE_GET_COMPOSITE (value);
     case SWFDEC_AS_TYPE_INT:
     default:
       g_assert_not_reached ();
@@ -603,12 +649,12 @@ swfdec_as_value_to_object (SwfdecAsContext *context, const SwfdecAsValue *value)
   }
 
   swfdec_as_object_get_variable (context->global, s, &val);
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&val) ||
-      !SWFDEC_IS_AS_FUNCTION (fun = (SwfdecAsFunction *) SWFDEC_AS_VALUE_GET_OBJECT (&val)))
+  if (!SWFDEC_AS_VALUE_IS_OBJECT (val) ||
+      !SWFDEC_IS_AS_FUNCTION (fun = (SwfdecAsFunction *) (SWFDEC_AS_VALUE_GET_OBJECT (val)->relay)))
     return NULL;
-  swfdec_as_object_create (fun, 1, value, &val);
-  if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
-    return SWFDEC_AS_VALUE_GET_OBJECT (&val);
+  swfdec_as_object_create (fun, 1, &value, &val);
+  if (SWFDEC_AS_VALUE_IS_OBJECT (val)) {
+    return SWFDEC_AS_VALUE_GET_OBJECT (val);
   } else {
     SWFDEC_ERROR ("did not construct an object");
     return NULL;
@@ -626,13 +672,12 @@ swfdec_as_value_to_object (SwfdecAsContext *context, const SwfdecAsValue *value)
 * Returns: either %TRUE or %FALSE.
 **/
 gboolean
-swfdec_as_value_to_boolean (SwfdecAsContext *context, const SwfdecAsValue *value)
+swfdec_as_value_to_boolean (SwfdecAsContext *context, SwfdecAsValue value)
 {
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), FALSE);
-  g_return_val_if_fail (SWFDEC_IS_AS_VALUE (value), FALSE);
 
   /* FIXME: what do we do when called in flash 4? */
-  switch (value->type) {
+  switch (SWFDEC_AS_VALUE_GET_TYPE (value)) {
     case SWFDEC_AS_TYPE_UNDEFINED:
     case SWFDEC_AS_TYPE_NULL:
       return FALSE;
@@ -651,6 +696,7 @@ swfdec_as_value_to_boolean (SwfdecAsContext *context, const SwfdecAsValue *value
 	return SWFDEC_AS_VALUE_GET_STRING (value) != SWFDEC_AS_STR_EMPTY;
       }
     case SWFDEC_AS_TYPE_OBJECT:
+    case SWFDEC_AS_TYPE_MOVIE:
       return TRUE;
     case SWFDEC_AS_TYPE_INT:
     default:
@@ -667,16 +713,49 @@ swfdec_as_value_to_boolean (SwfdecAsContext *context, const SwfdecAsValue *value
 * values are values that are not objects. If the value is an object, the 
 * object's valueOf function is called. If the result of that function is still 
 * an object, it is returned nonetheless.
+*
+* Returns: The primitive value for &value
 **/
-void
-swfdec_as_value_to_primitive (SwfdecAsValue *value)
+SwfdecAsValue
+swfdec_as_value_to_primitive (SwfdecAsValue value)
 {
-  g_return_if_fail (SWFDEC_IS_AS_VALUE (value));
 
-  if (SWFDEC_AS_VALUE_IS_OBJECT (value) && !SWFDEC_IS_MOVIE (SWFDEC_AS_VALUE_GET_OBJECT (value))) {
+  if (SWFDEC_AS_VALUE_IS_OBJECT (value)) {
     swfdec_as_object_call (SWFDEC_AS_VALUE_GET_OBJECT (value), SWFDEC_AS_STR_valueOf,
-	0, NULL, value);
+	0, NULL, &value);
   }
+  return value;
+}
+
+/**
+ * swfdec_as_value_get_variable:
+ * @cx: the context
+ * @value: the value to get the variable from
+ * @name: name of the variable to get
+ * @ret: The return value to set. May be identical to the passed in @value.
+ *
+ * Gets a variable from the given @value. This function is a shortcut for 
+ * converting to a #SwfdecAsObject and then calling 
+ * swfdec_As_object_get_variable(). When the @value cannot be converted to an
+ * object, @ret is set to undefined.
+ **/
+void
+swfdec_as_value_get_variable (SwfdecAsContext *cx, const SwfdecAsValue *value, 
+    const char *name, SwfdecAsValue *ret)
+{
+  SwfdecAsObject *object;
+
+  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (cx));
+  g_return_if_fail (value != NULL);
+  g_return_if_fail (name != NULL);
+  g_return_if_fail (ret != NULL);
+
+  object = swfdec_as_value_to_object (cx, *value);
+  if (object == NULL) {
+    SWFDEC_AS_VALUE_SET_UNDEFINED (ret);
+    return;
+  }
+  swfdec_as_object_get_variable (object, name, ret);
 }
 
 /* from swfdec_internal.h */
@@ -690,10 +769,10 @@ swfdec_as_value_to_twips (SwfdecAsContext *context, const SwfdecAsValue *val,
   g_return_val_if_fail (val != NULL, FALSE);
   g_return_val_if_fail (result != NULL, FALSE);
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (val) || SWFDEC_AS_VALUE_IS_NULL (val))
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (*val) || SWFDEC_AS_VALUE_IS_NULL (*val))
     return FALSE;
 
-  d = swfdec_as_value_to_number (context, val);
+  d = swfdec_as_value_to_number (context, *val);
   if (isnan (d))
     return FALSE;
   if (is_length && d < 0)
@@ -706,15 +785,3 @@ swfdec_as_value_to_twips (SwfdecAsContext *context, const SwfdecAsValue *val,
   return TRUE;
 }
 
-SwfdecAsObject *
-swfdec_as_value_get_object (const SwfdecAsValue *val)
-{
-  SwfdecAsObject *o = val->value.object;
-
-  if (SWFDEC_IS_MOVIE (o)) {
-    o = SWFDEC_AS_OBJECT (swfdec_movie_resolve (SWFDEC_MOVIE (o)));
-    if (o == NULL)
-      o = val->value.object;
-  }
-  return o;
-}
