@@ -70,11 +70,9 @@ struct _SwfdecAudioDecoderClass
                                          SwfdecAudioFormat      format,
 					 char **                missing);
   SwfdecAudioDecoder *	(* create)	(guint                  codec,
-					 SwfdecAudioFormat      format);
+					 SwfdecAudioFormat      format,
+					 SwfdecBuffer *		data);
 
-  void			(* set_codec_data)
-					(SwfdecAudioDecoder *	decoder,
-					 SwfdecBuffer *		buffer);
   void			(* push)	(SwfdecAudioDecoder *	decoder,
 					 SwfdecBuffer *		buffer);
   SwfdecBuffer *	(* pull)	(SwfdecAudioDecoder *	decoder);
@@ -88,11 +86,9 @@ gboolean		swfdec_audio_decoder_prepare	(guint			codec,
 							 SwfdecAudioFormat	format,
 							 char **		missing);
 SwfdecAudioDecoder *   	swfdec_audio_decoder_new      	(guint			codec,
-							 SwfdecAudioFormat	format);
+							 SwfdecAudioFormat	format,
+							 SwfdecBuffer *		data);
 
-void			swfdec_audio_decoder_set_codec_data
-							(SwfdecAudioDecoder *	decoder,
-							 SwfdecBuffer *		buffer);
 void			swfdec_audio_decoder_push	(SwfdecAudioDecoder *	decoder,
 							 SwfdecBuffer *		buffer);
 SwfdecBuffer *		swfdec_audio_decoder_pull	(SwfdecAudioDecoder *	decoder);
