@@ -24,6 +24,8 @@
 
 #include <swfdec/swfdec_types.h>
 
+G_BEGIN_DECLS
+
 struct _SwfdecColorTransform {
   gboolean mask;			/* TRUE if this is a mask - masks are always black */
   /* naming here is taken from ActionScript, where ?a is the multiplier and ?b the offset */
@@ -46,33 +48,43 @@ struct _SwfdecColorTransform {
 
 #define SWFDEC_COLOR_COMBINE(r,g,b,a)	((SwfdecColor) (((a)<<24) | ((r)<<16) | ((g)<<8) | (b)))
 #define SWFDEC_COLOR_OPAQUE(color)	((SwfdecColor) ((color) | SWFDEC_COLOR_COMBINE (0, 0, 0, 0xFF)))
-#define SWFDEC_COLOR_A(x)		(((x)>>24)&0xff)
-#define SWFDEC_COLOR_R(x)		(((x)>>16)&0xff)
-#define SWFDEC_COLOR_G(x)		(((x)>>8)&0xff)
-#define SWFDEC_COLOR_B(x)		((x)&0xff)
+#define SWFDEC_COLOR_ALPHA(x)		(((x)>>24)&0xff)
+#define SWFDEC_COLOR_RED(x)		(((x)>>16)&0xff)
+#define SWFDEC_COLOR_GREEN(x)		(((x)>>8)&0xff)
+#define SWFDEC_COLOR_BLUE(x)		((x)&0xff)
 
 #define SWFDEC_COLOR_WHITE		SWFDEC_COLOR_COMBINE (0xFF, 0xFF, 0xFF, 0xFF)
 
-SwfdecColor swfdec_color_apply_morph (SwfdecColor start, SwfdecColor end, guint ratio);
-void swfdec_color_set_source (cairo_t *cr, SwfdecColor color);
-void swfdec_color_transform_init_identity (SwfdecColorTransform * trans);
-void swfdec_color_transform_init_mask (SwfdecColorTransform * trans);
-void swfdec_color_transform_init_color (SwfdecColorTransform *trans, SwfdecColor color);
-gboolean swfdec_color_transform_is_identity (const SwfdecColorTransform * trans);
+SwfdecColor	swfdec_color_apply_morph			(SwfdecColor	      	start,
+								 SwfdecColor		end,
+								 guint			ratio);
+void		swfdec_color_set_source				(cairo_t *		cr,
+								 SwfdecColor		color);
+void		swfdec_color_transform_init_identity		(SwfdecColorTransform *	trans);
+void		swfdec_color_transform_init_mask		(SwfdecColorTransform *	trans);
+void		swfdec_color_transform_init_color		(SwfdecColorTransform *	trans,
+								 SwfdecColor		color);
+gboolean	swfdec_color_transform_is_identity		(const SwfdecColorTransform *	trans);
+gboolean	swfdec_color_transform_is_alpha			(const SwfdecColorTransform *	trans);
 #define swfdec_color_transform_is_mask(trans) ((trans)->mask)
-void swfdec_color_transform_chain (SwfdecColorTransform *dest,
-    const SwfdecColorTransform *last, const SwfdecColorTransform *first);
-SwfdecColor swfdec_color_apply_transform (SwfdecColor in,
-    const SwfdecColorTransform * trans);
-SwfdecColor swfdec_color_apply_transform_premultiplied (SwfdecColor in, 
-    const SwfdecColorTransform * trans);
+void		swfdec_color_transform_chain			(SwfdecColorTransform *	dest,
+								 const SwfdecColorTransform *	last,
+								 const SwfdecColorTransform *	first);
+SwfdecColor	swfdec_color_apply_transform			(SwfdecColor		in,
+								 const SwfdecColorTransform *	trans);
+SwfdecColor	swfdec_color_apply_transform_premultiplied	(SwfdecColor		in, 
+								 const SwfdecColorTransform *	trans);
 
-void swfdec_matrix_ensure_invertible (cairo_matrix_t *matrix, cairo_matrix_t *inverse);
-double swfdec_matrix_get_xscale (const cairo_matrix_t *matrix);
-double swfdec_matrix_get_yscale (const cairo_matrix_t *matrix);
-double swfdec_matrix_get_rotation (const cairo_matrix_t *matrix);
-void swfdec_matrix_morph (cairo_matrix_t *dest, const cairo_matrix_t *start,
-    const cairo_matrix_t *end, guint ratio);
+void		swfdec_matrix_ensure_invertible			(cairo_matrix_t *	matrix,
+								 cairo_matrix_t *	inverse);
+double		swfdec_matrix_get_xscale			(const cairo_matrix_t *	matrix);
+double		swfdec_matrix_get_yscale			(const cairo_matrix_t *	matrix);
+double		swfdec_matrix_get_rotation			(const cairo_matrix_t *	matrix);
+void		swfdec_matrix_morph				(cairo_matrix_t *	dest,
+								 const cairo_matrix_t *	start,
+								 const cairo_matrix_t *	end,
+								 guint			ratio);
 
 
+G_END_DECLS
 #endif
