@@ -35,20 +35,15 @@ static SwfdecMovie *
 swfdec_movie_color_get_movie (SwfdecAsObject *object)
 {
   SwfdecAsValue val;
-  SwfdecAsObject *target;
 
   if (object == NULL)
     return NULL;
 
   swfdec_as_object_get_variable (object, SWFDEC_AS_STR_target, &val);
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&val))
+  if (!SWFDEC_AS_VALUE_IS_MOVIE (val))
     return NULL;
 
-  target = SWFDEC_AS_VALUE_GET_OBJECT (&val);
-  if (!SWFDEC_IS_MOVIE (target))
-    return NULL;
-
-  return SWFDEC_MOVIE (target);
+  return SWFDEC_AS_VALUE_GET_MOVIE (val);
 }
 
 SWFDEC_AS_NATIVE (700, 2, swfdec_movie_color_getRGB)
@@ -67,7 +62,7 @@ swfdec_movie_color_getRGB (SwfdecAsContext *cx, SwfdecAsObject *obj,
   result = (movie->color_transform.rb << 16) |
 	   ((movie->color_transform.gb % 256) << 8) | 
 	   (movie->color_transform.bb % 256);
-  SWFDEC_AS_VALUE_SET_INT (ret, result);
+  *ret = swfdec_as_value_from_integer (cx, result);
 }
 
 static void
@@ -75,7 +70,7 @@ add_variable (SwfdecAsObject *obj, const char *name, double value)
 {
   SwfdecAsValue val;
 
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, value);
+  val = swfdec_as_value_from_number (obj->context, value);
   swfdec_as_object_set_variable (obj, name, &val);
 }
 
@@ -92,7 +87,7 @@ swfdec_movie_color_getTransform (SwfdecAsContext *cx, SwfdecAsObject *obj,
   if (movie == NULL)
     return;
 
-  ret = swfdec_as_object_new (cx);
+  ret = swfdec_as_object_new (cx, SWFDEC_AS_STR_Object, NULL);
 
   add_variable (ret, SWFDEC_AS_STR_ra, movie->color_transform.ra * 100.0 / 256.0);
   add_variable (ret, SWFDEC_AS_STR_ga, movie->color_transform.ga * 100.0 / 256.0);
@@ -121,7 +116,7 @@ swfdec_movie_color_setRGB (SwfdecAsContext *cx, SwfdecAsObject *obj,
   if (movie == NULL)
     return;
 
-  color = swfdec_as_value_to_integer (cx, &argv[0]);
+  color = swfdec_as_value_to_integer (cx, argv[0]);
 
   movie->color_transform.ra = 0;
   movie->color_transform.rb = (color & 0xFF0000) >> 16;
@@ -140,7 +135,7 @@ parse_property (SwfdecAsObject *obj, const char *name, int *target, gboolean sca
 
   if (!swfdec_as_object_get_variable (obj, name, &val))
     return;
-  d = swfdec_as_value_to_number (swfdec_gc_object_get_context (obj), &val);
+  d = swfdec_as_value_to_number (obj->context, val);
   if (scale) {
     *target = d * 256.0 / 100.0;
   } else {
@@ -164,9 +159,11 @@ swfdec_movie_color_setTransform (SwfdecAsContext *cx, SwfdecAsObject *obj,
   if (movie == NULL)
     return;
 
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&argv[0]))
+  if (!SWFDEC_AS_VALUE_IS_COMPOSITE (argv[0]))
+    return;
+  parse = SWFDEC_AS_VALUE_GET_COMPOSITE (argv[0]);
+  if (parse == NULL)
     return;
-  parse = SWFDEC_AS_VALUE_GET_OBJECT (&argv[0]);
   parse_property (parse, SWFDEC_AS_STR_ra, &movie->color_transform.ra, TRUE);
   parse_property (parse, SWFDEC_AS_STR_ga, &movie->color_transform.ga, TRUE);
   parse_property (parse, SWFDEC_AS_STR_ba, &movie->color_transform.ba, TRUE);
