@@ -1,5 +1,5 @@
 /* Swfdec
- * Copyright (C) 2007 Benjamin Otte <otte@gnome.org>
+ * Copyright (C) 2007-2008 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
 #define _SWFDEC_FILTER_H_
 
 #include <swfdec/swfdec.h>
-#include <swfdec/swfdec_as_object.h>
+#include <swfdec/swfdec_as_relay.h>
 #include <swfdec/swfdec_bits.h>
 #include <swfdec/swfdec_types.h>
 
@@ -37,23 +37,44 @@ typedef struct _SwfdecFilterClass SwfdecFilterClass;
 #define SWFDEC_FILTER_GET_CLASS(obj)          (G_TYPE_INSTANCE_GET_CLASS ((obj), SWFDEC_TYPE_FILTER, SwfdecFilterClass))
 
 struct _SwfdecFilter {
-  SwfdecAsObject	object;
+  SwfdecAsRelay		relay;
 };
 
 struct _SwfdecFilterClass {
-  SwfdecAsObjectClass	object_class;
+  SwfdecAsRelayClass	relay_class;
 
-  SwfdecFilter *	(* clone)		(SwfdecFilter *		filter);
+  void			(* clone)		(SwfdecFilter *		dest,
+						 SwfdecFilter *		source);
+  void			(* get_rectangle)	(SwfdecFilter *		filter,
+						 SwfdecRectangle *	dest,
+						 double			xscale,
+						 double			yscale,
+						 const SwfdecRectangle *source);
   cairo_pattern_t *	(* apply)		(SwfdecFilter *		filter,
-						 cairo_pattern_t *	pattern);
+						 cairo_pattern_t *	pattern,
+						 double			xscale,
+						 double			yscale,
+						 const SwfdecRectangle *rect);
 };
 
 GType			swfdec_filter_get_type	(void);
 
-cairo_pattern_t *	swfdec_filter_apply	(SwfdecFilter *		filter,
-						 cairo_pattern_t *	pattern);
-GSList *		swfdec_filter_parse	(SwfdecBits *		bits);
+SwfdecFilter *		swfdec_filter_clone		(SwfdecFilter *		filter);
+cairo_pattern_t *	swfdec_filter_apply		(SwfdecFilter *		filter,
+							 cairo_pattern_t *	pattern,
+							 double			xscale,
+							 double			yscale,
+							 const SwfdecRectangle *source);
+void			swfdec_filter_get_rectangle	(SwfdecFilter *		filter,
+							 SwfdecRectangle *	dest,
+							 double			xscale,
+							 double			yscale,
+							 const SwfdecRectangle *source);
 
+GSList *		swfdec_filter_parse		(SwfdecPlayer *		player,
+							 SwfdecBits *		bits);
+void			swfdec_filter_skip		(SwfdecBits *		bits);
+						
 
 G_END_DECLS
 #endif
