@@ -34,7 +34,7 @@ swfdec_key_getAscii (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   SwfdecPlayer *player = SWFDEC_PLAYER (cx);
 
-  SWFDEC_AS_VALUE_SET_INT (retval, player->priv->last_character);
+  *retval = swfdec_as_value_from_integer (cx, player->priv->last_character);
 }
 
 SWFDEC_AS_NATIVE (800, 1, swfdec_key_getCode)
@@ -44,7 +44,7 @@ swfdec_key_getCode (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   SwfdecPlayer *player = SWFDEC_PLAYER (cx);
 
-  SWFDEC_AS_VALUE_SET_INT (retval, player->priv->last_keycode);
+  *retval = swfdec_as_value_from_integer (cx, player->priv->last_keycode);
 }
 
 SWFDEC_AS_NATIVE (800, 2, swfdec_key_isDown)
@@ -58,7 +58,7 @@ swfdec_key_isDown (SwfdecAsContext *cx, SwfdecAsObject *object,
   if (argc < 1)
     return;
 
-  id = swfdec_as_value_to_integer (cx, &argv[0]);
+  id = swfdec_as_value_to_integer (cx, argv[0]);
   if (id >= 256) {
     SWFDEC_FIXME ("id %u too big for a keycode", id);
     id %= 256;
