@@ -88,7 +88,7 @@ swfdec_sprite_movie_beginFill (SwfdecAsContext *cx, SwfdecAsObject *object,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_MOVIE, &movie, "|ii", &color, &alpha);
   movie->draw_fill = NULL;
   
-  if (argc == 0 || SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0])) {
+  if (argc == 0 || SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0])) {
     color = 0;
   } else {
     color = color & 0xFFFFFF;
@@ -106,7 +106,7 @@ swfdec_sprite_movie_gradient_fill_get_length (SwfdecAsObject *o)
   SwfdecAsValue val;
 
   swfdec_as_object_get_variable (o, SWFDEC_AS_STR_length, &val);
-  length = swfdec_as_value_to_integer (swfdec_gc_object_get_context (o), &val);
+  length = swfdec_as_value_to_integer (o->context, val);
   return MAX (length, 0);
 }
 
@@ -126,27 +126,27 @@ swfdec_sprite_movie_gradient_fill_check_length (SwfdecAsObject *colors, SwfdecAs
 static void
 swfdec_sprite_movie_extract_matrix (SwfdecAsObject *o, cairo_matrix_t *mat)
 {
-  SwfdecAsContext *cx = swfdec_gc_object_get_context (o);
+  SwfdecAsContext *cx = o->context;
   SwfdecAsValue val;
 
   /* FIXME: This function does not call valueOf in the right order */
   if (swfdec_as_object_get_variable (o, SWFDEC_AS_STR_matrixType, &val)) {
-    const char *s = swfdec_as_value_to_string (cx, &val);
+    const char *s = swfdec_as_value_to_string (cx, val);
     cairo_matrix_init_translate (mat, SWFDEC_TWIPS_SCALE_FACTOR / 2.0, SWFDEC_TWIPS_SCALE_FACTOR / 2.0);
     cairo_matrix_scale (mat, SWFDEC_TWIPS_SCALE_FACTOR / 32768.0, SWFDEC_TWIPS_SCALE_FACTOR / 32768.0);
     if (s == SWFDEC_AS_STR_box) {
       double x, y, w, h, r;
       cairo_matrix_t input;
       swfdec_as_object_get_variable (o, SWFDEC_AS_STR_x, &val);
-      x = swfdec_as_value_to_number (cx, &val);
+      x = swfdec_as_value_to_number (cx, val);
       swfdec_as_object_get_variable (o, SWFDEC_AS_STR_y, &val);
-      y = swfdec_as_value_to_number (cx, &val);
+      y = swfdec_as_value_to_number (cx, val);
       swfdec_as_object_get_variable (o, SWFDEC_AS_STR_w, &val);
-      w = swfdec_as_value_to_number (cx, &val);
+      w = swfdec_as_value_to_number (cx, val);
       swfdec_as_object_get_variable (o, SWFDEC_AS_STR_h, &val);
-      h = swfdec_as_value_to_number (cx, &val);
+      h = swfdec_as_value_to_number (cx, val);
       swfdec_as_object_get_variable (o, SWFDEC_AS_STR_r, &val);
-      r = swfdec_as_value_to_number (cx, &val);
+      r = swfdec_as_value_to_number (cx, val);
       cairo_matrix_init_translate (&input, (x + w) / 2, (y + h) / 2);
       cairo_matrix_scale (&input, w, h);
       cairo_matrix_rotate (&input, r);
@@ -160,17 +160,17 @@ swfdec_sprite_movie_extract_matrix (SwfdecAsObject *o, cairo_matrix_t *mat)
   } else {
     cairo_matrix_t input;
     swfdec_as_object_get_variable (o, SWFDEC_AS_STR_a, &val);
-    input.xx = swfdec_as_value_to_number (cx, &val);
+    input.xx = swfdec_as_value_to_number (cx, val);
     swfdec_as_object_get_variable (o, SWFDEC_AS_STR_b, &val);
-    input.yx = swfdec_as_value_to_number (cx, &val);
+    input.yx = swfdec_as_value_to_number (cx, val);
     swfdec_as_object_get_variable (o, SWFDEC_AS_STR_d, &val);
-    input.xy = swfdec_as_value_to_number (cx, &val);
+    input.xy = swfdec_as_value_to_number (cx, val);
     swfdec_as_object_get_variable (o, SWFDEC_AS_STR_e, &val);
-    input.yy = swfdec_as_value_to_number (cx, &val);
+    input.yy = swfdec_as_value_to_number (cx, val);
     swfdec_as_object_get_variable (o, SWFDEC_AS_STR_g, &val);
-    input.x0 = swfdec_as_value_to_number (cx, &val) * SWFDEC_TWIPS_SCALE_FACTOR;
+    input.x0 = swfdec_as_value_to_number (cx, val) * SWFDEC_TWIPS_SCALE_FACTOR;
     swfdec_as_object_get_variable (o, SWFDEC_AS_STR_h, &val);
-    input.y0 = swfdec_as_value_to_number (cx, &val) * SWFDEC_TWIPS_SCALE_FACTOR;
+    input.y0 = swfdec_as_value_to_number (cx, val) * SWFDEC_TWIPS_SCALE_FACTOR;
     cairo_matrix_init_scale (mat, SWFDEC_TWIPS_SCALE_FACTOR / 32768.0, SWFDEC_TWIPS_SCALE_FACTOR / 32768.0);
     cairo_matrix_multiply (mat, mat, &input);
   }
@@ -219,23 +219,23 @@ swfdec_sprite_movie_beginGradientFill (SwfdecAsContext *cx, SwfdecAsObject *obje
     if (check > i) {
       const char *name = swfdec_as_integer_to_string (cx, i);
       if (swfdec_as_object_get_variable (colors, name, &v)
-	  && SWFDEC_AS_VALUE_IS_NUMBER (&v))
-	c = swfdec_as_value_to_integer (cx, &v);
+	  && SWFDEC_AS_VALUE_IS_NUMBER (v))
+	c = swfdec_as_value_to_integer (cx, v);
       else
 	c = 0;
       if (!swfdec_as_object_get_variable (alphas, name, &v)) {
 	a = c;
-      } else if (!SWFDEC_AS_VALUE_IS_NUMBER (&v)) {
+      } else if (!SWFDEC_AS_VALUE_IS_NUMBER (v)) {
 	a = 0;
       } else {
-	a = swfdec_as_value_to_integer (cx, &v);
+	a = swfdec_as_value_to_integer (cx, v);
       }
       if (!swfdec_as_object_get_variable (ratios, name, &v))
 	r = CLAMP (a, 0, 255);
-      else if (!SWFDEC_AS_VALUE_IS_NUMBER (&v))
+      else if (!SWFDEC_AS_VALUE_IS_NUMBER (v))
 	r = 0;
       else
-	r = swfdec_as_value_to_integer (cx, &v);
+	r = swfdec_as_value_to_integer (cx, v);
     } else {
       c = a = r = 0;
     }
@@ -360,7 +360,7 @@ swfdec_sprite_movie_lineStyle (SwfdecAsContext *cx, SwfdecAsObject *object,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_MOVIE, &movie, "i|ii", &width, &color, &alpha);
 
   movie->draw_line = NULL;
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]))
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0]))
     return;
   if (argc > 3) {
     SWFDEC_FIXME ("implement Flash 8 arguments to lineStyle");
@@ -433,7 +433,7 @@ swfdec_sprite_movie_beginBitmapFill (SwfdecAsContext *cx,
     SwfdecAsValue *rval)
 {
   SwfdecMovie *movie;
-  SwfdecBitmapData *bitmap;
+  SwfdecAsObject *bitmap;
   SwfdecPattern *pattern;
   SwfdecDraw *draw;
   SwfdecAsObject *mat = NULL;
@@ -443,10 +443,10 @@ swfdec_sprite_movie_beginBitmapFill (SwfdecAsContext *cx,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_MOVIE, &movie, "O|Obb", 
       &bitmap, &mat, &repeat, &smoothing);
   movie->draw_fill = NULL;
-  if (!SWFDEC_IS_BITMAP_DATA (bitmap))
+  if (!SWFDEC_IS_BITMAP_DATA (bitmap->relay))
     return;
   
-  pattern = swfdec_bitmap_pattern_new (bitmap);
+  pattern = swfdec_bitmap_pattern_new (SWFDEC_BITMAP_DATA (bitmap->relay));
   /* NB: This signal assumes that the pattern is destroyed before the movie is,
    * because it is never removed anywhere */
   g_signal_connect_swapped (pattern, "invalidate", G_CALLBACK (swfdec_movie_invalidate_last), movie);
