@@ -1,5 +1,5 @@
 /* Swfdec
- * Copyright (C) 2007 Benjamin Otte <otte@gnome.org>
+ * Copyright (C) 2007-2008 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@
 #include "swfdec_access.h"
 #include "swfdec_amf.h"
 #include "swfdec_as_frame_internal.h"
+#include "swfdec_as_internal.h"
 #include "swfdec_as_strings.h"
 #include "swfdec_audio_flv.h"
 #include "swfdec_cached_video.h"
@@ -44,9 +45,11 @@ swfdec_net_stream_onstatus (SwfdecNetStream *stream, const char *code, const cha
 {
   SwfdecAsValue val;
   SwfdecAsObject *object;
+  SwfdecAsContext *cx;
 
+  cx = swfdec_gc_object_get_context (stream);
   swfdec_sandbox_use (stream->sandbox);
-  object = swfdec_as_object_new (swfdec_gc_object_get_context (stream));
+  object = swfdec_as_object_new (cx, SWFDEC_AS_STR_Object, NULL);
   SWFDEC_INFO ("emitting onStatus for %s %s", level, code);
   SWFDEC_AS_VALUE_SET_STRING (&val, code);
   swfdec_as_object_set_variable (object, SWFDEC_AS_STR_code, &val);
@@ -54,18 +57,18 @@ swfdec_net_stream_onstatus (SwfdecNetStream *stream, const char *code, const cha
   swfdec_as_object_set_variable (object, SWFDEC_AS_STR_level, &val);
 
   SWFDEC_AS_VALUE_SET_OBJECT (&val, object);
-  if (!swfdec_as_object_call (SWFDEC_AS_OBJECT (stream),
+  if (!swfdec_as_relay_call (SWFDEC_AS_RELAY (stream),
         SWFDEC_AS_STR_onStatus, 1, &val, NULL)) {
     // if it's an error message and the stream object didn't have onStatus
     // handler, call System.onStatus
     if (level == SWFDEC_AS_STR_error) {
       SwfdecAsValue system;
 
-      swfdec_as_object_get_variable (swfdec_gc_object_get_context (stream)->global,
+      swfdec_as_object_get_variable (cx->global,
           SWFDEC_AS_STR_System, &system);
-      if (SWFDEC_AS_VALUE_IS_OBJECT (&system)) {
-        swfdec_as_object_call (SWFDEC_AS_VALUE_GET_OBJECT (&system),
-              SWFDEC_AS_STR_onStatus, 1, &val, NULL);
+      if (SWFDEC_AS_VALUE_IS_COMPOSITE (system) &&
+	  (object = SWFDEC_AS_VALUE_GET_COMPOSITE (system)) != NULL) {
+        swfdec_as_object_call (object, SWFDEC_AS_STR_onStatus, 1, &val, NULL);
       }
     }
   }
@@ -100,11 +103,62 @@ swfdec_net_stream_decode_video (SwfdecVideoDecoder *decoder, SwfdecBuffer *buffe
       decoder->width -= wsub;
       decoder->height -= hsub;
     }
+  } else if (decoder->codec == SWFDEC_VIDEO_CODEC_H264) {
+    SwfdecBits bits;
+    guint type;
+    SwfdecBuffer *data;
+    swfdec_bits_init (&bits, buffer);
+    type = swfdec_bits_get_u8 (&bits);
+    /* composition_time_offset = */ swfdec_bits_get_bu24 (&bits);
+    switch (type) {
+      case 0:
+	SWFDEC_ERROR ("new data stream?!");
+	break;
+      case 1:
+	data = swfdec_bits_get_buffer (&bits, -1);
+	if (data) {
+	  swfdec_video_decoder_decode (decoder, data);
+	} else {
+	  SWFDEC_ERROR ("no data in H264 buffer?");
+	}
+	break;
+      case 2:
+	break;
+      default:
+	SWFDEC_ERROR ("H264 data type %u not supported", type);
+	break;
+    }
   } else {
     swfdec_video_decoder_decode (decoder, buffer);
   }
 }
 
+/* returns TRUE if the buffer was consumed */
+static gboolean
+swfdec_net_stream_new_video_decoder (SwfdecNetStream *stream, guint format, SwfdecBuffer *buffer)
+{
+  if (format == SWFDEC_VIDEO_CODEC_H264) {
+    SwfdecBits bits;
+
+    swfdec_bits_init (&bits, buffer);
+    if (swfdec_bits_get_u8 (&bits) == 0) {
+      SwfdecBuffer *data;
+      /* composition_time_offset = */ swfdec_bits_get_bu24 (&bits);
+      data = swfdec_bits_get_buffer (&bits, -1);
+      stream->decoder = swfdec_video_decoder_new (format, data);
+      if (data)
+	swfdec_buffer_unref (data);
+      return TRUE;
+    } else {
+      stream->decoder = swfdec_video_decoder_new (format, NULL);
+      return FALSE;
+    }
+  } else {
+    stream->decoder = swfdec_video_decoder_new (format, NULL);
+    return FALSE;
+  }
+}
+
 static void swfdec_net_stream_update_playing (SwfdecNetStream *stream);
 static void
 swfdec_net_stream_video_goto (SwfdecNetStream *stream, guint timestamp)
@@ -112,7 +166,7 @@ swfdec_net_stream_video_goto (SwfdecNetStream *stream, guint timestamp)
   SwfdecBuffer *buffer;
   guint format;
   cairo_surface_t *old;
-  gboolean process_events;
+  gboolean process_events, skip;
   guint process_events_from;
 
   SWFDEC_LOG ("goto %ums", timestamp);
@@ -140,18 +194,22 @@ swfdec_net_stream_video_goto (SwfdecNetStream *stream, guint timestamp)
       g_object_unref (stream->decoder);
       stream->decoder = NULL;
     }
+#if 0
     if (stream->decoder != NULL &&
 	(stream->decoder_time >= stream->current_time)) {
       g_object_unref (stream->decoder);
       stream->decoder = NULL;
     }
+#endif
 
+    skip = FALSE;
     if (stream->decoder == NULL) {
       buffer = swfdec_flv_decoder_get_video (stream->flvdecoder, 
 	  stream->current_time, TRUE, &format, &stream->decoder_time,
 	  &next);
-      stream->decoder = swfdec_video_decoder_new (format);
-    } else {
+      skip = !swfdec_net_stream_new_video_decoder (stream, format, buffer);
+    }
+    if (!skip) {
       swfdec_flv_decoder_get_video (stream->flvdecoder, 
 	  stream->decoder_time, FALSE, NULL, NULL, &next);
       if (next != stream->current_time) {
@@ -182,9 +240,10 @@ swfdec_net_stream_video_goto (SwfdecNetStream *stream, guint timestamp)
     for (;;) {
       if (format != swfdec_video_decoder_get_codec (stream->decoder)) {
 	g_object_unref (stream->decoder);
-	stream->decoder = swfdec_video_decoder_new (format);
+	skip = swfdec_net_stream_new_video_decoder (stream, format, buffer);
       }
-      swfdec_net_stream_decode_video (stream->decoder, buffer);
+      if (!skip)
+	swfdec_net_stream_decode_video (stream->decoder, buffer);
       if (stream->decoder_time >= stream->current_time)
 	break;
 
@@ -219,8 +278,8 @@ swfdec_net_stream_video_goto (SwfdecNetStream *stream, guint timestamp)
 	    SWFDEC_AMF_STRING, &name, SWFDEC_AMF_MIXED_ARRAY, &value) != 2) {
 	SWFDEC_ERROR ("could not parse data tag");
       } else {
-	swfdec_as_object_call (SWFDEC_AS_OBJECT (stream), 
-	    SWFDEC_AS_VALUE_GET_STRING (&name), 1, &value, NULL);
+	swfdec_as_relay_call (SWFDEC_AS_RELAY (stream), 
+	    SWFDEC_AS_VALUE_GET_STRING (name), 1, &value, NULL);
       }
       swfdec_sandbox_unuse (stream->sandbox);
     }
@@ -446,7 +505,7 @@ swfdec_net_stream_video_provider_init (SwfdecVideoProviderInterface *iface)
 
 /*** SWFDEC_NET_STREAM ***/
 
-G_DEFINE_TYPE_WITH_CODE (SwfdecNetStream, swfdec_net_stream, SWFDEC_TYPE_AS_OBJECT,
+G_DEFINE_TYPE_WITH_CODE (SwfdecNetStream, swfdec_net_stream, SWFDEC_TYPE_AS_RELAY,
     G_IMPLEMENT_INTERFACE (SWFDEC_TYPE_STREAM_TARGET, swfdec_net_stream_stream_target_init)
     G_IMPLEMENT_INTERFACE (SWFDEC_TYPE_VIDEO_PROVIDER, swfdec_net_stream_video_provider_init))
 
@@ -472,68 +531,6 @@ swfdec_net_stream_dispose (GObject *object)
   G_OBJECT_CLASS (swfdec_net_stream_parent_class)->dispose (object);
 }
 
-static gboolean
-swfdec_net_stream_get_variable (SwfdecAsObject *object, SwfdecAsObject *orig,
-    const char *variable, SwfdecAsValue *val, guint *flags)
-{
-  SwfdecNetStream *stream;
-
-  if (SWFDEC_AS_OBJECT_CLASS (swfdec_net_stream_parent_class)->get (object, orig, variable, val, flags))
-    return TRUE;
-
-  stream = SWFDEC_NET_STREAM (object);
-  /* FIXME: need case insensitive comparisons? */
-  if (variable == SWFDEC_AS_STR_time) {
-    guint msecs;
-    if (stream->flvdecoder == NULL ||
-	!swfdec_flv_decoder_get_video_info (stream->flvdecoder, &msecs, NULL)) {
-      SWFDEC_AS_VALUE_SET_INT (val, 0);
-    } else {
-      if (msecs >= stream->current_time)
-	msecs = 0;
-      else 
-	msecs = stream->current_time - msecs;
-      SWFDEC_AS_VALUE_SET_NUMBER (val, msecs / 1000.);
-    }
-    *flags = 0;
-    return TRUE;
-  } else if (variable == SWFDEC_AS_STR_bytesLoaded) {
-    if (stream->loader == NULL)
-      SWFDEC_AS_VALUE_SET_INT (val, 0);
-    else
-      SWFDEC_AS_VALUE_SET_NUMBER (val, swfdec_loader_get_loaded (stream->loader));
-    *flags = 0;
-    return TRUE;
-  } else if (variable == SWFDEC_AS_STR_bytesTotal) {
-    glong bytes;
-    if (stream->loader == NULL) {
-      bytes = 0;
-    } else { 
-      bytes = swfdec_loader_get_size (stream->loader);
-      if (bytes < 0)
-	bytes = swfdec_loader_get_loaded (stream->loader);
-    }
-    SWFDEC_AS_VALUE_SET_NUMBER (val, bytes);
-    *flags = 0;
-    return TRUE;
-  } else if (variable == SWFDEC_AS_STR_bufferLength) {
-    SWFDEC_STUB ("Netstream.bufferLength (get)");
-  } else if (variable == SWFDEC_AS_STR_bufferTime) {
-    SWFDEC_STUB ("Netstream.bufferTime (get)");
-  } else if (variable == SWFDEC_AS_STR_audiocodec) {
-    SWFDEC_STUB ("Netstream.audiocodec (get)");
-  } else if (variable == SWFDEC_AS_STR_currentFps) {
-    SWFDEC_STUB ("Netstream.currentFps (get)");
-  } else if (variable == SWFDEC_AS_STR_decodedFrames) {
-    SWFDEC_STUB ("Netstream.decodedFrames (get)");
-  } else if (variable == SWFDEC_AS_STR_liveDelay) {
-    SWFDEC_STUB ("Netstream.liveDelay (get)");
-  } else if (variable == SWFDEC_AS_STR_videoCodec) {
-    SWFDEC_STUB ("Netstream.videoCodec (get)");
-  }
-  return FALSE;
-}
-
 static void
 swfdec_net_stream_mark (SwfdecGcObject *object)
 {
@@ -552,13 +549,10 @@ swfdec_net_stream_class_init (SwfdecNetStreamClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (klass);
-  SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
 
   object_class->dispose = swfdec_net_stream_dispose;
 
   gc_class->mark = swfdec_net_stream_mark;
-
-  asobject_class->get = swfdec_net_stream_get_variable;
 }
 
 static void
@@ -614,7 +608,7 @@ swfdec_net_stream_set_url (SwfdecNetStream *stream, const char *url_string)
     return;
   }
   stream->requested_url = g_strdup (url_string);
-  stream->sandbox = SWFDEC_SANDBOX (SWFDEC_AS_CONTEXT (player)->global);
+  stream->sandbox = swfdec_sandbox_get (player);
 
   swfdec_player_allow_by_matrix (player, stream->sandbox, url_string, 
       swfdec_net_stream_matrix, swfdec_net_stream_load, stream);
@@ -703,10 +697,12 @@ swfdec_net_stream_seek (SwfdecNetStream *stream, double secs)
     SWFDEC_ERROR ("FIXME: implement seeking in audio only NetStream");
     return;
   }
+#if 0
   if (stream->decoder) {
     g_object_unref (stream->decoder);
     stream->decoder = NULL;
   }
+#endif
   msecs = secs * 1000;
   msecs += first;
   if (msecs > last)
