@@ -46,7 +46,7 @@
 
 
 static void swfdec_resource_stream_target_init (SwfdecStreamTargetInterface *iface);
-G_DEFINE_TYPE_WITH_CODE (SwfdecResource, swfdec_resource, SWFDEC_TYPE_AS_OBJECT,
+G_DEFINE_TYPE_WITH_CODE (SwfdecResource, swfdec_resource, SWFDEC_TYPE_GC_OBJECT,
     G_IMPLEMENT_INTERFACE (SWFDEC_TYPE_STREAM_TARGET, swfdec_resource_stream_target_init))
 
 /*** SWFDEC_STREAM_TARGET interface ***/
@@ -57,7 +57,7 @@ swfdec_resource_is_root (SwfdecResource *resource)
   g_return_val_if_fail (SWFDEC_IS_RESOURCE (resource), FALSE);
 
   return
-    resource->movie == SWFDEC_PLAYER (swfdec_gc_object_get_context (resource))->priv->roots->data;
+    resource == SWFDEC_MOVIE (SWFDEC_PLAYER (swfdec_gc_object_get_context (resource))->priv->roots->data)->resource;
 }
 
 static SwfdecPlayer *
@@ -70,9 +70,10 @@ static void
 swfdec_resource_stream_target_image (SwfdecResource *instance)
 {
   SwfdecPlayer *player = SWFDEC_PLAYER (swfdec_gc_object_get_context (instance));
-  SwfdecSpriteMovie *movie = instance->movie;
+  SwfdecSpriteMovie *movie = (SwfdecSpriteMovie *) SWFDEC_AS_VALUE_GET_MOVIE (instance->movie);
 
-  if (movie->sprite != NULL)
+  if (!SWFDEC_IS_SPRITE_MOVIE (movie) || movie->sprite != NULL ||
+      swfdec_movie_get_own_resource (SWFDEC_MOVIE (movie)) != instance)
     return;
 
   if (SWFDEC_IS_SWF_DECODER (instance->decoder)) {
@@ -88,7 +89,10 @@ swfdec_resource_stream_target_image (SwfdecResource *instance)
       g_assert (movie->sprite->parse_frame > 0);
       movie->n_frames = movie->sprite->n_frames;
       swfdec_movie_invalidate_last (SWFDEC_MOVIE (movie));
-      swfdec_as_object_set_constructor (SWFDEC_AS_OBJECT (movie), instance->sandbox->MovieClip);
+      swfdec_sandbox_use (instance->sandbox);
+      swfdec_as_object_set_constructor_by_name (swfdec_as_relay_get_as_object (
+	    SWFDEC_AS_RELAY (movie)), SWFDEC_AS_STR_MovieClip, NULL);
+      swfdec_sandbox_unuse (instance->sandbox);
       if (swfdec_resource_is_root (instance)) {
 	swfdec_player_start_ticking (player);
 	swfdec_movie_initialize (SWFDEC_MOVIE (movie));
@@ -121,18 +125,18 @@ swfdec_resource_emit_signal (SwfdecResource *resource, const char *name, gboolea
   cx = swfdec_gc_object_get_context (resource->clip_loader);
 
   SWFDEC_AS_VALUE_SET_STRING (&vals[0], name);
-  if (resource->movie) {
-    movie = swfdec_movie_resolve (SWFDEC_MOVIE (resource->movie));
+  if (SWFDEC_AS_VALUE_IS_MOVIE (resource->movie)) {
+    movie = SWFDEC_AS_VALUE_GET_MOVIE (resource->movie);
     if (movie == NULL) {
       SWFDEC_DEBUG ("no movie, not emitting signal");
       return;
     }
     if (name == SWFDEC_AS_STR_onLoadInit &&
-	movie != SWFDEC_MOVIE (resource->movie)) {
+	movie->as_value != SWFDEC_AS_VALUE_GET_VALUE (resource->movie)) {
       SWFDEC_INFO ("not emitting onLoadInit - the movie is different");
       return;
     }
-    SWFDEC_AS_VALUE_SET_OBJECT (&vals[1], SWFDEC_AS_OBJECT (movie));
+    SWFDEC_AS_VALUE_SET_MOVIE (&vals[1], movie);
   } else {
     SWFDEC_AS_VALUE_SET_UNDEFINED (&vals[1]);
     movie = NULL;
@@ -146,18 +150,18 @@ swfdec_resource_emit_signal (SwfdecResource *resource, const char *name, gboolea
       res = NULL;
     if (res && res->decoder) {
       SwfdecDecoder *dec = res->decoder;
-      SWFDEC_AS_VALUE_SET_INT (&vals[2], dec->bytes_loaded);
-      SWFDEC_AS_VALUE_SET_INT (&vals[3], dec->bytes_total);
+      vals[2] = swfdec_as_value_from_integer (cx, dec->bytes_loaded);
+      vals[3] = swfdec_as_value_from_integer (cx, dec->bytes_total);
     } else {
-      SWFDEC_AS_VALUE_SET_INT (&vals[2], 0);
-      SWFDEC_AS_VALUE_SET_INT (&vals[3], 0);
+      vals[2] = swfdec_as_value_from_integer (cx, 0);
+      vals[3] = swfdec_as_value_from_integer (cx, 0);
     }
   }
   if (n_args)
     memcpy (&vals[skip], args, sizeof (SwfdecAsValue) * n_args);
   /* FIXME: what's the correct sandbox here? */
   swfdec_sandbox_use (resource->clip_loader_sandbox);
-  swfdec_as_object_call (SWFDEC_AS_OBJECT (resource->clip_loader), SWFDEC_AS_STR_broadcastMessage, 
+  swfdec_as_relay_call (SWFDEC_AS_RELAY (resource->clip_loader), SWFDEC_AS_STR_broadcastMessage, 
       n_args + skip, vals, NULL);
   swfdec_sandbox_unuse (resource->clip_loader_sandbox);
 }
@@ -168,7 +172,7 @@ swfdec_resource_emit_error (SwfdecResource *resource, const char *message)
   SwfdecAsValue vals[2];
 
   SWFDEC_AS_VALUE_SET_STRING (&vals[0], message);
-  SWFDEC_AS_VALUE_SET_INT (&vals[1], 0);
+  vals[1] = swfdec_as_value_from_integer (swfdec_gc_object_get_context (resource), 0);
 
   swfdec_resource_emit_signal (resource, SWFDEC_AS_STR_onLoadError, FALSE, vals, 2);
 }
@@ -182,11 +186,8 @@ swfdec_resource_replace_movie (SwfdecSpriteMovie *movie, SwfdecResource *resourc
   
   copy = swfdec_movie_new (SWFDEC_PLAYER (swfdec_gc_object_get_context (movie)), 
       mov->depth, mov->parent, resource, NULL, mov->name);
-  if (copy == NULL)
-    return FALSE;
   swfdec_movie_begin_update_matrix (copy);
   copy->matrix = mov->matrix;
-  copy->original_name = mov->original_name;
   copy->modified = mov->modified;
   copy->xscale = mov->xscale;
   copy->yscale = mov->yscale;
@@ -206,17 +207,22 @@ swfdec_resource_stream_target_open (SwfdecStreamTarget *target, SwfdecStream *st
 {
   SwfdecLoader *loader = SWFDEC_LOADER (stream);
   SwfdecResource *instance = SWFDEC_RESOURCE (target);
+  SwfdecAsObject *object;
+  SwfdecMovie *movie;
   const char *query;
 
-  g_assert (instance->movie);
+  g_assert (SWFDEC_AS_VALUE_IS_MOVIE (instance->movie));
+  movie = SWFDEC_AS_VALUE_GET_MOVIE (instance->movie);
+  g_assert (movie);
+  object = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (movie));
   query = swfdec_url_get_query (swfdec_loader_get_url (loader));
   if (query) {
     SWFDEC_INFO ("set url query movie variables: %s", query);
-    swfdec_as_object_decode (SWFDEC_AS_OBJECT (instance->movie), query);
+    swfdec_as_object_decode (object, query);
   }
   if (instance->variables) {
     SWFDEC_INFO ("set manual movie variables: %s", instance->variables);
-    swfdec_as_object_decode (SWFDEC_AS_OBJECT (instance->movie), instance->variables);
+    swfdec_as_object_decode (object, instance->variables);
   }
   swfdec_resource_emit_signal (instance, SWFDEC_AS_STR_onLoadStart, FALSE, NULL, 0);
   instance->state = SWFDEC_RESOURCE_OPENED;
@@ -320,14 +326,17 @@ swfdec_resource_stream_target_close (SwfdecStreamTarget *target, SwfdecStream *s
       swfdec_loader_set_data_type (SWFDEC_LOADER (stream), dec->data_type);
   }
 
-  SWFDEC_AS_VALUE_SET_INT (&val, 0); /* FIXME */
+  val = swfdec_as_value_from_integer (swfdec_gc_object_get_context (resource), 0); /* FIXME */
   swfdec_resource_emit_signal (resource, SWFDEC_AS_STR_onLoadComplete, FALSE, &val, 1);
   resource->state = SWFDEC_RESOURCE_COMPLETE;
   if (swfdec_resource_abort_if_not_initialized (resource))
     return;
 
-  if (resource->movie != NULL)
-    swfdec_actor_queue_script (SWFDEC_ACTOR (resource->movie), SWFDEC_EVENT_LOAD);
+  if (SWFDEC_AS_VALUE_IS_MOVIE (resource->movie)) {
+    SwfdecMovie *movie = SWFDEC_AS_VALUE_GET_MOVIE (resource->movie);
+    if (movie)
+      swfdec_actor_queue_script (SWFDEC_ACTOR (movie), SWFDEC_EVENT_LOAD);
+  }
 }
 
 static void
@@ -378,6 +387,7 @@ swfdec_resource_mark (SwfdecGcObject *object)
     swfdec_gc_object_mark (resource->sandbox);
   if (resource->target)
     swfdec_gc_object_mark (resource->target);
+  swfdec_as_value_mark (&resource->movie);
 
   SWFDEC_GC_OBJECT_CLASS (swfdec_resource_parent_class)->mark (object);
 }
@@ -487,8 +497,7 @@ swfdec_resource_add_export (SwfdecResource *instance, SwfdecCharacter *character
 typedef struct _SwfdecResourceLoad SwfdecResourceLoad;
 struct _SwfdecResourceLoad {
   SwfdecSandbox *		sandbox;
-  char *			target_string;
-  SwfdecSpriteMovie *		target_movie;
+  SwfdecAsValue			target;
   char *			url;
   SwfdecBuffer *		buffer;
   SwfdecMovieClipLoader *	loader;
@@ -500,7 +509,6 @@ swfdec_resource_load_free (gpointer loadp)
   SwfdecResourceLoad *load = loadp;
 
   g_free (load->url);
-  g_free (load->target_string);
   if (load->buffer)
     swfdec_buffer_unref (load->buffer);
   g_slice_free (SwfdecResourceLoad, load);
@@ -514,8 +522,7 @@ swfdec_resource_load_mark (gpointer loadp, gpointer playerp)
   swfdec_gc_object_mark (load->sandbox);
   if (load->loader)
     swfdec_gc_object_mark (load->loader);
-  if (load->target_movie)
-    swfdec_gc_object_mark (load->target_movie);
+  swfdec_as_value_mark (&load->target);
 }
 
 static gboolean
@@ -524,17 +531,17 @@ swfdec_resource_create_movie (SwfdecResource *resource, SwfdecResourceLoad *load
   SwfdecPlayer *player;
   SwfdecSpriteMovie *movie;
 
-  if (resource->movie)
+  if (SWFDEC_AS_VALUE_IS_MOVIE (resource->movie))
     return TRUE;
   player = SWFDEC_PLAYER (swfdec_gc_object_get_context (resource));
-  if (load->target_movie) {
-    movie = (SwfdecSpriteMovie *) swfdec_movie_resolve (SWFDEC_MOVIE (load->target_movie));
+  if (SWFDEC_AS_VALUE_IS_MOVIE (load->target)) {
+    movie = (SwfdecSpriteMovie *) SWFDEC_AS_VALUE_GET_MOVIE (load->target);
     if (SWFDEC_IS_SPRITE_MOVIE (movie))
       movie = swfdec_resource_replace_movie (movie, resource);
     else
       movie = NULL;
-  } else {
-    int level = swfdec_player_get_level (player, load->target_string, 7);
+  } else if (SWFDEC_AS_VALUE_IS_STRING (load->target)) {
+    int level = swfdec_player_get_level (player, SWFDEC_AS_VALUE_GET_STRING (load->target), 7);
     if (level >= 0) {
       movie = swfdec_player_get_movie_at_level (player, level);
       if (movie)
@@ -543,6 +550,9 @@ swfdec_resource_create_movie (SwfdecResource *resource, SwfdecResourceLoad *load
     } else {
       movie = NULL;
     }
+  } else {
+    /* we only set target to string or movie values */
+    g_assert_not_reached ();
   }
   if (movie == NULL) {
     SWFDEC_WARNING ("target does not reference a movie, not loading %s", load->url);
@@ -571,8 +581,10 @@ swfdec_resource_do_load (SwfdecPlayer *player, gboolean allowed, gpointer loadp)
     SWFDEC_WARNING ("SECURITY: no access to %s from %s",
 	load->url, swfdec_url_get_url (load->sandbox->url));
     /* FIXME: is replacing correct? */
-    if (load->target_movie) {
-      resource->movie = SWFDEC_SPRITE_MOVIE (swfdec_movie_resolve (SWFDEC_MOVIE (load->target_movie)));
+    if (SWFDEC_AS_VALUE_IS_MOVIE (load->target)) {
+      if (SWFDEC_AS_VALUE_IS_MOVIE (load->target) &&
+	  SWFDEC_IS_SPRITE_MOVIE (SWFDEC_AS_VALUE_GET_MOVIE (load->target)))
+	resource->movie = load->target;
       swfdec_resource_emit_error (resource, SWFDEC_AS_STR_IllegalRequest);
     }
     swfdec_player_unroot (player, load);
@@ -606,7 +618,11 @@ swfdec_resource_load_request (gpointer loadp, gpointer playerp)
   if (load->url[0] == '\0') {
     SwfdecSpriteMovie *movie;
       
-    movie = load->target_movie ? (SwfdecSpriteMovie *) swfdec_movie_resolve (SWFDEC_MOVIE (load->target_movie)) : NULL;
+    if (SWFDEC_AS_VALUE_IS_MOVIE (load->target)) {
+      movie = (SwfdecSpriteMovie *) SWFDEC_AS_VALUE_GET_MOVIE (load->target);
+    } else {
+      movie = NULL;
+    }
     if (SWFDEC_IS_SPRITE_MOVIE (movie)) {
       swfdec_resource_replace_movie (movie, SWFDEC_MOVIE (movie)->resource);
     } else {
@@ -619,22 +635,20 @@ swfdec_resource_load_request (gpointer loadp, gpointer playerp)
   /* fscommand? */
   if (g_ascii_strncasecmp (load->url, "FSCommand:", 10) == 0) {
     char *command = load->url + 10;
-    if (load->target_movie) {
-      char *target = swfdec_movie_get_path (SWFDEC_MOVIE (load->target_movie), TRUE);
+    const char *target = swfdec_as_value_to_string (SWFDEC_AS_CONTEXT (player), load->target);
+    if (SWFDEC_AS_VALUE_IS_MOVIE (load->target)) {
       SWFDEC_FIXME ("Adobe 9.0.124.0 and later don't emit fscommands here. "
 	  "We just do for compatibility reasons with the testsuite.");
-      g_signal_emit_by_name (player, "fscommand", command, target);
-      g_free (target);
-    } else {
-      g_signal_emit_by_name (player, "fscommand", command, load->target_string);
     }
+    g_signal_emit_by_name (player, "fscommand", command, target);
     swfdec_player_unroot (player, load);
     return;
   }
 
   /* LAUNCH command (aka getURL) */
-  if (load->target_string && swfdec_player_get_level (player, load->target_string, 7) < 0) {
-    swfdec_player_launch (player, load->url, load->target_string, load->buffer);
+  if (SWFDEC_AS_VALUE_IS_STRING (load->target) && swfdec_player_get_level (player, 
+	SWFDEC_AS_VALUE_GET_STRING (load->target), 7) < 0) {
+    swfdec_player_launch (player, load->url, SWFDEC_AS_VALUE_GET_STRING (load->target), load->buffer);
     swfdec_player_unroot (player, load);
     return;
   }
@@ -644,10 +658,9 @@ swfdec_resource_load_request (gpointer loadp, gpointer playerp)
 }
 
 /* NB: must be called from a script */
-/* FIXME: 6 arguments?! */
+/* FIXME: 5 arguments?! */
 static void
-swfdec_resource_load_internal (SwfdecPlayer *player,
-    SwfdecSpriteMovie *target_movie, const char *target_string,
+swfdec_resource_load_internal (SwfdecPlayer *player, const SwfdecAsValue *val,
     const char *url, SwfdecBuffer *buffer, SwfdecMovieClipLoader *loader)
 {
   SwfdecResourceLoad *load;
@@ -655,10 +668,9 @@ swfdec_resource_load_internal (SwfdecPlayer *player,
   g_assert (SWFDEC_AS_CONTEXT (player)->frame != NULL);
   load = g_slice_new (SwfdecResourceLoad);
 
-  load->sandbox = SWFDEC_SANDBOX (SWFDEC_AS_CONTEXT (player)->global);
+  load->sandbox = swfdec_sandbox_get (player);
   load->url = g_strdup (url);
-  load->target_movie = target_movie;
-  load->target_string = g_strdup (target_string);
+  load->target = *val;
   load->buffer = buffer;
   load->loader = loader;
 
@@ -671,6 +683,7 @@ swfdec_resource_load_movie (SwfdecPlayer *player, const SwfdecAsValue *target,
     const char *url, SwfdecBuffer *buffer, SwfdecMovieClipLoader *loader)
 {
   SwfdecMovie *movie;
+  SwfdecAsValue val;
   const char *s;
 
   g_return_val_if_fail (SWFDEC_IS_PLAYER (player), FALSE);
@@ -678,45 +691,44 @@ swfdec_resource_load_movie (SwfdecPlayer *player, const SwfdecAsValue *target,
   g_return_val_if_fail (url != NULL, FALSE);
   g_return_val_if_fail (loader == NULL || SWFDEC_IS_MOVIE_CLIP_LOADER (loader), FALSE);
 
-  if (SWFDEC_AS_VALUE_IS_OBJECT (target)) {
-    SwfdecAsObject *object = SWFDEC_AS_VALUE_GET_OBJECT (target);
-    if (SWFDEC_IS_SPRITE_MOVIE (object)) {
-      swfdec_resource_load_internal (player, SWFDEC_SPRITE_MOVIE (object),
-	  NULL, url, buffer, loader);
+  if (SWFDEC_AS_VALUE_IS_MOVIE (*target)) {
+    movie = SWFDEC_AS_VALUE_GET_MOVIE (*target);
+    if (SWFDEC_IS_SPRITE_MOVIE (movie)) {
+      swfdec_resource_load_internal (player, target, url, buffer, loader);
       return TRUE;
     }
   }
 
   if (loader) {
-    if (SWFDEC_AS_VALUE_IS_NUMBER (target)) {
-      int i = swfdec_as_double_to_integer (SWFDEC_AS_VALUE_GET_NUMBER (target));
+    if (SWFDEC_AS_VALUE_IS_NUMBER (*target)) {
+      int i = swfdec_as_double_to_integer (SWFDEC_AS_VALUE_GET_NUMBER (*target));
       if (i < 0)
 	return FALSE;
-      s = swfdec_as_context_give_string (swfdec_gc_object_get_context (loader),
-	  g_strdup_printf ("_level%d", i));
-      swfdec_resource_load_internal (player, NULL, s, url, buffer, loader);
+      SWFDEC_AS_VALUE_SET_STRING (&val, swfdec_as_context_give_string (
+	    SWFDEC_AS_CONTEXT (player), g_strdup_printf ("_level%d", i)));
+      swfdec_resource_load_internal (player, &val, url, buffer, loader);
       return TRUE;
-    } else if (SWFDEC_AS_VALUE_IS_STRING (target) ||
-	(SWFDEC_AS_VALUE_IS_OBJECT (target) && SWFDEC_IS_MOVIE (SWFDEC_AS_VALUE_GET_OBJECT (target)))) {
-      s = swfdec_as_value_to_string (SWFDEC_AS_CONTEXT (player), target);
+    } else if (SWFDEC_AS_VALUE_IS_STRING (*target) || SWFDEC_AS_VALUE_IS_MOVIE (*target)) {
+      s = swfdec_as_value_to_string (SWFDEC_AS_CONTEXT (player), *target);
     } else {
       SWFDEC_WARNING ("target does not reference a movie, not loading %s", url);
       return FALSE;
     }
   } else {
-    s = swfdec_as_value_to_string (SWFDEC_AS_CONTEXT (player), target);
+    s = swfdec_as_value_to_string (SWFDEC_AS_CONTEXT (player), *target);
   }
   if (swfdec_player_get_level (player, s, SWFDEC_AS_CONTEXT (player)->version) >= 0) {
     /* lowercase the string, so we can do case insensitive level lookups later on */
     char *tmp = g_ascii_strdown (s, -1);
-    swfdec_resource_load_internal (player, NULL, tmp, url, buffer, NULL);
-    g_free (tmp);
+    SWFDEC_AS_VALUE_SET_STRING (&val, swfdec_as_context_give_string (
+	  SWFDEC_AS_CONTEXT (player), tmp));
+    swfdec_resource_load_internal (player, &val, url, buffer, NULL);
     return TRUE;
   }
   movie = swfdec_player_get_movie_from_string (player, s);
   if (SWFDEC_IS_SPRITE_MOVIE (movie)) {
-    swfdec_resource_load_internal (player, SWFDEC_SPRITE_MOVIE (movie),
-	NULL, url, buffer, loader);
+    SWFDEC_AS_VALUE_SET_MOVIE (&val, movie);
+    swfdec_resource_load_internal (player, &val, url, buffer, loader);
     return TRUE;
   }
   SWFDEC_WARNING ("%s does not reference a movie, not loading %s", s, url);
@@ -727,11 +739,14 @@ void
 swfdec_resource_load (SwfdecPlayer *player, const char *target, 
     const char *url, SwfdecBuffer *buffer)
 {
+  SwfdecAsValue val;
+
   g_return_if_fail (SWFDEC_IS_PLAYER (player));
   g_return_if_fail (target != NULL);
   g_return_if_fail (url != NULL);
 
-  swfdec_resource_load_internal (player, NULL, target, url, buffer, NULL);
+  SWFDEC_AS_VALUE_SET_STRING (&val, target);
+  swfdec_resource_load_internal (player, &val, url, buffer, NULL);
 }
 
 gboolean
@@ -742,12 +757,13 @@ swfdec_resource_emit_on_load_init (SwfdecResource *resource)
   if (resource->state != SWFDEC_RESOURCE_COMPLETE)
     return FALSE;
 
-  if (resource->movie && SWFDEC_IS_IMAGE_DECODER (resource->decoder)) {
+  if (SWFDEC_AS_VALUE_IS_MOVIE (resource->movie) && SWFDEC_IS_IMAGE_DECODER (resource->decoder)) {
     SwfdecImage *image = SWFDEC_IMAGE_DECODER (resource->decoder)->image;
-    if (image) {
-      swfdec_movie_invalidate_next (SWFDEC_MOVIE (resource->movie));
-      swfdec_movie_queue_update (SWFDEC_MOVIE (resource->movie), SWFDEC_MOVIE_INVALID_EXTENTS);
-      SWFDEC_MOVIE (resource->movie)->image = g_object_ref (image);
+    SwfdecMovie *movie = SWFDEC_AS_VALUE_GET_MOVIE (resource->movie);
+    if (image && movie) {
+      swfdec_movie_invalidate_next (movie);
+      swfdec_movie_queue_update (movie, SWFDEC_MOVIE_INVALID_EXTENTS);
+      movie->image = g_object_ref (image);
     }
   }
   swfdec_resource_emit_signal (resource, SWFDEC_AS_STR_onLoadInit, FALSE, NULL, 0);
