@@ -65,9 +65,8 @@ set_scaleMode (SwfdecAsContext *cx, SwfdecAsObject *object,
   const char *s;
   SwfdecScaleMode mode;
 
-  if (argc == 0)
-    return;
-  s = swfdec_as_value_to_string (cx, &argv[0]);
+  SWFDEC_AS_CHECK (0, NULL, "s", &s);
+
   if (g_ascii_strcasecmp (s, SWFDEC_AS_STR_noBorder) == 0) {
     mode = SWFDEC_SCALE_NO_BORDER;
   } else if (g_ascii_strcasecmp (s, SWFDEC_AS_STR_exactFit) == 0) {
@@ -111,10 +110,8 @@ set_align (SwfdecAsContext *cx, SwfdecAsObject *object,
   guint flags = 0;
   const char *s;
 
-  if (argc == 0)
-    return;
+  SWFDEC_AS_CHECK (0, NULL, "s", &s);
 
-  s = swfdec_as_value_to_string (cx, &argv[0]);
   if (strchr (s, 'l') || strchr (s, 'L'))
     flags |= SWFDEC_ALIGN_FLAG_LEFT;
   if (strchr (s, 't') || strchr (s, 'T'))
@@ -139,9 +136,9 @@ get_width (SwfdecAsContext *cx, SwfdecAsObject *object,
   SwfdecPlayerPrivate *priv = SWFDEC_PLAYER (cx)->priv;
 
   if (priv->scale_mode == SWFDEC_SCALE_NONE)
-    SWFDEC_AS_VALUE_SET_INT (ret, priv->internal_width);
+    *ret = swfdec_as_value_from_integer (cx, priv->internal_width);
   else
-    SWFDEC_AS_VALUE_SET_INT (ret, priv->width);
+    *ret = swfdec_as_value_from_integer (cx, priv->width);
 }
 
 SWFDEC_AS_NATIVE (666, 7, get_height)
@@ -152,9 +149,9 @@ get_height (SwfdecAsContext *cx, SwfdecAsObject *object,
   SwfdecPlayerPrivate *priv = SWFDEC_PLAYER (cx)->priv;
 
   if (priv->scale_mode == SWFDEC_SCALE_NONE)
-    SWFDEC_AS_VALUE_SET_INT (ret, priv->internal_height);
+    *ret = swfdec_as_value_from_integer (cx, priv->internal_height);
   else
-    SWFDEC_AS_VALUE_SET_INT (ret, priv->height);
+    *ret = swfdec_as_value_from_integer (cx, priv->height);
 }
 
 /* FIXME: do this smarter */
