@@ -165,6 +165,7 @@ swfdec_text_field_movie_autosize (SwfdecTextFieldMovie *text)
       break;
     case SWFDEC_AUTO_SIZE_NONE:
     default:
+      x0 = 0;
       g_assert_not_reached ();
   }
   z0 = 0;
@@ -207,8 +208,8 @@ swfdec_text_field_movie_invalidate (SwfdecMovie *movie, const cairo_matrix_t *ma
 
   swfdec_rect_transform (&rect, &rect,
       &SWFDEC_PLAYER (swfdec_gc_object_get_context (text))->priv->stage_to_global);
-  swfdec_player_invalidate (
-      SWFDEC_PLAYER (swfdec_gc_object_get_context (movie)), &rect);
+  swfdec_player_invalidate (SWFDEC_PLAYER (swfdec_gc_object_get_context (movie)),
+      movie, &rect);
 }
 
 static gboolean
@@ -288,8 +289,8 @@ swfdec_text_field_movie_dispose (GObject *object)
   text = SWFDEC_TEXT_FIELD_MOVIE (object);
 
   if (text->style_sheet) {
-    if (SWFDEC_IS_STYLESHEET (text->style_sheet)) {
-      g_signal_handlers_disconnect_matched (text->style_sheet, 
+    if (SWFDEC_IS_STYLE_SHEET (text->style_sheet->relay)) {
+      g_signal_handlers_disconnect_matched (text->style_sheet->relay, 
 	  G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, text);
     }
     text->style_sheet = NULL;
@@ -321,7 +322,7 @@ swfdec_text_field_movie_mark (SwfdecGcObject *object)
 
   swfdec_text_buffer_mark (text->text);
   if (text->style_sheet != NULL)
-    swfdec_gc_object_mark (text->style_sheet);
+    swfdec_as_object_mark (text->style_sheet);
   if (text->style_sheet_input != NULL)
     swfdec_as_string_mark (text->style_sheet_input);
   if (text->restrict_ != NULL)
@@ -395,24 +396,22 @@ swfdec_text_field_movie_init_movie (SwfdecMovie *movie)
   needs_unuse = swfdec_sandbox_try_use (movie->resource->sandbox);
 
   cx = swfdec_gc_object_get_context (movie);
+  if (movie->resource->version > 5) {
+    SwfdecAsObject *o = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (movie));
 
-  swfdec_text_field_movie_init_properties (cx);
+    swfdec_text_field_movie_init_properties (cx);
 
-  swfdec_as_object_set_constructor_by_name (SWFDEC_AS_OBJECT (movie),
-      SWFDEC_AS_STR_TextField, NULL);
+    swfdec_as_object_set_constructor_by_name (o,
+	SWFDEC_AS_STR_TextField, NULL);
 
-  /* create _listeners array */
-  array = swfdec_as_array_new (cx);
-  SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (movie));
-  swfdec_as_array_push (SWFDEC_AS_ARRAY (array), &val);
-  SWFDEC_AS_VALUE_SET_OBJECT (&val, array);
-  swfdec_as_object_set_variable_and_flags (SWFDEC_AS_OBJECT (movie), SWFDEC_AS_STR__listeners, 
-      &val, SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
-
-  /* add self to listeners */
-  SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (movie));
-  swfdec_as_object_call (SWFDEC_AS_OBJECT (movie), SWFDEC_AS_STR_addListener,
-      1, &val, NULL);
+    /* create _listeners array containing self */
+    array = swfdec_as_array_new (cx);
+    SWFDEC_AS_VALUE_SET_MOVIE (&val, movie);
+    swfdec_as_array_push (array, &val);
+    SWFDEC_AS_VALUE_SET_OBJECT (&val, array);
+    swfdec_as_object_set_variable_and_flags (o, SWFDEC_AS_STR__listeners, 
+	&val, SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
+  }
 
   text->border_color = SWFDEC_COLOR_COMBINE (0, 0, 0, 0);
   text->background_color = SWFDEC_COLOR_COMBINE (255, 255, 255, 0);
@@ -546,9 +545,8 @@ swfdec_text_field_movie_parse_listen_variable (SwfdecTextFieldMovie *text,
   if (SWFDEC_MOVIE (text)->parent == NULL)
     return;
 
-  g_assert (SWFDEC_IS_AS_OBJECT (SWFDEC_MOVIE (text)->parent));
   cx = swfdec_gc_object_get_context (text);
-  parent = SWFDEC_AS_OBJECT (SWFDEC_MOVIE (text)->parent);
+  parent = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (SWFDEC_MOVIE (text)->parent));
 
   p1 = strrchr (variable, '.');
   p2 = strrchr (variable, ':');
@@ -606,7 +604,7 @@ swfdec_text_field_movie_asfunction (SwfdecTextFieldMovie *text,
   } else {
     swfdec_as_object_call (object, name, 0, NULL, NULL);
   }
-  swfdec_sandbox_use (SWFDEC_MOVIE (text)->resource->sandbox);
+  swfdec_sandbox_unuse (SWFDEC_MOVIE (text)->resource->sandbox);
 
   g_strfreev (parts);
 }
@@ -853,11 +851,11 @@ swfdec_text_field_movie_key_release (SwfdecActor *actor, guint keycode, guint ch
   //SwfdecTextFieldMovie *text = SWFDEC_TEXT_FIELD_MOVIE (actor);
 }
 
-static void
-swfdec_text_field_movie_property_get (SwfdecMovie *movie, guint prop_id, 
-    SwfdecAsValue *val)
+static SwfdecAsValue
+swfdec_text_field_movie_property_get (SwfdecMovie *movie, guint prop_id)
 {
   SwfdecTextFieldMovie *text = SWFDEC_TEXT_FIELD_MOVIE (movie);
+  SwfdecAsContext *cx = swfdec_gc_object_get_context (text);
   double d;
 
   switch (prop_id) {
@@ -865,14 +863,12 @@ swfdec_text_field_movie_property_get (SwfdecMovie *movie, guint prop_id,
       swfdec_text_field_movie_autosize (text);
       swfdec_movie_update (movie);
       d = SWFDEC_TWIPS_TO_DOUBLE (movie->matrix.x0 + text->extents.x0);
-      SWFDEC_AS_VALUE_SET_NUMBER (val, d);
-      return;
+      return swfdec_as_value_from_number (cx, d);
     case SWFDEC_MOVIE_PROPERTY_Y:
       swfdec_text_field_movie_autosize (text);
       swfdec_movie_update (movie);
       d = SWFDEC_TWIPS_TO_DOUBLE (movie->matrix.y0 + text->extents.y0);
-      SWFDEC_AS_VALUE_SET_NUMBER (val, d);
-      return;
+      return swfdec_as_value_from_number (cx, d);
     case SWFDEC_MOVIE_PROPERTY_WIDTH:
     case SWFDEC_MOVIE_PROPERTY_HEIGHT:
       swfdec_text_field_movie_autosize (text);
@@ -881,12 +877,12 @@ swfdec_text_field_movie_property_get (SwfdecMovie *movie, guint prop_id,
       break;
   }
 
-  SWFDEC_MOVIE_CLASS (swfdec_text_field_movie_parent_class)->property_get (movie, prop_id, val);
+  return SWFDEC_MOVIE_CLASS (swfdec_text_field_movie_parent_class)->property_get (movie, prop_id);
 }
 
 static void
 swfdec_text_field_movie_property_set (SwfdecMovie *movie, guint prop_id, 
-    const SwfdecAsValue *val)
+    SwfdecAsValue val)
 {
   SwfdecTextFieldMovie *text = SWFDEC_TEXT_FIELD_MOVIE (movie);
   SwfdecAsContext *cx = swfdec_gc_object_get_context (movie);
@@ -894,7 +890,7 @@ swfdec_text_field_movie_property_set (SwfdecMovie *movie, guint prop_id,
 
   switch (prop_id) {
     case SWFDEC_MOVIE_PROPERTY_X:
-      if (!swfdec_as_value_to_twips (swfdec_gc_object_get_context (movie), val, FALSE, &twips))
+      if (!swfdec_as_value_to_twips (swfdec_gc_object_get_context (movie), &val, FALSE, &twips))
 	return;
       movie->modified = TRUE;
       twips -= text->extents.x0;
@@ -905,7 +901,7 @@ swfdec_text_field_movie_property_set (SwfdecMovie *movie, guint prop_id,
       }
       return;
     case SWFDEC_MOVIE_PROPERTY_Y:
-      if (!swfdec_as_value_to_twips (swfdec_gc_object_get_context (movie), val, FALSE, &twips))
+      if (!swfdec_as_value_to_twips (swfdec_gc_object_get_context (movie), &val, FALSE, &twips))
 	return;
       movie->modified = TRUE;
       twips -= text->extents.y0;
@@ -916,7 +912,7 @@ swfdec_text_field_movie_property_set (SwfdecMovie *movie, guint prop_id,
       }
       return;
     case SWFDEC_MOVIE_PROPERTY_WIDTH:
-      if (swfdec_as_value_to_twips (cx, val, TRUE, &twips)) {
+      if (swfdec_as_value_to_twips (cx, &val, TRUE, &twips)) {
 	movie->modified = TRUE;
 	if (text->extents.x1 != text->extents.x0 + twips) {
 	  swfdec_movie_invalidate_next (movie);
@@ -928,7 +924,7 @@ swfdec_text_field_movie_property_set (SwfdecMovie *movie, guint prop_id,
       return;
     case SWFDEC_MOVIE_PROPERTY_HEIGHT:
       movie->modified = TRUE;
-      if (swfdec_as_value_to_twips (cx, val, TRUE, &twips)) {
+      if (swfdec_as_value_to_twips (cx, &val, TRUE, &twips)) {
 	movie->modified = TRUE;
 	if (text->extents.y1 != text->extents.y0 + twips) {
 	  swfdec_movie_invalidate_next (movie);
@@ -1031,16 +1027,13 @@ swfdec_text_field_movie_set_listen_variable_text (SwfdecTextFieldMovie *text,
 }
 
 static void
-swfdec_text_field_movie_variable_listener_callback (SwfdecAsObject *object,
+swfdec_text_field_movie_variable_listener_callback (gpointer data,
     const char *name, const SwfdecAsValue *val)
 {
-  SwfdecTextFieldMovie *text;
+  SwfdecTextFieldMovie *text = SWFDEC_TEXT_FIELD_MOVIE (data);
 
-  g_return_if_fail (SWFDEC_IS_TEXT_FIELD_MOVIE (object));
-
-  text = SWFDEC_TEXT_FIELD_MOVIE (object);
   swfdec_text_field_movie_set_text (text,
-      swfdec_as_value_to_string (swfdec_gc_object_get_context (text), val), text->html);
+      swfdec_as_value_to_string (swfdec_gc_object_get_context (text), *val), text->html);
 }
 
 void
@@ -1053,10 +1046,9 @@ swfdec_text_field_movie_set_listen_variable (SwfdecTextFieldMovie *text,
   if (text->variable != NULL) {
     swfdec_text_field_movie_parse_listen_variable (text, text->variable,
 	&object, &name);
-    if (object != NULL && SWFDEC_IS_MOVIE (object)) {
-      swfdec_movie_remove_variable_listener (SWFDEC_MOVIE (object),
-	  SWFDEC_AS_OBJECT (text), name,
-	  swfdec_text_field_movie_variable_listener_callback);
+    if (object != NULL && object->movie) {
+      swfdec_movie_remove_variable_listener (SWFDEC_MOVIE (object->relay),
+	  text, name, swfdec_text_field_movie_variable_listener_callback);
     }
   }
 
@@ -1071,7 +1063,7 @@ swfdec_text_field_movie_set_listen_variable (SwfdecTextFieldMovie *text,
 	&name);
     if (object != NULL && swfdec_as_object_get_variable (object, name, &val)) {
       swfdec_text_field_movie_set_text (text,
-	  swfdec_as_value_to_string (swfdec_gc_object_get_context (text), &val),
+	  swfdec_as_value_to_string (swfdec_gc_object_get_context (text), val),
 	  text->html);
     } else if (text_field != NULL && text_field->input != NULL) {
       // Set to the original value from the tag, not current value
@@ -1081,10 +1073,9 @@ swfdec_text_field_movie_set_listen_variable (SwfdecTextFieldMovie *text,
       // FIXME: html value correct here?
       swfdec_text_field_movie_set_text (text, initial, text_field->html);
     }
-    if (object != NULL && SWFDEC_IS_MOVIE (object)) {
-      swfdec_movie_add_variable_listener (SWFDEC_MOVIE (object),
-	  SWFDEC_AS_OBJECT (text), name,
-	  swfdec_text_field_movie_variable_listener_callback);
+    if (object != NULL && object->movie) {
+      swfdec_movie_add_variable_listener (SWFDEC_MOVIE (object->relay),
+	  text, name, swfdec_text_field_movie_variable_listener_callback);
     }
   }
 }
