@@ -62,7 +62,7 @@ swfdec_text_field_movie_set_readonly (SwfdecAsContext *cx,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
   if (argc > 0)
-    swfdec_as_value_to_number (cx, &argv[0]);
+    swfdec_as_value_to_number (cx, argv[0]);
 }
 
 /*
@@ -124,7 +124,7 @@ swfdec_text_field_movie_set_html (SwfdecAsContext *cx, SwfdecAsObject *object,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   text->html = value;
 
@@ -182,7 +182,7 @@ swfdec_text_field_movie_get_length (SwfdecAsContext *cx, SwfdecAsObject *object,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_INT (ret, g_utf8_strlen (swfdec_text_buffer_get_text (text->text), -1));
+  *ret = swfdec_as_value_from_integer (cx, g_utf8_strlen (swfdec_text_buffer_get_text (text->text), -1));
 }
 
 /*
@@ -210,7 +210,7 @@ swfdec_text_field_movie_set_condenseWhite (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   text->condense_white = value;
 }
@@ -225,7 +225,7 @@ swfdec_text_field_movie_get_maxChars (SwfdecAsContext *cx,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
   if (text->max_chars != 0) {
-    SWFDEC_AS_VALUE_SET_INT (ret, text->max_chars);
+    *ret = swfdec_as_value_from_integer (cx, text->max_chars);
   } else {
     SWFDEC_AS_VALUE_SET_NULL (ret);
   }
@@ -243,8 +243,8 @@ swfdec_text_field_movie_set_maxChars (SwfdecAsContext *cx,
   if (argc < 1)
     return;
 
-  swfdec_as_value_to_number (cx, &argv[0]);
-  text->max_chars = swfdec_as_value_to_integer (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
+  text->max_chars = swfdec_as_value_to_integer (cx, argv[0]);
 }
 
 static void
@@ -269,7 +269,7 @@ swfdec_text_field_movie_set_multiline (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   text->multiline = value;
 }
@@ -299,15 +299,15 @@ swfdec_text_field_movie_set_restrict (SwfdecAsContext *cx,
   const char *value;
 
   if (argc > 0)
-    swfdec_as_value_to_number (cx, &argv[0]);
+    swfdec_as_value_to_number (cx, argv[0]);
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "s", &value);
 
   if (text->restrict_ == NULL && value == SWFDEC_AS_STR_EMPTY)
     return;
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
-      SWFDEC_AS_VALUE_IS_NULL (&argv[0])) {
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0]) ||
+      SWFDEC_AS_VALUE_IS_NULL (argv[0])) {
     text->restrict_ = NULL;
   } else {
     text->restrict_ = value;
@@ -336,7 +336,7 @@ swfdec_text_field_movie_set_selectable (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   text->selectable = value;
 
@@ -368,7 +368,7 @@ swfdec_text_field_movie_do_set_type (SwfdecAsContext *cx,
   const char *value;
 
   if (argc > 0)
-    swfdec_as_value_to_number (cx, &argv[0]);
+    swfdec_as_value_to_number (cx, argv[0]);
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "s", &value);
 
@@ -407,12 +407,12 @@ swfdec_text_field_movie_do_set_variable (SwfdecAsContext *cx,
   const char *value;
 
   if (argc > 0)
-    swfdec_as_value_to_number (cx, &argv[0]);
+    swfdec_as_value_to_number (cx, argv[0]);
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "s", &value);
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
-      SWFDEC_AS_VALUE_IS_NULL (&argv[0]) || value == SWFDEC_AS_STR_EMPTY) {
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0]) ||
+      SWFDEC_AS_VALUE_IS_NULL (argv[0]) || value == SWFDEC_AS_STR_EMPTY) {
     value = NULL;
   }
 
@@ -433,7 +433,7 @@ swfdec_text_field_movie_get_textHeight (SwfdecAsContext *cx,
 
   swfdec_movie_update (SWFDEC_MOVIE (text));
   swfdec_text_field_movie_update_layout (text);
-  SWFDEC_AS_VALUE_SET_INT (ret, floor (text->layout_height * 
+  *ret = swfdec_as_value_from_integer (cx, floor (text->layout_height * 
 	text->from_layout.yy / SWFDEC_TWIPS_SCALE_FACTOR));
 }
 
@@ -448,7 +448,7 @@ swfdec_text_field_movie_get_textWidth (SwfdecAsContext *cx,
 
   swfdec_movie_update (SWFDEC_MOVIE (text));
   swfdec_text_field_movie_update_layout (text);
-  SWFDEC_AS_VALUE_SET_INT (ret, floor (text->layout_width *
+  *ret = swfdec_as_value_from_integer (cx, floor (text->layout_width *
 	text->from_layout.xx / SWFDEC_TWIPS_SCALE_FACTOR));
 }
 
@@ -477,7 +477,7 @@ swfdec_text_field_movie_set_background (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   if (text->background != value) {
     text->background = value;
@@ -494,7 +494,7 @@ swfdec_text_field_movie_get_backgroundColor (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret,
+  *ret = swfdec_as_value_from_number (cx,
       SWFDEC_COLOR_RED (text->background_color) << 16 |
       SWFDEC_COLOR_GREEN (text->background_color) << 8 |
       SWFDEC_COLOR_BLUE (text->background_color));
@@ -540,7 +540,7 @@ swfdec_text_field_movie_set_border (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   if (text->border != value) {
     text->border = value;
@@ -558,7 +558,7 @@ swfdec_text_field_movie_get_borderColor (SwfdecAsContext *cx,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret,
+  *ret = swfdec_as_value_from_number (cx,
       SWFDEC_COLOR_RED (text->border_color) << 16 |
       SWFDEC_COLOR_GREEN (text->border_color) << 8 |
       SWFDEC_COLOR_BLUE (text->border_color));
@@ -594,7 +594,7 @@ swfdec_text_field_movie_get_bottomScroll (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret, text->scroll + text->lines_visible);
+  *ret = swfdec_as_value_from_number (cx, text->scroll + text->lines_visible);
 }
 
 static void
@@ -606,7 +606,7 @@ swfdec_text_field_movie_do_get_hscroll (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret, text->hscroll);
+  *ret = swfdec_as_value_from_number (cx, text->hscroll);
 }
 
 static void
@@ -636,7 +636,7 @@ swfdec_text_field_movie_get_maxhscroll (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_INT (ret, swfdec_text_field_movie_get_hscroll_max (text));
+  *ret = swfdec_as_value_from_integer (cx, swfdec_text_field_movie_get_hscroll_max (text));
 }
 
 static void
@@ -648,7 +648,7 @@ swfdec_text_field_movie_get_maxscroll (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret, text->scroll_max + 1);
+  *ret = swfdec_as_value_from_number (cx, text->scroll_max + 1);
 }
 
 static void
@@ -673,7 +673,7 @@ swfdec_text_field_movie_set_mouseWheelEnabled (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   text->mouse_wheel_enabled = value;
 }
@@ -687,7 +687,7 @@ swfdec_text_field_movie_do_get_scroll (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret, text->scroll + 1);
+  *ret = swfdec_as_value_from_number (cx, text->scroll + 1);
 }
 
 static void
@@ -752,15 +752,15 @@ swfdec_text_field_movie_set_autoSize (SwfdecAsContext *cx,
   if (argc < 1)
     return;
 
-  if (SWFDEC_AS_VALUE_IS_BOOLEAN (&argv[0])) {
-    if (SWFDEC_AS_VALUE_GET_BOOLEAN (&argv[0])) {
+  if (SWFDEC_AS_VALUE_IS_BOOLEAN (argv[0])) {
+    if (SWFDEC_AS_VALUE_GET_BOOLEAN (argv[0])) {
       text->auto_size = SWFDEC_AUTO_SIZE_LEFT;
     } else {
       text->auto_size = SWFDEC_AUTO_SIZE_NONE;
     }
   } else {
-    swfdec_as_value_to_number (cx, &argv[0]);
-    s = swfdec_as_value_to_string (cx, &argv[0]);
+    swfdec_as_value_to_number (cx, argv[0]);
+    s = swfdec_as_value_to_string (cx, argv[0]);
 
     if (!g_ascii_strcasecmp (s, "left")) {
       text->auto_size = SWFDEC_AUTO_SIZE_LEFT;
@@ -796,7 +796,7 @@ swfdec_text_field_movie_set_password (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   swfdec_text_layout_set_password (text->layout, value);
   swfdec_movie_invalidate_last (SWFDEC_MOVIE (text));
@@ -825,7 +825,7 @@ swfdec_text_field_movie_set_wordWrap (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   swfdec_text_layout_set_word_wrap (text->layout, value);
 }
@@ -855,7 +855,7 @@ swfdec_text_field_movie_set_embedFonts (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "b", &value);
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
   if (!text->embed_fonts && value)
     SWFDEC_FIXME ("Using embed fonts in TextField not supported");
@@ -875,7 +875,7 @@ swfdec_text_field_movie_get_styleSheet (SwfdecAsContext *cx,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
   if (text->style_sheet != NULL) {
-    SWFDEC_AS_VALUE_SET_OBJECT (ret, SWFDEC_AS_OBJECT (text->style_sheet));
+    SWFDEC_AS_VALUE_SET_OBJECT (ret, text->style_sheet);
   } else {
     SWFDEC_AS_VALUE_SET_UNDEFINED (ret);
   }
@@ -901,12 +901,10 @@ swfdec_text_field_movie_set_styleSheet (SwfdecAsContext *cx,
   if (argc < 1)
     return;
 
-  swfdec_as_value_to_number (cx, &argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
 
-  if (SWFDEC_AS_VALUE_IS_OBJECT (&argv[0])) {
-    value = SWFDEC_AS_VALUE_GET_OBJECT (&argv[0]);
-    if (SWFDEC_IS_MOVIE (value))
-      value = NULL;
+  if (SWFDEC_AS_VALUE_IS_OBJECT (argv[0])) {
+    value = SWFDEC_AS_VALUE_GET_OBJECT (argv[0]);
   } else {
     value = NULL;
   }
@@ -914,19 +912,19 @@ swfdec_text_field_movie_set_styleSheet (SwfdecAsContext *cx,
   if (text->style_sheet == value)
     return;
 
-  if (text->style_sheet != NULL && SWFDEC_IS_STYLESHEET (text->style_sheet)) {
-    g_signal_handlers_disconnect_by_func (text->style_sheet,
+  if (text->style_sheet != NULL && SWFDEC_IS_STYLE_SHEET (text->style_sheet->relay)) {
+    g_signal_handlers_disconnect_by_func (text->style_sheet->relay,
 	 swfdec_text_field_movie_style_sheet_update, text);
-    g_object_remove_weak_pointer (G_OBJECT (text->style_sheet), 
+    g_object_remove_weak_pointer (G_OBJECT (text->style_sheet->relay), 
 	(gpointer) &text->style_sheet);
   }
 
   text->style_sheet = value;
 
-  if (SWFDEC_IS_STYLESHEET (value)) {
-    g_signal_connect_swapped (value, "update",
+  if (value && SWFDEC_IS_STYLE_SHEET (value->relay)) {
+    g_signal_connect_swapped (value->relay, "update",
 	G_CALLBACK (swfdec_text_field_movie_style_sheet_update), text);
-    g_object_add_weak_pointer (G_OBJECT (text->style_sheet), 
+    g_object_add_weak_pointer (G_OBJECT (text->style_sheet->relay), 
 	(gpointer) &text->style_sheet);
 
     swfdec_text_field_movie_style_sheet_update (text);
@@ -942,7 +940,7 @@ swfdec_text_field_movie_get_textColor (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret, 
+  *ret = swfdec_as_value_from_number (cx, 
       swfdec_text_buffer_get_default_attributes (text->text)->color);
 }
 
@@ -1065,16 +1063,14 @@ swfdec_text_field_movie_getNewTextFormat (SwfdecAsContext *cx,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  format = SWFDEC_TEXT_FORMAT (swfdec_text_format_new (cx));
-  if (format == NULL)
-    return;
+  format = swfdec_text_format_new (cx);
 
   swfdec_text_attributes_copy (&format->attr, 
       swfdec_text_buffer_get_default_attributes (text->text),
       SWFDEC_TEXT_ATTRIBUTES_MASK);
   format->values_set = SWFDEC_TEXT_ATTRIBUTES_MASK;
 
-  SWFDEC_AS_VALUE_SET_OBJECT (ret, SWFDEC_AS_OBJECT (format));
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (format)));
 }
 
 SWFDEC_AS_NATIVE (104, 105, swfdec_text_field_movie_setNewTextFormat)
@@ -1084,13 +1080,15 @@ swfdec_text_field_movie_setNewTextFormat (SwfdecAsContext *cx,
     SwfdecAsValue *ret)
 {
   SwfdecTextFieldMovie *text;
+  SwfdecAsObject *format_object;
   SwfdecTextFormat *format;
 
-  SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "o", &format);
+  SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "o", &format_object);
 
-  if (!SWFDEC_IS_TEXT_FORMAT (format))
+  if (!SWFDEC_IS_TEXT_FORMAT (format_object->relay))
     return;
 
+  format = SWFDEC_TEXT_FORMAT (format_object->relay);
   swfdec_text_buffer_set_default_attributes (text->text, 
       &format->attr, format->values_set);
 }
@@ -1120,7 +1118,7 @@ swfdec_text_field_movie_setTextFormat (SwfdecAsContext *cx,
     end = length;
     i = 0;
   } else {
-    val = swfdec_as_value_to_integer (cx, &argv[0]);
+    val = swfdec_as_value_to_integer (cx, argv[0]);
     start = MAX (val, 0);
     start = MIN (start, length);
     if (argc <= 2) {
@@ -1131,7 +1129,7 @@ swfdec_text_field_movie_setTextFormat (SwfdecAsContext *cx,
       }
       i = 1;
     } else {
-      val = swfdec_as_value_to_integer (cx, &argv[1]);
+      val = swfdec_as_value_to_integer (cx, argv[1]);
       end = MAX (val, 0);
       end = CLAMP (end, start, length);
       i = 2;
@@ -1140,12 +1138,12 @@ swfdec_text_field_movie_setTextFormat (SwfdecAsContext *cx,
   if (start == end)
     return;
 
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&argv[i]))
+  if (!SWFDEC_AS_VALUE_IS_OBJECT (argv[i]))
     return;
-  if (!SWFDEC_IS_TEXT_FORMAT (SWFDEC_AS_VALUE_GET_OBJECT (&argv[i])))
+  if (!SWFDEC_IS_TEXT_FORMAT (SWFDEC_AS_VALUE_GET_OBJECT (argv[i])->relay))
     return;
 
-  format = SWFDEC_TEXT_FORMAT (SWFDEC_AS_VALUE_GET_OBJECT (&argv[i]));
+  format = SWFDEC_TEXT_FORMAT (SWFDEC_AS_VALUE_GET_OBJECT (argv[i])->relay);
   start = g_utf8_offset_to_pointer (string, start) - string;
   end = g_utf8_offset_to_pointer (string, end) - string;
 
@@ -1182,14 +1180,14 @@ swfdec_text_field_movie_getTextFormat (SwfdecAsContext *cx,
   string = swfdec_text_buffer_get_text (text->text);
   length = g_utf8_strlen (string, -1);
 
-  format = SWFDEC_TEXT_FORMAT (swfdec_text_format_new (cx));
-  SWFDEC_AS_VALUE_SET_OBJECT (ret, SWFDEC_AS_OBJECT (format));
+  format = swfdec_text_format_new (cx);
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (format)));
 
   if (argc == 0) {
     start = 0;
     end = length;
   } else {
-    val = swfdec_as_value_to_integer (cx, &argv[0]);
+    val = swfdec_as_value_to_integer (cx, argv[0]);
     start = MAX (val, 0);
     start = MIN (start, length);
     if (argc == 1) {
@@ -1199,7 +1197,7 @@ swfdec_text_field_movie_getTextFormat (SwfdecAsContext *cx,
 	end = MIN (start + 1, length);
       }
     } else {
-      val = swfdec_as_value_to_integer (cx, &argv[1]);
+      val = swfdec_as_value_to_integer (cx, argv[1]);
       end = MAX (val, 0);
       end = CLAMP (end, start, length);
     }
@@ -1273,7 +1271,7 @@ swfdec_text_field_movie_getFontList (SwfdecAsContext *cx,
     SwfdecAsObject *object, guint argc, SwfdecAsValue *argv,
     SwfdecAsValue *ret)
 {
-  SwfdecAsArray *array;
+  SwfdecAsObject *array;
   SwfdecAsValue val;
   PangoFontFamily **families;
   int i, n_families;
@@ -1281,7 +1279,7 @@ swfdec_text_field_movie_getFontList (SwfdecAsContext *cx,
   pango_font_map_list_families (pango_cairo_font_map_get_default (),
       &families, &n_families);
 
-  array = SWFDEC_AS_ARRAY (swfdec_as_array_new (cx));
+  array = swfdec_as_array_new (cx);
   for (i = 0; i < n_families; i++) {
     SWFDEC_AS_VALUE_SET_STRING (&val, swfdec_as_context_get_string (cx,
 	  pango_font_family_get_name (families[i])));
@@ -1296,7 +1294,7 @@ swfdec_text_field_movie_getFontList (SwfdecAsContext *cx,
 
   g_free (families);
 
-  SWFDEC_AS_VALUE_SET_OBJECT (ret, SWFDEC_AS_OBJECT (array));
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, array);
 }
 
 SWFDEC_AS_NATIVE (104, 106, swfdec_text_field_movie_getDepth)
@@ -1308,7 +1306,7 @@ swfdec_text_field_movie_getDepth (SwfdecAsContext *cx, SwfdecAsObject *object,
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FIELD_MOVIE, &text, "");
 
-  SWFDEC_AS_VALUE_SET_INT (rval, SWFDEC_MOVIE (text)->depth);
+  *rval = swfdec_as_value_from_integer (cx, SWFDEC_MOVIE (text)->depth);
 }
 
 SWFDEC_AS_NATIVE (104, 103, swfdec_text_field_movie_removeTextField)
@@ -1341,6 +1339,7 @@ swfdec_text_field_movie_createTextField (SwfdecAsContext *cx,
   int depth, x, y, width, height;
   const char *name;
   SwfdecAsFunction *fun;
+  SwfdecAsObject *fun_object, *o;
   SwfdecAsValue val;
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_MOVIE, &parent, "siiiii", &name, &depth, &x, &y, &width, &height);
@@ -1392,29 +1391,31 @@ swfdec_text_field_movie_createTextField (SwfdecAsContext *cx,
   swfdec_movie_update (movie);
 
   swfdec_as_object_get_variable (cx->global, SWFDEC_AS_STR_TextField, &val);
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&val))
+  if (!SWFDEC_AS_VALUE_IS_OBJECT (val))
     return;
-  fun = (SwfdecAsFunction *) SWFDEC_AS_VALUE_GET_OBJECT (&val);
-  if (!SWFDEC_IS_AS_FUNCTION (fun))
+  fun_object = SWFDEC_AS_VALUE_GET_OBJECT (val);
+  if (!SWFDEC_IS_AS_FUNCTION (fun_object->relay))
     return;
+  fun = SWFDEC_AS_FUNCTION (fun_object->relay);
 
   /* set initial variables */
-  if (swfdec_as_object_get_variable (SWFDEC_AS_OBJECT (fun),
+  o = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (movie));
+  if (swfdec_as_object_get_variable (fun_object,
 	SWFDEC_AS_STR_prototype, &val)) {
-    swfdec_as_object_set_variable_and_flags (SWFDEC_AS_OBJECT (movie),
+    swfdec_as_object_set_variable_and_flags (o,
 	SWFDEC_AS_STR___proto__, &val,
 	SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
   }
-  SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (fun));
+  SWFDEC_AS_VALUE_SET_OBJECT (&val, fun_object);
   if (cx->version < 7) {
-    swfdec_as_object_set_variable_and_flags (SWFDEC_AS_OBJECT (movie),
+    swfdec_as_object_set_variable_and_flags (o,
 	SWFDEC_AS_STR_constructor, &val, SWFDEC_AS_VARIABLE_HIDDEN);
   }
-  swfdec_as_object_set_variable_and_flags (SWFDEC_AS_OBJECT (movie),
+  swfdec_as_object_set_variable_and_flags (o,
       SWFDEC_AS_STR___constructor__, &val,
       SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_VERSION_6_UP);
 
-  swfdec_as_function_call_full (fun, SWFDEC_AS_OBJECT (movie), TRUE, NULL,
+  swfdec_as_function_call_full (fun, o, TRUE, NULL,
       0, NULL, cx->version > 7 ? rval : &val);
 }
 
@@ -1431,13 +1432,13 @@ swfdec_text_field_movie_init_properties (SwfdecAsContext *cx)
   g_return_if_fail (SWFDEC_IS_AS_CONTEXT (cx));
 
   swfdec_as_object_get_variable (cx->global, SWFDEC_AS_STR_TextField, &val);
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&val))
+  if (!SWFDEC_AS_VALUE_IS_OBJECT (val))
     return;
-  object = SWFDEC_AS_VALUE_GET_OBJECT (&val);
+  object = SWFDEC_AS_VALUE_GET_OBJECT (val);
   swfdec_as_object_get_variable (object, SWFDEC_AS_STR_prototype, &val);
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&val))
+  if (!SWFDEC_AS_VALUE_IS_OBJECT (val))
     return;
-  proto = SWFDEC_AS_VALUE_GET_OBJECT (&val);
+  proto = SWFDEC_AS_VALUE_GET_OBJECT (val);
 
   // text
   swfdec_as_object_add_native_variable (proto, SWFDEC_AS_STR_text,
@@ -1558,5 +1559,5 @@ swfdec_text_field_movie_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 
   // FIXME: do object.addListener (object);
 
-  SWFDEC_AS_VALUE_SET_OBJECT (ret, object);
+  SWFDEC_AS_VALUE_SET_COMPOSITE (ret, object);
 }
