@@ -116,6 +116,7 @@ swfdec_text_field_movie_html_tag_set_attribute (ParserData *data,
 {
   SwfdecAsValue val;
   SwfdecAsObject *object;
+  SwfdecAsContext *cx;
 
   g_return_if_fail (data != NULL);
   g_return_if_fail (tag != NULL);
@@ -127,9 +128,10 @@ swfdec_text_field_movie_html_tag_set_attribute (ParserData *data,
   if (!tag->format)
     return;
 
-  object = SWFDEC_AS_OBJECT (tag->format);
+  object = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (tag->format));
+  cx = swfdec_gc_object_get_context (tag->format);
   SWFDEC_AS_VALUE_SET_STRING (&val, swfdec_as_context_give_string (
-	swfdec_gc_object_get_context (object), g_strndup (value, value_length)));
+	cx, g_strndup (value, value_length)));
 
   if (tag->name_length == 10 && !g_strncasecmp (tag->name, "textformat", 10))
   {
@@ -177,7 +179,7 @@ swfdec_text_field_movie_html_tag_set_attribute (ParserData *data,
       SwfdecAsValue val_number;
 
       if (value_length != 7 || *value != '#') {
-	SWFDEC_AS_VALUE_SET_NUMBER (&val_number, 0);
+	val_number = swfdec_as_value_from_number (cx, 0);
       } else {
 	int number;
 	char *tail;
@@ -185,7 +187,7 @@ swfdec_text_field_movie_html_tag_set_attribute (ParserData *data,
 	number = g_ascii_strtoll (value + 1, &tail, 16);
 	if (tail != value + 7)
 	  number = 0;
-	SWFDEC_AS_VALUE_SET_NUMBER (&val_number, number);
+	val_number = swfdec_as_value_from_number (cx, number);
       }
 
       swfdec_as_object_set_variable (object, SWFDEC_AS_STR_color, &val_number);
@@ -379,14 +381,14 @@ swfdec_text_field_movie_html_parse_tag (ParserData *data, const char *p)
     tag = g_new0 (ParserTag, 1);
     tag->name = name;
     tag->name_length = name_length;
-    tag->format = SWFDEC_TEXT_FORMAT (swfdec_text_format_new (data->cx));
+    tag->format = swfdec_text_format_new (data->cx);
     tag->index = swfdec_text_buffer_get_length (data->text);
 
     data->tags_open = g_slist_prepend (data->tags_open, tag);
 
     // set format based on tag
     if (tag->format != NULL) {
-      object = SWFDEC_AS_OBJECT (tag->format);
+      object = swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (tag->format));
       SWFDEC_AS_VALUE_SET_BOOLEAN (&val, TRUE);
 
       if (tag->name_length == 2 && !g_strncasecmp (tag->name, "li", 2)) {
@@ -497,8 +499,8 @@ swfdec_text_field_movie_html_parse (SwfdecTextFieldMovie *text, const char *str)
   data.cx = swfdec_gc_object_get_context (text);
   data.multiline = (data.cx->version < 7 || text->multiline);
   data.condense_white = text->condense_white;
-  if (text->style_sheet != NULL && SWFDEC_IS_STYLESHEET (text->style_sheet)) {
-    data.style_sheet = SWFDEC_STYLESHEET (text->style_sheet);
+  if (text->style_sheet != NULL && SWFDEC_IS_STYLE_SHEET (text->style_sheet->relay)) {
+    data.style_sheet = SWFDEC_STYLE_SHEET (text->style_sheet->relay);
   } else {
     data.style_sheet = NULL;
   }
