@@ -27,8 +27,9 @@
 #include <pango/pangocairo.h>
 
 #include "swfdec_text_format.h"
-#include "swfdec_as_native_function.h"
 #include "swfdec_as_array.h"
+#include "swfdec_as_internal.h"
+#include "swfdec_as_native_function.h"
 #include "swfdec_as_object.h"
 #include "swfdec_as_strings.h"
 #include "swfdec_debug.h"
@@ -39,7 +40,7 @@
 #include "swfdec_text_buffer.h"
 #include "swfdec_text_layout.h"
 
-G_DEFINE_TYPE (SwfdecTextFormat, swfdec_text_format, SWFDEC_TYPE_AS_OBJECT)
+G_DEFINE_TYPE (SwfdecTextFormat, swfdec_text_format, SWFDEC_TYPE_AS_RELAY)
 
 static int property_offsets[] = {
   G_STRUCT_OFFSET (SwfdecTextFormat, attr.align),
@@ -106,9 +107,9 @@ swfdec_text_format_get_string (SwfdecAsObject *object,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (!SWFDEC_TEXT_ATTRIBUTE_IS_SET (format->values_set, property)) {
     SWFDEC_AS_VALUE_SET_NULL (ret);
@@ -127,20 +128,20 @@ swfdec_text_format_set_string (SwfdecAsObject *object,
   SwfdecAsContext *context;
   const char *s;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (argc < 1)
     return;
 
   context = swfdec_gc_object_get_context (format);
-  swfdec_as_value_to_integer (context, &argv[0]);
-  swfdec_as_value_to_number (context, &argv[0]);
-  s = swfdec_as_value_to_string (context, &argv[0]);
+  swfdec_as_value_to_integer (context, argv[0]);
+  swfdec_as_value_to_number (context, argv[0]);
+  s = swfdec_as_value_to_string (context, argv[0]);
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
-      SWFDEC_AS_VALUE_IS_NULL (&argv[0])) {
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0]) ||
+      SWFDEC_AS_VALUE_IS_NULL (argv[0])) {
     /* FIXME: reset to defaults here? */
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set, property);
   } else {
@@ -156,9 +157,9 @@ swfdec_text_format_get_boolean (SwfdecAsObject *object,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (!SWFDEC_TEXT_ATTRIBUTE_IS_SET (format->values_set, property)) {
     SWFDEC_AS_VALUE_SET_NULL (ret);
@@ -179,24 +180,24 @@ swfdec_text_format_set_boolean (SwfdecAsObject *object,
   SwfdecTextFormat *format;
   SwfdecAsContext *context;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (argc < 1)
     return;
 
   context = swfdec_gc_object_get_context (format);
-  swfdec_as_value_to_integer (context, &argv[0]);
-  swfdec_as_value_to_number (context, &argv[0]);
-  swfdec_as_value_to_string (context, &argv[0]);
+  swfdec_as_value_to_integer (context, argv[0]);
+  swfdec_as_value_to_number (context, argv[0]);
+  swfdec_as_value_to_string (context, argv[0]);
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
-      SWFDEC_AS_VALUE_IS_NULL (&argv[0])) {
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0]) ||
+      SWFDEC_AS_VALUE_IS_NULL (argv[0])) {
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set, property);
   } else {
     G_STRUCT_MEMBER (gboolean, format, property_offsets[property]) =
-      swfdec_as_value_to_boolean (context, &argv[0]);
+      swfdec_as_value_to_boolean (context, argv[0]);
     SWFDEC_TEXT_ATTRIBUTE_SET (format->values_set, property);
   }
 }
@@ -207,16 +208,16 @@ swfdec_text_format_get_integer (SwfdecAsObject *object,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (!SWFDEC_TEXT_ATTRIBUTE_IS_SET (format->values_set, property)) {
     SWFDEC_AS_VALUE_SET_NULL (ret);
     return;
   }
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret,
+  *ret = swfdec_as_value_from_number (object->context,
       (double)G_STRUCT_MEMBER (int, format, property_offsets[property]));
 }
 
@@ -227,9 +228,9 @@ swfdec_text_format_value_to_integer (SwfdecAsContext *cx, SwfdecAsValue *val,
   double d;
   int n;
 
-  n = swfdec_as_value_to_integer (cx, val);
-  d = swfdec_as_value_to_number (cx, val);
-  swfdec_as_value_to_string (cx, val);
+  n = swfdec_as_value_to_integer (cx, *val);
+  d = swfdec_as_value_to_number (cx, *val);
+  swfdec_as_value_to_string (cx, *val);
 
   if (cx->version >= 8) {
     if (isnan (d))
@@ -267,15 +268,15 @@ swfdec_text_format_set_integer (SwfdecAsObject *object,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (argc < 1)
     return;
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
-      SWFDEC_AS_VALUE_IS_NULL (&argv[0])) {
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0]) ||
+      SWFDEC_AS_VALUE_IS_NULL (argv[0])) {
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set, property);
   } else {
     G_STRUCT_MEMBER (int, format, property_offsets[property]) =
@@ -291,9 +292,9 @@ swfdec_text_format_do_get_align (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (!SWFDEC_TEXT_ATTRIBUTE_IS_SET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_ALIGN)) {
     SWFDEC_AS_VALUE_SET_NULL (ret);
@@ -325,16 +326,16 @@ swfdec_text_format_do_set_align (SwfdecAsContext *cx, SwfdecAsObject *object,
   SwfdecTextFormat *format;
   const char *s;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (argc < 1)
     return;
 
-  swfdec_as_value_to_integer (cx, &argv[0]);
-  swfdec_as_value_to_number (cx, &argv[0]);
-  s = swfdec_as_value_to_string (cx, &argv[0]);
+  swfdec_as_value_to_integer (cx, argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
+  s = swfdec_as_value_to_string (cx, argv[0]);
 
   if (!g_ascii_strcasecmp (s, "left")) {
     format->attr.align = SWFDEC_TEXT_ALIGN_LEFT;
@@ -402,16 +403,16 @@ swfdec_text_format_do_get_color (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (!SWFDEC_TEXT_ATTRIBUTE_IS_SET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_COLOR)) {
     SWFDEC_AS_VALUE_SET_NULL (ret);
     return;
   }
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret, format->attr.color);
+  *ret = swfdec_as_value_from_number (cx, format->attr.color);
 }
 
 static void
@@ -420,20 +421,20 @@ swfdec_text_format_do_set_color (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (argc < 1)
     return;
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
-      SWFDEC_AS_VALUE_IS_NULL (&argv[0])) {
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0]) ||
+      SWFDEC_AS_VALUE_IS_NULL (argv[0])) {
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_COLOR);
   } else {
-    format->attr.color = (unsigned) swfdec_as_value_to_integer (cx, &argv[0]);
-    swfdec_as_value_to_integer (cx, &argv[0]);
-    swfdec_as_value_to_string (cx, &argv[0]);
+    format->attr.color = (unsigned) swfdec_as_value_to_integer (cx, argv[0]);
+    swfdec_as_value_to_integer (cx, argv[0]);
+    swfdec_as_value_to_string (cx, argv[0]);
 
     SWFDEC_TEXT_ATTRIBUTE_SET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_COLOR);
   }
@@ -445,9 +446,9 @@ swfdec_text_format_do_get_display (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (!SWFDEC_TEXT_ATTRIBUTE_IS_SET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_DISPLAY))
   {
@@ -477,14 +478,14 @@ swfdec_text_format_do_set_display (SwfdecAsContext *cx, SwfdecAsObject *object,
   SwfdecTextFormat *format;
   const char *s;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
-  swfdec_as_value_to_integer (cx, &argv[0]);
-  swfdec_as_value_to_number (cx, &argv[0]);
-  swfdec_as_value_to_string (cx, &argv[0]);
-  s = swfdec_as_value_to_string (cx, &argv[0]); // oh yes, let's call it twice
+  swfdec_as_value_to_integer (cx, argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
+  swfdec_as_value_to_string (cx, argv[0]);
+  s = swfdec_as_value_to_string (cx, argv[0]); // oh yes, let's call it twice
 
   if (!g_ascii_strcasecmp (s, "none")) {
     format->attr.display = SWFDEC_TEXT_DISPLAY_NONE;
@@ -592,16 +593,16 @@ swfdec_text_format_do_get_letter_spacing (SwfdecAsContext *cx,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (!SWFDEC_TEXT_ATTRIBUTE_IS_SET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_LETTER_SPACING)) {
     SWFDEC_AS_VALUE_SET_NULL (ret);
     return;
   }
 
-  SWFDEC_AS_VALUE_SET_NUMBER (ret, format->attr.letter_spacing);
+  *ret = swfdec_as_value_from_number (cx, format->attr.letter_spacing);
 }
 
 static void
@@ -612,19 +613,19 @@ swfdec_text_format_do_set_letter_spacing (SwfdecAsContext *cx,
   SwfdecTextFormat *format;
   double d;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (argc < 1)
     return;
 
-  swfdec_as_value_to_integer (cx, &argv[0]);
-  d = swfdec_as_value_to_number (cx, &argv[0]);
-  swfdec_as_value_to_string (cx, &argv[0]);
+  swfdec_as_value_to_integer (cx, argv[0]);
+  d = swfdec_as_value_to_number (cx, argv[0]);
+  swfdec_as_value_to_string (cx, argv[0]);
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
-      SWFDEC_AS_VALUE_IS_NULL (&argv[0]))
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0]) ||
+      SWFDEC_AS_VALUE_IS_NULL (argv[0]))
   {
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set,
 	SWFDEC_TEXT_ATTRIBUTE_LETTER_SPACING);
@@ -678,9 +679,9 @@ swfdec_text_format_do_get_tab_stops (SwfdecAsContext *cx,
   SwfdecAsValue val;
   SwfdecAsObject *array;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (!SWFDEC_TEXT_ATTRIBUTE_IS_SET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_TAB_STOPS)) {
     SWFDEC_AS_VALUE_SET_NULL (ret);
@@ -689,8 +690,8 @@ swfdec_text_format_do_get_tab_stops (SwfdecAsContext *cx,
 
   array = swfdec_as_array_new (cx);
   for (i = 0; i < format->attr.n_tab_stops; i++) {
-    SWFDEC_AS_VALUE_SET_INT (&val, format->attr.tab_stops[i]);
-    swfdec_as_array_push (SWFDEC_AS_ARRAY (array), &val);
+    val = swfdec_as_value_from_integer (cx, format->attr.tab_stops[i]);
+    swfdec_as_array_push (array, &val);
   }
   SWFDEC_AS_VALUE_SET_OBJECT (ret, array);
 }
@@ -702,34 +703,34 @@ swfdec_text_format_do_set_tab_stops (SwfdecAsContext *cx,
 {
   SwfdecTextFormat *format;
 
-  if (!SWFDEC_IS_TEXT_FORMAT (object))
+  if (object == NULL || !SWFDEC_IS_TEXT_FORMAT (object->relay))
     return;
-  format = SWFDEC_TEXT_FORMAT (object);
+  format = SWFDEC_TEXT_FORMAT (object->relay);
 
   if (argc < 1)
     return;
 
-  swfdec_as_value_to_integer (cx, &argv[0]);
-  swfdec_as_value_to_number (cx, &argv[0]);
-  swfdec_as_value_to_string (cx, &argv[0]);
+  swfdec_as_value_to_integer (cx, argv[0]);
+  swfdec_as_value_to_number (cx, argv[0]);
+  swfdec_as_value_to_string (cx, argv[0]);
 
-  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
-      SWFDEC_AS_VALUE_IS_NULL (&argv[0]))
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (argv[0]) ||
+      SWFDEC_AS_VALUE_IS_NULL (argv[0]))
   {
     g_free (format->attr.tab_stops);
     format->attr.tab_stops = NULL;
     format->attr.n_tab_stops = 0;
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_TAB_STOPS);
   }
-  else if (SWFDEC_AS_VALUE_IS_OBJECT (&argv[0]) &&
-	SWFDEC_IS_AS_ARRAY (SWFDEC_AS_VALUE_GET_OBJECT (&argv[0])))
+  else if (SWFDEC_AS_VALUE_IS_OBJECT (argv[0]) &&
+	SWFDEC_AS_VALUE_GET_OBJECT (argv[0])->array)
   {
-    SwfdecAsArray *array;
+    SwfdecAsObject *array;
     SwfdecAsValue val;
     guint i;
     int len;
 
-    array = SWFDEC_AS_ARRAY (SWFDEC_AS_VALUE_GET_OBJECT (&argv[0]));
+    array = SWFDEC_AS_VALUE_GET_OBJECT (argv[0]);
     len = swfdec_as_array_get_length (array);
 
     if (!SWFDEC_TEXT_ATTRIBUTE_IS_SET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_TAB_STOPS)) {
@@ -747,12 +748,12 @@ swfdec_text_format_do_set_tab_stops (SwfdecAsContext *cx,
       format->attr.tab_stops[i] = swfdec_text_format_value_to_integer (cx, &val, TRUE);
     }
   }
-  else if (SWFDEC_AS_VALUE_IS_STRING (&argv[0]))
+  else if (SWFDEC_AS_VALUE_IS_STRING (argv[0]))
   {
     gsize i;
 
     // special case: empty strings mean null
-    if (SWFDEC_AS_VALUE_GET_STRING (&argv[0]) == SWFDEC_AS_STR_EMPTY) {
+    if (SWFDEC_AS_VALUE_GET_STRING (argv[0]) == SWFDEC_AS_STR_EMPTY) {
       g_free (format->attr.tab_stops);
       format->attr.tab_stops = NULL;
       format->attr.n_tab_stops = 0;
@@ -760,7 +761,7 @@ swfdec_text_format_do_set_tab_stops (SwfdecAsContext *cx,
     } else {
       int n = cx->version >= 8 ? G_MININT : 0;
       SWFDEC_TEXT_ATTRIBUTE_SET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_TAB_STOPS);
-      format->attr.n_tab_stops = strlen (SWFDEC_AS_VALUE_GET_STRING (&argv[0]));
+      format->attr.n_tab_stops = strlen (SWFDEC_AS_VALUE_GET_STRING (argv[0]));
       format->attr.tab_stops = g_new (guint, format->attr.n_tab_stops);
       for (i = 0; i < format->attr.n_tab_stops; i++) {
 	format->attr.tab_stops[i] = n;
@@ -842,25 +843,25 @@ swfdec_text_format_getTextExtent (SwfdecAsContext *cx, SwfdecAsObject *object,
   layout = swfdec_text_layout_new (buffer);
   
   i = swfdec_text_layout_get_width (layout);
-  SWFDEC_AS_VALUE_SET_INT (&val, i);
+  val = swfdec_as_value_from_integer (cx, i);
   swfdec_as_object_set_variable (obj, SWFDEC_AS_STR_width, &val);
   if (i)
     i += 4;
-  SWFDEC_AS_VALUE_SET_INT (&val, i);
+  val = swfdec_as_value_from_integer (cx, i);
   swfdec_as_object_set_variable (obj, SWFDEC_AS_STR_textFieldWidth, &val);
 
   i = swfdec_text_layout_get_height (layout);
-  SWFDEC_AS_VALUE_SET_INT (&val, i);
+  val = swfdec_as_value_from_integer (cx, i);
   swfdec_as_object_set_variable (obj, SWFDEC_AS_STR_height, &val);
   if (i)
     i += 4;
-  SWFDEC_AS_VALUE_SET_INT (&val, i);
+  val = swfdec_as_value_from_integer (cx, i);
   swfdec_as_object_set_variable (obj, SWFDEC_AS_STR_textFieldHeight, &val);
 
   swfdec_text_layout_get_ascent_descent (layout, &i, &j);
-  SWFDEC_AS_VALUE_SET_INT (&val, i);
+  val = swfdec_as_value_from_integer (cx, i);
   swfdec_as_object_set_variable (obj, SWFDEC_AS_STR_ascent, &val);
-  SWFDEC_AS_VALUE_SET_INT (&val, j);
+  val = swfdec_as_value_from_integer (cx, j);
   swfdec_as_object_set_variable (obj, SWFDEC_AS_STR_descent, &val);
 
   SWFDEC_AS_VALUE_SET_OBJECT (ret, obj);
@@ -949,13 +950,13 @@ swfdec_text_format_init_properties (SwfdecAsContext *cx)
   g_return_if_fail (SWFDEC_IS_AS_CONTEXT (cx));
 
   swfdec_as_object_get_variable (cx->global, SWFDEC_AS_STR_TextFormat, &val);
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&val))
+  if (!SWFDEC_AS_VALUE_IS_OBJECT (val))
     return;
-  proto = SWFDEC_AS_VALUE_GET_OBJECT (&val);
+  proto = SWFDEC_AS_VALUE_GET_OBJECT (val);
   swfdec_as_object_get_variable (proto, SWFDEC_AS_STR_prototype, &val);
-  if (!SWFDEC_AS_VALUE_IS_OBJECT (&val))
+  if (!SWFDEC_AS_VALUE_IS_OBJECT (val))
     return;
-  proto = SWFDEC_AS_VALUE_GET_OBJECT (&val);
+  proto = SWFDEC_AS_VALUE_GET_OBJECT (val);
 
   swfdec_as_object_add_native_variable (proto, SWFDEC_AS_STR_align,
       swfdec_text_format_do_get_align, swfdec_text_format_do_set_align);
@@ -1007,7 +1008,7 @@ swfdec_text_format_init_properties (SwfdecAsContext *cx)
       swfdec_text_format_do_get_url, swfdec_text_format_do_set_url);
 }
 
-SWFDEC_AS_CONSTRUCTOR (110, 0, swfdec_text_format_construct, swfdec_text_format_get_type)
+SWFDEC_AS_NATIVE (110, 0, swfdec_text_format_construct)
 void
 swfdec_text_format_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
@@ -1028,8 +1029,8 @@ swfdec_text_format_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
     SWFDEC_AS_STR_leading,
     NULL
   };
+  SwfdecTextFormat *format;
   SwfdecAsFunction *function;
-  SwfdecAsObject *tmp;
   SwfdecAsValue val;
   guint i;
 
@@ -1038,22 +1039,16 @@ swfdec_text_format_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
     return;
   }
 
-  g_assert (SWFDEC_IS_TEXT_FORMAT (object));
-
   swfdec_text_format_init_properties (cx);
 
-  swfdec_text_format_clear (SWFDEC_TEXT_FORMAT (object));
+  format = g_object_new (SWFDEC_TYPE_TEXT_FORMAT, "context", cx, NULL);
+  swfdec_text_format_clear (format);
+  swfdec_as_object_set_relay (object, SWFDEC_AS_RELAY (format));
 
-  // FIXME: Need better way to create function without prototype/constructor
-  tmp = cx->Function;
-  cx->Function = NULL;
-  function = swfdec_as_native_function_new (cx, SWFDEC_AS_STR_getTextExtent,
-      swfdec_text_format_getTextExtent, NULL);
-  cx->Function = tmp;
-  if (function != NULL) {
-    SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (function));
-    swfdec_as_object_set_variable (object, SWFDEC_AS_STR_getTextExtent, &val);
-  }
+  function = swfdec_as_native_function_new_bare (cx, 
+	SWFDEC_AS_STR_getTextExtent, swfdec_text_format_getTextExtent);
+  SWFDEC_AS_VALUE_SET_OBJECT (&val, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (function)));
+  swfdec_as_object_set_variable (object, SWFDEC_AS_STR_getTextExtent, &val);
 
   for (i = 0; i < argc && arguments[i] != NULL; i++) {
     swfdec_as_object_set_variable (object, arguments[i], &argv[i]);
@@ -1063,16 +1058,12 @@ swfdec_text_format_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 SwfdecTextFormat *
 swfdec_text_format_copy (SwfdecTextFormat *copy_from)
 {
-  SwfdecAsObject *object_to;
   SwfdecTextFormat *copy_to;
 
   g_return_val_if_fail (SWFDEC_IS_TEXT_FORMAT (copy_from), NULL);
 
-  object_to = swfdec_text_format_new_no_properties (
+  copy_to = swfdec_text_format_new_no_properties (
       swfdec_gc_object_get_context (copy_from));
-  if (object_to == NULL)
-    return NULL;
-  copy_to = SWFDEC_TEXT_FORMAT (object_to);
 
   swfdec_text_attributes_copy (&copy_to->attr, &copy_from->attr, -1);
   copy_to->values_set = copy_from->values_set;
@@ -1080,10 +1071,11 @@ swfdec_text_format_copy (SwfdecTextFormat *copy_from)
   return copy_to;
 }
 
-SwfdecAsObject *
+SwfdecTextFormat *
 swfdec_text_format_new_no_properties (SwfdecAsContext *context)
 {
-  SwfdecAsObject *tmp, *ret;
+  SwfdecAsObject *object;
+  SwfdecTextFormat *ret;
   SwfdecAsFunction *function;
   SwfdecAsValue val;
 
@@ -1091,24 +1083,21 @@ swfdec_text_format_new_no_properties (SwfdecAsContext *context)
 
   ret = g_object_new (SWFDEC_TYPE_TEXT_FORMAT, "context", context, NULL);
 
-  swfdec_text_format_clear (SWFDEC_TEXT_FORMAT (ret));
-  swfdec_as_object_set_constructor_by_name (ret, SWFDEC_AS_STR_TextFormat, NULL);
+  swfdec_text_format_clear (ret);
+  object = swfdec_as_object_new (context, NULL);
+  swfdec_as_object_set_constructor_by_name (object, SWFDEC_AS_STR_TextFormat, NULL);
+  swfdec_as_object_set_relay (object, SWFDEC_AS_RELAY (ret));
 
   // FIXME: Need better way to create function without prototype/constructor
-  tmp = context->Function;
-  context->Function = NULL;
-  function = swfdec_as_native_function_new (context, SWFDEC_AS_STR_getTextExtent,
-      swfdec_text_format_getTextExtent, NULL);
-  context->Function = tmp;
-  if (function != NULL) {
-    SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (function));
-    swfdec_as_object_set_variable (ret, SWFDEC_AS_STR_getTextExtent, &val);
-  }
+  function = swfdec_as_native_function_new_bare (context, 
+	SWFDEC_AS_STR_getTextExtent, swfdec_text_format_getTextExtent);
+  SWFDEC_AS_VALUE_SET_OBJECT (&val, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (function)));
+  swfdec_as_object_set_variable (object, SWFDEC_AS_STR_getTextExtent, &val);
 
   return ret;
 }
 
-SwfdecAsObject *
+SwfdecTextFormat *
 swfdec_text_format_new (SwfdecAsContext *context)
 {
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
