@@ -1,5 +1,6 @@
 /* Swfdec
  * Copyright (C) 2007-2008 Pekka Lampila <pekka.lampila@iki.fi>
+ *		 2008 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,17 +25,30 @@
 #include "swfdec_transform_as.h"
 
 #include "swfdec_color_transform_as.h"
-#include "swfdec_as_strings.h"
 #include "swfdec_as_internal.h"
-#include "swfdec_as_frame_internal.h"
+#include "swfdec_as_strings.h"
 #include "swfdec_debug.h"
 #include "swfdec_utils.h"
 
-G_DEFINE_TYPE (SwfdecTransformAs, swfdec_transform_as, SWFDEC_TYPE_AS_OBJECT)
+G_DEFINE_TYPE (SwfdecTransformAs, swfdec_transform_as, SWFDEC_TYPE_AS_RELAY)
+
+static void
+swfdec_transform_as_mark (SwfdecGcObject *object)
+{
+  SwfdecTransformAs *trans = SWFDEC_TRANSFORM_AS (object);
+
+  if (trans->target != NULL)
+    swfdec_gc_object_mark (trans->target);
+
+  SWFDEC_GC_OBJECT_CLASS (swfdec_transform_as_parent_class)->mark (object);
+}
 
 static void
 swfdec_transform_as_class_init (SwfdecTransformAsClass *klass)
 {
+  SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (klass);
+
+  gc_class->mark = swfdec_transform_as_mark;
 }
 
 static void
@@ -63,19 +77,19 @@ swfdec_transform_as_get_matrix (SwfdecAsContext *cx, SwfdecAsObject *object,
   swfdec_as_object_set_constructor_by_name (o, SWFDEC_AS_STR_flash,
 	SWFDEC_AS_STR_geom, SWFDEC_AS_STR_Matrix, NULL);
 
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, matrix->xx);
+  val = swfdec_as_value_from_number (cx, matrix->xx);
   swfdec_as_object_set_variable (o, SWFDEC_AS_STR_a, &val);
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, matrix->yx);
+  val = swfdec_as_value_from_number (cx, matrix->yx);
   swfdec_as_object_set_variable (o, SWFDEC_AS_STR_b, &val);
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, matrix->xy);
+  val = swfdec_as_value_from_number (cx, matrix->xy);
   swfdec_as_object_set_variable (o, SWFDEC_AS_STR_c, &val);
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, matrix->yy);
+  val = swfdec_as_value_from_number (cx, matrix->yy);
   swfdec_as_object_set_variable (o, SWFDEC_AS_STR_d, &val);
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, matrix->yy);
+  val = swfdec_as_value_from_number (cx, matrix->yy);
   swfdec_as_object_set_variable (o, SWFDEC_AS_STR_d, &val);
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, SWFDEC_TWIPS_TO_DOUBLE (matrix->x0));
+  val = swfdec_as_value_from_number (cx, SWFDEC_TWIPS_TO_DOUBLE (matrix->x0));
   swfdec_as_object_set_variable (o, SWFDEC_AS_STR_tx, &val);
-  SWFDEC_AS_VALUE_SET_NUMBER (&val, SWFDEC_TWIPS_TO_DOUBLE (matrix->y0));
+  val = swfdec_as_value_from_number (cx, SWFDEC_TWIPS_TO_DOUBLE (matrix->y0));
   swfdec_as_object_set_variable (o, SWFDEC_AS_STR_ty, &val);
 
   SWFDEC_AS_VALUE_SET_OBJECT (ret, o);
@@ -136,15 +150,16 @@ swfdec_transform_as_get_colorTransform (SwfdecAsContext *cx,
     SwfdecAsValue *ret)
 {
   SwfdecTransformAs *transform;
+  SwfdecColorTransformAs *ctrans;
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TRANSFORM_AS, &transform, "");
 
   if (transform->target == NULL)
     return;
 
-  SWFDEC_AS_VALUE_SET_OBJECT (ret,
-    SWFDEC_AS_OBJECT (swfdec_color_transform_as_new_from_transform (cx,
-	&transform->target->color_transform)));
+  ctrans = swfdec_color_transform_as_new_from_transform (cx,
+      &transform->target->color_transform);
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (ctrans)));
 }
 
 SWFDEC_AS_NATIVE (1106, 106, swfdec_transform_as_set_colorTransform)
@@ -162,10 +177,10 @@ swfdec_transform_as_set_colorTransform (SwfdecAsContext *cx,
   if (self->target == NULL)
     return;
 
-  if (!SWFDEC_IS_COLOR_TRANSFORM_AS (color))
+  if (color == NULL || !SWFDEC_IS_COLOR_TRANSFORM_AS (color->relay))
     return;
 
-  transform_as = SWFDEC_COLOR_TRANSFORM_AS (color);
+  transform_as = SWFDEC_COLOR_TRANSFORM_AS (color->relay);
 
   swfdec_color_transform_get_transform (transform_as, &self->target->color_transform);
 }
@@ -178,6 +193,7 @@ swfdec_transform_as_get_concatenatedColorTransform (SwfdecAsContext *cx,
 {
   SwfdecTransformAs *self;
   SwfdecColorTransform chain;
+  SwfdecColorTransformAs *ctrans;
   SwfdecMovie *movie;
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TRANSFORM_AS, &self, "");
@@ -191,8 +207,8 @@ swfdec_transform_as_get_concatenatedColorTransform (SwfdecAsContext *cx,
     swfdec_color_transform_chain (&chain, &movie->color_transform, &chain);
   }
 
-  SWFDEC_AS_VALUE_SET_OBJECT (ret, SWFDEC_AS_OBJECT (
-      swfdec_color_transform_as_new_from_transform (cx, &chain)));
+  ctrans = swfdec_color_transform_as_new_from_transform (cx, &chain);
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, swfdec_as_relay_get_as_object (SWFDEC_AS_RELAY (ctrans)));
 }
 
 SWFDEC_AS_NATIVE (1106, 108, swfdec_transform_as_set_concatenatedColorTransform)
@@ -223,38 +239,42 @@ swfdec_transform_as_set_pixelBounds (SwfdecAsContext *cx,
 }
 
 // constructor
-SWFDEC_AS_CONSTRUCTOR (1106, 0, swfdec_transform_as_construct, swfdec_transform_as_get_type)
+SWFDEC_AS_NATIVE (1106, 0, swfdec_transform_as_construct)
 void
 swfdec_transform_as_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  if (!cx->frame->construct)
+  SwfdecTransformAs *trans;
+  SwfdecMovie *movie;
+
+  if (!swfdec_as_context_is_constructing (cx))
     return;
 
-  if (argc < 1 ||
-      !SWFDEC_AS_VALUE_IS_OBJECT (&argv[0]) ||
-      !SWFDEC_IS_MOVIE (SWFDEC_AS_VALUE_GET_OBJECT (&argv[0]))) {
-    SWFDEC_FIXME ("new Transform without movieclip should give undefined");
-    return;
-  }
+  SWFDEC_AS_CHECK (0, NULL, "M", &movie);
 
-  SWFDEC_TRANSFORM_AS (object)->target =
-    SWFDEC_MOVIE (SWFDEC_AS_VALUE_GET_OBJECT (&argv[0]));
+  trans = g_object_new (SWFDEC_TYPE_TRANSFORM_AS, "context", cx, NULL);
+  trans->target = SWFDEC_AS_VALUE_GET_MOVIE (argv[0]);
+  swfdec_as_object_set_relay (object, SWFDEC_AS_RELAY (trans));
+  SWFDEC_AS_VALUE_SET_OBJECT (ret, object);
 }
 
 SwfdecTransformAs *
 swfdec_transform_as_new (SwfdecAsContext *context, SwfdecMovie *target)
 {
   SwfdecTransformAs *transform;
+  SwfdecAsObject *object;
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
   g_return_val_if_fail (SWFDEC_IS_MOVIE (target), NULL);
 
   transform = g_object_new (SWFDEC_TYPE_TRANSFORM_AS, "context", context, NULL);
-
-  swfdec_as_object_set_constructor_by_name (SWFDEC_AS_OBJECT (transform),
-      SWFDEC_AS_STR_flash, SWFDEC_AS_STR_geom, SWFDEC_AS_STR_Transform, NULL);
   transform->target = target;
 
+  object = swfdec_as_object_new (context, NULL);
+  swfdec_as_object_set_constructor_by_name (object,
+      SWFDEC_AS_STR_flash, SWFDEC_AS_STR_geom, SWFDEC_AS_STR_Transform, NULL);
+
+  swfdec_as_object_set_relay (object, SWFDEC_AS_RELAY (transform));
+
   return transform;
 }
