@@ -29,18 +29,9 @@
 
 G_DEFINE_TYPE (SwfdecVideoDecoder, swfdec_video_decoder, G_TYPE_OBJECT)
 
-static void
-swfdec_video_decoder_do_set_codec_data (SwfdecVideoDecoder *decoder,
-    SwfdecBuffer *buffer)
-{
-  SWFDEC_WARNING ("%s does not implement codec data", 
-      G_OBJECT_TYPE_NAME (decoder));
-}
-
 static void
 swfdec_video_decoder_class_init (SwfdecVideoDecoderClass *klass)
 {
-  klass->set_codec_data = swfdec_video_decoder_do_set_codec_data;
 }
 
 static void
@@ -121,6 +112,8 @@ swfdec_video_decoder_prepare (guint codec, char **missing)
 /**
  * swfdec_video_decoder_new:
  * @codec: codec id
+ * @data: initialization data for the video codec or %NULL if none. Currently 
+ *        only used for H264
  *
  * Creates a decoder suitable for decoding @format. If no decoder is available
  * for the given for mat, %NULL is returned.
@@ -128,14 +121,14 @@ swfdec_video_decoder_prepare (guint codec, char **missing)
  * Returns: a new decoder or %NULL
  **/
 SwfdecVideoDecoder *
-swfdec_video_decoder_new (guint codec)
+swfdec_video_decoder_new (guint codec, SwfdecBuffer *buffer)
 {
   SwfdecVideoDecoder *ret = NULL;
   GSList *walk;
   
   for (walk = video_codecs; walk; walk = walk->next) {
     SwfdecVideoDecoderClass *klass = g_type_class_ref (GPOINTER_TO_SIZE (walk->data));
-    ret = klass->create (codec);
+    ret = klass->create (codec, buffer);
     g_type_class_unref (klass);
     if (ret)
       break;
@@ -151,28 +144,6 @@ swfdec_video_decoder_new (guint codec)
   return ret;
 }
 
-/**
- * swfdec_video_decoder_set_codec_data:
- * @decoder: a video decoder
- * @buffer: setup data for the decoder. May be %NULL
- *
- * Provides setup data for the video decoder. This function is usually called 
- * on initialization, but can be called at any time. Currently this 
- * functionality is only used for H264.
- **/
-void
-swfdec_video_decoder_set_codec_data (SwfdecVideoDecoder *decoder, SwfdecBuffer *buffer)
-{
-  SwfdecVideoDecoderClass *klass;
-
-  g_return_if_fail (SWFDEC_IS_VIDEO_DECODER (decoder));
-
-  if (decoder->error)
-    return;
-  klass = SWFDEC_VIDEO_DECODER_GET_CLASS (decoder);
-  klass->set_codec_data (decoder, buffer);
-}
-
 /**
  * swfdec_video_decoder_decode:
  * @decoder: a #SwfdecVideoDecoder
