@@ -38,7 +38,7 @@ swfdec_video_decoder_vp6_alpha_prepare (guint codec, char **missing)
 }
 
 static SwfdecVideoDecoder *
-swfdec_video_decoder_vp6_alpha_create (guint codec)
+swfdec_video_decoder_vp6_alpha_create (guint codec, SwfdecBuffer *buffer)
 {
   if (codec != SWFDEC_VIDEO_CODEC_VP6_ALPHA)
     return NULL;
@@ -83,6 +83,8 @@ swfdec_video_decoder_vp6_alpha_decode (SwfdecVideoDecoder *dec, SwfdecBuffer *bu
 	swfdec_video_decoder_get_width (vp6->image), swfdec_video_decoder_get_height (vp6->image));
     return;
   }
+  dec->width = swfdec_video_decoder_get_width (vp6->image);
+  dec->height = swfdec_video_decoder_get_height (vp6->image);
   dec->plane[0] = vp6->image->plane[0];
   dec->plane[1] = vp6->image->plane[1];
   dec->plane[2] = vp6->image->plane[2];
@@ -126,8 +128,8 @@ swfdec_video_decoder_vp6_alpha_class_init (SwfdecVideoDecoderVp6AlphaClass *klas
 static void
 swfdec_video_decoder_vp6_alpha_init (SwfdecVideoDecoderVp6Alpha *vp6)
 {
-  vp6->image = swfdec_video_decoder_new (SWFDEC_VIDEO_CODEC_VP6);
-  vp6->mask = swfdec_video_decoder_new (SWFDEC_VIDEO_CODEC_VP6);
+  vp6->image = swfdec_video_decoder_new (SWFDEC_VIDEO_CODEC_VP6, NULL);
+  vp6->mask = swfdec_video_decoder_new (SWFDEC_VIDEO_CODEC_VP6, NULL);
 
   if (swfdec_video_decoder_get_error (vp6->image) ||
       swfdec_video_decoder_get_error (vp6->mask)) {
